@@ -18,6 +18,7 @@ SUBROUTINE d3_recover (ilab, isw)
   USE d3com
   USE io_global, ONLY : ionode
   USE mp,        ONLY: mp_bcast
+  USE mp_world,  ONLY: world_comm
   USE io_files,  ONLY : seqopn
   !
   IMPLICIT NONE
@@ -56,8 +57,8 @@ SUBROUTINE d3_recover (ilab, isw)
         !
      END IF
      !
-     CALL mp_bcast (d3dyn, root)
-     CALL mp_bcast (ilab, root)
+     CALL mp_bcast (d3dyn, root, world_comm)
+     CALL mp_bcast (ilab, root, world_comm)
      !
   ENDIF
   RETURN
