@@ -15,11 +15,10 @@ SUBROUTINE clean_pw_ph(iq)
   ! a new q.
   !
   USE kinds,           ONLY : DP
-  USE control_flags,   ONLY : twfcollect
   USE modes,           ONLY : nirr, nsymq
   USE partial,         ONLY : done_irr
   USE disp,            ONLY : done_iq
-  USE control_ph,      ONLY : done_bands, rec_code_read
+  USE control_ph,      ONLY : rec_code_read
   USE save_ph,         ONLY : restore_ph_input_variables
   !
   IMPLICIT NONE
@@ -28,12 +27,10 @@ SUBROUTINE clean_pw_ph(iq)
   !
   INTEGER :: irr
   !
-  done_bands=.FALSE.
-  done_iq(iq)=1
+  done_iq(iq)=.TRUE.
   DO irr=1,nirr
-     IF (done_irr(irr)==0) done_iq(iq)=0
+     IF (.NOT.done_irr(irr)) done_iq(iq)=.FALSE.
   ENDDO
-  twfcollect=.FALSE.
   CALL clean_pw( .FALSE. )
   CALL deallocate_phq()
   rec_code_read=-1000
