@@ -19,11 +19,12 @@ subroutine compute_drhous_nc (drhous, dbecsum, wgg, becq, alpq)
   USE ions_base,  ONLY : nat
   USE lsda_mod,   ONLY : lsda, nspin, current_spin, isk
   USE klist,      ONLY : xk, wk
+  USE buffers,    ONLY : get_buffer
   USE fft_base,   ONLY : dffts, dfftp
   USE fft_interfaces, ONLY : invfft
   USE gvecs,    ONLY : nls
   USE wvfct,      ONLY : npw, npwx, nbnd, igk
-  USE noncollin_module, ONLY : noncolin, npol, nspin_mag
+  USE noncollin_module, ONLY : npol, nspin_mag
   USE wavefunctions_module,  ONLY: evc
   USE io_files,   ONLY: iunigk
   USE uspp,       ONLY: okvan, nkb, vkb
@@ -52,9 +53,7 @@ subroutine compute_drhous_nc (drhous, dbecsum, wgg, becq, alpq)
   real(DP) :: wgg (nbnd, nbnd, nksq)
   ! input: the weights
 
-  integer :: ipert, mode, ik, ikq, ikk, is, ig, nu_i, ibnd, ios
-  ! counter on the pertubations
-  ! counter on the modes
+  integer :: ik, ikq, ikk, ig, nu_i, ibnd, ios
   ! counter on k points
   ! the point k+q
   ! record for wfcs at k point
@@ -103,7 +102,7 @@ subroutine compute_drhous_nc (drhous, dbecsum, wgg, becq, alpq)
      !   Read the wavefunctions at k and transform to real space
      !
 
-     call davcio (evc, lrwfc, iuwfc, ikk, - 1)
+     call get_buffer(evc, lrwfc, iuwfc, ikk)
      evcr = (0.d0, 0.d0)
      do ibnd = 1, nbnd
         do ig = 1, npw
@@ -116,7 +115,7 @@ subroutine compute_drhous_nc (drhous, dbecsum, wgg, becq, alpq)
      !
      !   Read the wavefunctions at k+q
      !
-     if (.not.lgamma.and.nksq.gt.1) call davcio (evq, lrwfc, iuwfc, ikq, -1)
+     if (.not.lgamma.and.nksq.gt.1) call get_buffer(evq, lrwfc, iuwfc, ikq)
      !
      !   And compute the contribution of this k point to the change of
      !   the charge density
