@@ -62,16 +62,25 @@ end Module dynamical
 !                     previous implementation).
 !  axis    integer    indicates the rotation axis for a 1D system
 !                     (1=Ox, 2=Oy, 3=Oz ; default =3)
+!  lperm   logical    .true. to calculate Gamma-point mode contributions to
+!                     dielectric permittivity tensor
+!                     (default: lperm=.false.)
+!  lplasma logical    .true. to calculate Gamma-point mode effective plasma 
+!                     frequencies, automatically triggers lperm = .true. 
+!                     (default: lplasma=.false.)
 !  filout  character output file containing frequencies and modes
 !                    (default: filout='dynmat.out')
+!  fileig  character output file containing frequencies and eigenvectors
+!                    (default: fileig=' ')
 !  filmol  character as above, in a format suitable for 'molden'
 !                    (default: filmol='dynmat.mold')
 !  filxsf  character as above, in axsf format suitable for xcrysden
-!                    (default: filmol='dynmat.axsf')
+!                    (default: filxsf='dynmat.axsf')
 !
       USE kinds, ONLY: DP
       USE mp,         ONLY : mp_bcast
       USE mp_global,  ONLY : mp_startup, mp_global_end
+      USE mp_world,   ONLY : world_comm
       USE io_global,  ONLY : ionode, ionode_id, stdout
       USE environment, ONLY : environment_start, environment_end
       USE io_dyn_mat,  ONLY : read_dyn_mat_param, read_dyn_mat_header, &
@@ -81,7 +90,7 @@ end Module dynamical
       !
       implicit none
       integer, parameter :: ntypx = 10
-      character(len=256):: fildyn, filout, filmol, filxsf
+      character(len=256):: fildyn, filout, filmol, filxsf, fileig
       character(len=3) :: atm(ntypx)
       character(len=10) :: asr
       logical :: lread, gamma
@@ -89,13 +98,14 @@ end Module dynamical
       real(DP) :: amass(ntypx), amass_(ntypx), eps0(3,3), a0, omega, &
            at(3,3), bg(3,3), q(3), q_(3)
       real(DP), allocatable :: w2(:)
-      integer :: nat, na, nt, ntyp, iout, axis, nax, nspin_mag, ios
+      integer :: nat, na, nt, ntyp, iout, axis, nspin_mag, ios
       real(DP) :: celldm(6)
-      logical :: xmldyn, lrigid, lraman
+      logical :: xmldyn, lrigid, lraman, lperm, lplasma
       logical, external :: has_xml
       integer :: ibrav, nqs
       integer, allocatable :: itau(:)
-      namelist /input/ amass, asr, axis, fildyn, filout, filmol, filxsf, q
+      namelist /input/ amass, asr, axis, fildyn, filout, filmol, filxsf, &
+                       fileig, lperm, lplasma, q
       !
       ! code is parallel-compatible but not parallel
       !
@@ -110,24 +120,28 @@ end Module dynamical
       filout='dynmat.out'
       filmol='dynmat.mold'
       filxsf='dynmat.axsf'
+      fileig=' '
       amass(:)=0.0d0
       q(:)=0.0d0
+      lperm=.false.
+      lplasma=.false.
       !
       IF (ionode) read (5,input, iostat=ios)
-      CALL mp_bcast(ios, ionode_id)
+      CALL mp_bcast(ios, ionode_id, world_comm)
       CALL errore('dynmat', 'reading input namelist', ABS(ios))
       !
-      CALL mp_bcast(asr,ionode_id)
-      CALL mp_bcast(axis,ionode_id)
-      CALL mp_bcast(amass,ionode_id)
-      CALL mp_bcast(fildyn,ionode_id)
-      CALL mp_bcast(filout,ionode_id)
-      CALL mp_bcast(filmol,ionode_id)
-      CALL mp_bcast(filxsf,ionode_id)
-      CALL mp_bcast(q,ionode_id)
+      CALL mp_bcast(asr,ionode_id, world_comm)
+      CALL mp_bcast(axis,ionode_id, world_comm)
+      CALL mp_bcast(amass,ionode_id, world_comm)
+      CALL mp_bcast(fildyn,ionode_id, world_comm)
+      CALL mp_bcast(filout,ionode_id, world_comm)
+      CALL mp_bcast(filmol,ionode_id, world_comm)
+      CALL mp_bcast(fileig,ionode_id, world_comm)
+      CALL mp_bcast(filxsf,ionode_id, world_comm)
+      CALL mp_bcast(q,ionode_id, world_comm)
       !
       IF (ionode) inquire(file=fildyn,exist=lread)
-      CALL mp_bcast(lread, ionode_id)
+      CALL mp_bcast(lread, ionode_id, world_comm)
       IF (lread) THEN
          IF (ionode) WRITE(6,'(/5x,a,a)') 'Reading Dynamical Matrix from file '&
                                          , TRIM(fildyn)
@@ -153,21 +167,21 @@ end Module dynamical
          ALLOCATE (dyn(3,3,nat,nat) )
          CALL read_dyn_mat(nat,1,q_,dyn(:,:,:,:))
          CALL read_dyn_mat_tail(nat)
-         if(asr.ne.'no') then
-             call set_asr ( asr, axis, nat, tau, dyn, zstar )
-         endif
+         IF(asr.ne.'no') THEN
+             CALL set_asr ( asr, axis, nat, tau, dyn, zstar )
+         END IF
          IF (ionode) THEN
-            do nt=1, ntyp
-               if (amass(nt) <= 0.0d0) amass(nt)=amass_(nt)
-            end do
+            DO nt=1, ntyp
+               IF (amass(nt) <= 0.0d0) amass(nt)=amass_(nt)
+            END DO
          END IF
       ELSE
          IF (ionode) THEN
-            call readmat ( fildyn, asr, axis, nat, ntyp, atm, a0, &
+            CALL readmat ( fildyn, asr, axis, nat, ntyp, atm, a0, &
                             at, omega, amass_, eps0, q_ )
-            do nt=1, ntyp
-               if (amass(nt) <= 0.0d0) amass(nt)=amass_(nt)/amu_ry
-            end do
+            DO nt=1, ntyp
+               IF (amass(nt) <= 0.0d0) amass(nt)=amass_(nt)/amu_ry
+            END DO
          END IF
       ENDIF
       !
@@ -178,30 +192,39 @@ end Module dynamical
          gamma = ( abs( q_(1)**2+q_(2)**2+q_(3)**2 ) < 1.0d-8 )
          !
          IF (gamma) THEN
-            allocate (itau(nat))
-            do na=1,nat
+            ALLOCATE (itau(nat))
+            DO na=1,nat
                itau(na)=na
-            end do
-            call nonanal ( nat, nat, itau, eps0, q, zstar,omega, dyn )
-            deallocate (itau)
+            END DO
+            CALL nonanal ( nat, nat, itau, eps0, q, zstar, omega, dyn )
+            DEALLOCATE (itau)
          END IF
          !
-         nax = nat
-         allocate ( z(3*nat,3*nat), w2(3*nat) )
-         call dyndiag(nat,ntyp,amass,ityp,dyn,w2,z)
+         ALLOCATE ( z(3*nat,3*nat), w2(3*nat) )
+         CALL dyndiag(nat,ntyp,amass,ityp,dyn,w2,z)
          !
-         if (filout.eq.' ') then
+         IF (filout.eq.' ') then
             iout=6
-         else
+         ELSE
             iout=4
             OPEN (unit=iout,file=filout,status='unknown',form='formatted')
-         end if
-         call writemodes(nax,nat,q_,w2,z,iout)
-         if(iout .ne. 6) close(unit=iout)
-         call writemolden (filmol, gamma, nat, atm, a0, tau, ityp, w2, z)
-         call writexsf (filxsf, gamma, nat, atm, a0, at, tau, ityp, z)
-         IF (gamma) call RamanIR &
-           (nat, omega, w2, z, zstar, eps0, dchi_dtau)
+         END IF
+         CALL writemodes(nat,q_,w2,z,iout)
+         IF(iout .ne. 6) close(unit=iout)
+         IF (fileig .ne. ' ') THEN
+           OPEN (unit=15,file=TRIM(fileig),status='unknown',form='formatted')
+           CALL write_eigenvectors (nat,ntyp,amass,ityp,q_,w2,z,15)
+           CLOSE (unit=15)
+         ENDIF
+         CALL writemolden (filmol, gamma, nat, atm, a0, tau, ityp, w2, z)
+         CALL writexsf (filxsf, gamma, nat, atm, a0, at, tau, ityp, z)
+         IF (gamma) THEN 
+            CALL RamanIR (nat, omega, w2, z, zstar, eps0, dchi_dtau)
+            IF (lperm .OR. lplasma) THEN
+                CALL polar_mode_permittivity(nat,eps0,z,zstar,w2,omega, &
+                                             lplasma)
+            ENDIF
+         ENDIF
       ENDIF
       !
       IF (xmldyn) THEN
@@ -652,7 +675,7 @@ subroutine set_asr ( asr, axis, nat, tau, dyn, zeu )
      !
      zeu_new(:,:,:)=zeu_new(:,:,:) - zeu_w(:,:,:)
      call sp_zeu(zeu_w,zeu_w,nat,norm2)
-     write(6,'(5x,"Acoustic Sum Rule: || Z*(ASR) - Z*(orig)|| = ",E15.6)') &
+     write(6,'(5x,"Acoustic Sum Rule: || Z*(ASR) - Z*(orig)|| = ",ES15.6)') &
           SQRT(norm2)
      !
      ! Check projection
@@ -882,7 +905,7 @@ subroutine set_asr ( asr, axis, nat, tau, dyn, zeu )
      !
      dynr_new(1,:,:,:,:)=dynr_new(1,:,:,:,:) - w(:,:,:,:)
      call sp1(w,w,nat,norm2)
-     write(6,'(5x,"Acoustic Sum Rule: ||dyn(ASR) - dyn(orig)||= ",E15.6)') &
+     write(6,'(5x,"Acoustic Sum Rule: ||dyn(ASR) - dyn(orig)||= ",ES15.6)') &
           DSQRT(norm2)
      !
      ! Check projection
@@ -1033,3 +1056,154 @@ subroutine sp3(u,v,i,na,nat,scal)
   return
   !
 end subroutine sp3
+!
+!----------------------------------------------------------------------
+subroutine polar_mode_permittivity( nat, eps0, z, zstar, w2, omega, lplasma)
+  !----------------------------------------------------------------------
+
+  !
+  ! Algorithm from Fennie and Rabe, Phys. Rev. B 68, 18411 (2003)
+  !
+  USE kinds, ONLY: DP
+  USE constants, ONLY : pi, tpi, fpi, eps4, eps8, eps12, &
+                       ELECTRON_SI, BOHR_RADIUS_SI, AMU_SI, C_SI, &
+                       EPSNOUGHT_SI, AMU_RY, RY_TO_CMM1, RY_TO_THZ
+
+  !number of atoms
+  integer, intent(in) :: nat
+ 
+  !electronic part of the permittivity
+  real(DP), intent(in) :: eps0(3,3)
+ 
+  !displacement eigenvectors
+  complex(DP), intent(in) :: z(3*nat,3*nat)
+ 
+  !born effective charges
+  real(DP), intent(in) :: zstar(3,3,nat)
+ 
+  !square of the phonon frequencies
+  real(DP), intent(in) :: w2(3*nat)
+ 
+  !cell volume
+  real(DP), intent(in) :: omega
+ 
+  !calculate effective plasma frequencies
+  logical, intent(in) :: lplasma
+ 
+  !mode index
+  integer :: imode
+ 
+  !atom index
+  integer :: iat
+ 
+  !atom vector component index
+  integer :: iat_component
+ 
+  !Cartesian direction indices
+  integer :: i, j
+ 
+  !mode effective charge
+  real(DP) :: meffc(3)
+ 
+  !total effective plasma frequency
+  real(DP) :: weff_tot
+ 
+  !polar mode contribution to the permittivity
+  real(DP) :: deps(3,3)
+ 
+  !combined permittivity
+  real(DP) :: eps_new(3,3)
+ 
+  !Conversion factor for plasma frequency from Rydberg atomic units to SI
+  real(DP) :: plasma_frequency_si 
+  !Conversion factor for permittivity from Rydberg atomic units to SI
+  real(DP) :: permittivity_si 
+  ! some compiler do not like SQRT in initialization expressions
+  plasma_frequency_si = ELECTRON_SI/sqrt(EPSNOUGHT_SI*BOHR_RADIUS_SI**3*AMU_SI)
+  permittivity_si = plasma_frequency_si**2 / (fpi * pi)
+
+  IF (lplasma) THEN
+     WRITE(6,*)
+     WRITE(6,'("# mode    freq           Z~*_x         Z~*_y         Z~*_z      &
+               &   W_eff         deps")')
+     WRITE(6,'("#        [cm^-1]                  [e*Bohr/sqrt(2)]              &
+               &  [cm^-1]     [C^2/J*m^2]")')
+  END IF
+  
+  eps_new=eps0
+  
+  !Calculate contributions by mode
+  DO imode = 1,3*nat
+   
+     ! Calculate the mode effective charge
+     meffc = 0.0d0
+     DO i = 1 , 3
+        DO iat = 1 , nat
+           DO j = 1, 3
+              iat_component = 3*(iat-1) + j
+ 
+              ! Equation (3) of Finnie and Rabe
+              ! Rydberg units = (e / sqrt(2)) * Bohr
+              meffc(i) = meffc(i) + zstar(i,j,iat)*z(iat_component,imode)* &
+                                    sqrt(AMU_RY)
+ 
+           END DO
+        END DO
+     END DO
+ 
+     ! Calculate the polar mode contribution to the permittivity
+     deps = 0.0d0
+     ! Use only hard modes (frequency^2 > 10^-8 Ry)
+     IF (w2(imode) > eps8) THEN
+        DO i = 1 , 3
+           DO j = 1 , 3
+    
+              ! Equation (2) of Finnie and Rabe
+              deps(i,j) = (permittivity_si*eps12**2/omega)*meffc(i)*meffc(j) / & 
+                          (w2(imode)*RY_TO_THZ**2)
+    
+           END DO
+        END DO
+     END IF
+    
+     ! Add polar mode contribution to the total permittivity
+     DO i = 1 , 3
+        DO j = 1 , 3
+           eps_new(i,j) = eps_new(i,j) + deps(i,j)
+        END DO
+     END DO
+ 
+     IF (lplasma) THEN
+        ! Calculate the total effective plasma frequency for the mode
+         weff_tot = 0.0d0
+        DO j = 1, 3
+           weff_tot = weff_tot + meffc(j)*meffc(j)
+        END DO
+        ! Rydberg units = (e / sqrt(2)) / (Bohr * sqrt(2*m_e))
+        weff_tot = sqrt(weff_tot/omega)/tpi
+ 
+        !Mode frequency [units of sqrt(Ry)])
+        freq = sqrt(abs(w2(imode)))
+        IF (w2(imode) < 0.0_DP) freq = -freq
+ 
+        !write out mode index, mode effective charges, 
+        !          mode contribution to permittivity, mode plasma frequency
+        WRITE(6,'(i5,6f14.6)') imode,freq*RY_TO_CMM1,meffc(1),meffc(2),meffc(3), &
+                    weff_tot*plasma_frequency_si*eps12*(RY_TO_CMM1 / RY_TO_THZ), &
+               (weff_tot*plasma_frequency_si*eps12)**2/(w2(imode)*RY_TO_THZ**2)
+     END IF
+  END DO
+  
+  WRITE(6,*)
+  WRITE(6,'("Electronic dielectric permittivity tensor (F/m units)")')
+  WRITE(6,'(5x,3f12.6)') eps0(1,:)
+  WRITE(6,'(5x,3f12.6)') eps0(2,:)
+  WRITE(6,'(5x,3f12.6)') eps0(3,:)
+  WRITE(6,*)
+  WRITE(6,'(" ... with zone-center polar mode contributions")')
+  WRITE(6,'(5x,3f12.6)') eps_new(1,:)
+  WRITE(6,'(5x,3f12.6)') eps_new(2,:)
+  WRITE(6,'(5x,3f12.6)') eps_new(3,:)
+  WRITE(6,*)
+ 
+end subroutine polar_mode_permittivity
