@@ -21,28 +21,28 @@ subroutine dynmatrix_new(iq_)
   USE io_global,     ONLY : stdout
   USE control_flags, ONLY : modenum
   USE cell_base,     ONLY : at, bg, celldm, ibrav, omega
-  USE symm_base,     ONLY : s, sr, irt, nsym, time_reversal, invs
-  USE run_info, ONLY : title
+  USE symm_base,     ONLY : s, sr, irt, nsym, invs
   USE dynmat,        ONLY : dyn, w2
   USE qpoint,        ONLY : xq
   USE noncollin_module, ONLY : nspin_mag
   USE modes,         ONLY : u, nmodes, minus_q, irotmq, nsymq, &
-                            rtau, npert, nirr, name_rap_mode, num_rap_mode
+                            rtau, npert, nirr, num_rap_mode
   USE gamma_gamma,   ONLY : nasr, asr, equiv_atoms, has_equivalent, &
                             n_diff_sites
   USE efield_mod,    ONLY : epsilon, zstareu, zstarue0, zstarue
+  USE disp,          ONLY : omega_disp
   USE control_ph,    ONLY : epsil, zue, lgamma, lgamma_gamma, search_sym, ldisp, &
-                            start_irr, last_irr, done_zue, where_rec, &
-                            rec_code, ldiag, done_epsil, done_zeu, xmldyn
+                            done_zue, where_rec, &
+                            rec_code, ldiag, done_epsil, done_zeu, xmldyn, &
+                            current_iq, qplot
   USE ph_restart,    ONLY : ph_writefile
   USE partial,       ONLY : all_comp, comp_irr, done_irr, nat_todo_input
   USE units_ph,      ONLY : iudyn
+  USE control_ph,    ONLY : always_run
   USE noncollin_module, ONLY : m_loc, nspin_mag
-  USE output,        ONLY : fildyn, fildrho, fildvscf
+  USE output,        ONLY : fildyn
   USE io_dyn_mat,    ONLY : write_dyn_mat_header
   USE ramanm,        ONLY : lraman, ramtns
-  USE dfile_star,    ONLY : write_dfile_star, drho_star, dvscf_star !write_dfile_mq
-  USE units_ph,      ONLY : iudrho, iudvscf
 
   implicit none
   INTEGER, INTENT(IN) :: iq_
@@ -58,7 +58,7 @@ subroutine dynmatrix_new(iq_)
   ! list of vectors in the star of q
   real(DP), allocatable :: zstar(:,:,:)
   integer :: icart, jcart, ierr
-  logical :: ldiag_loc, opnd
+  logical :: ldiag_loc
   !
   call start_clock('dynmatrix')
   ldiag_loc=ldiag.OR.(nat_todo_input > 0).OR.all_comp
@@ -70,7 +70,7 @@ subroutine dynmatrix_new(iq_)
      do irr = 1, nirr
         jmode0 = 0
         do jrr = 1, nirr
-           if (done_irr (irr) .eq.0.and.done_irr (jrr) .eq.0) then
+           if (.NOT.done_irr (irr).and..NOT.done_irr (jrr)) then
               do ipert = 1, npert (irr)
                  mu = imode0 + ipert
                  do jpert = 1, npert (jrr)
@@ -78,7 +78,7 @@ subroutine dynmatrix_new(iq_)
                     dyn (mu, nu) = CMPLX(0.d0, 0.d0,kind=DP)
                  enddo
               enddo
-           elseif (done_irr (irr) .eq.0.and.done_irr (jrr) .ne.0) then
+           elseif (.NOT.done_irr (irr) .AND. done_irr (jrr) ) then
               do ipert = 1, npert (irr)
                  mu = imode0 + ipert
                  do jpert = 1, npert (jrr)
@@ -93,7 +93,7 @@ subroutine dynmatrix_new(iq_)
      enddo
   else
      do irr = 1, nirr
-        if (comp_irr(irr)==0) then
+        if (.NOT.comp_irr(irr)) then
            do nu=1,3*nat
               dyn(irr,nu)=(0.d0,0.d0)
            enddo
@@ -125,8 +125,8 @@ subroutine dynmatrix_new(iq_)
 
   IF ( .NOT. ldiag_loc ) THEN
      DO irr=0,nirr
-        IF (done_irr(irr)==0) THEN
-           IF (.not.ldisp) THEN
+        IF (.NOT.done_irr(irr)) THEN
+           IF (.not.ldisp.AND..NOT.always_run) THEN
               WRITE(stdout, '(/,5x,"Stopping because representation", &
                                  & i5, " is not done")') irr
               CALL close_phq(.TRUE.)
@@ -172,7 +172,6 @@ subroutine dynmatrix_new(iq_)
   !
   if (epsil) call write_epsilon_and_zeu (zstareu, epsilon, nat, iudyn)
   IF (zue.AND..NOT.done_zue) THEN
-     done_zue=.TRUE.
      IF (lgamma_gamma) THEN
         ALLOCATE(zstar(3,3,nat))
         zstar(:,:,:) = 0.d0
@@ -200,6 +199,7 @@ subroutine dynmatrix_new(iq_)
               zstarue(:,na,icart)=zstar(:,icart,na)
            ENDDO
         ENDDO
+        done_zue=.TRUE.
         CALL summarize_zue()
         DEALLOCATE(zstar)
      ELSE
@@ -221,7 +221,7 @@ subroutine dynmatrix_new(iq_)
               num_rap_mode, ierr)
          CALL print_mode_sym(w2, num_rap_mode, lgamma)
      ENDIF
-
+     IF (qplot) omega_disp(:,current_iq)=w2(:)
   END IF
 !
 ! Here we save the dynamical matrix and the effective charges dP/du on
@@ -230,7 +230,7 @@ subroutine dynmatrix_new(iq_)
 !
   rec_code=30
   where_rec='dynmatrix.'
-  CALL ph_writefile('data',0)
+  CALL ph_writefile('status_ph',current_iq,0,ierr)
 
   call stop_clock('dynmatrix')
   return
