@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2008 Quantum ESPRESSO group
+! Copyright (C) 2001-2013 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -13,22 +13,29 @@ SUBROUTINE elphon()
   ! Electron-phonon calculation from data saved in fildvscf
   !
   USE kinds, ONLY : DP
-  USE cell_base, ONLY : celldm, omega, ibrav
+  USE constants, ONLY : amu_ry
+  USE cell_base, ONLY : celldm, omega, ibrav, at, bg
   USE ions_base, ONLY : nat, ntyp => nsp, ityp, tau, amass
   USE gvecs, ONLY: doublegrid
   USE fft_base, ONLY : dfftp, dffts
-  USE noncollin_module, ONLY : nspin_mag, noncolin
+  USE noncollin_module, ONLY : nspin_mag, noncolin, m_loc
   USE lsda_mod, ONLY : nspin
   USE phus,       ONLY : int3, int3_nc, int3_paw
   USE uspp,  ONLY: okvan
   USE paw_variables, ONLY : okpaw
+  USE el_phon,  ONLY : done_elph
   USE dynmat, ONLY : dyn, w2
   USE qpoint, ONLY : xq
   USE modes,  ONLY : npert, nirr, u
   USE uspp_param, ONLY : nhm
-  USE control_ph, ONLY : trans
-  USE units_ph, ONLY : iudyn, lrdrho, iudvscf
+  USE control_ph, ONLY : trans, xmldyn
+  USE output,     ONLY : fildyn
+  USE io_dyn_mat, ONLY : read_dyn_mat_param, read_dyn_mat_header, &
+                         read_dyn_mat, read_dyn_mat_tail
+  USE units_ph, ONLY : iudyn, lrdrho, iudvscf, iuint3paw, lint3paw
   USE dfile_star,    ONLY : dvscf_star
+  USE mp_bands,  ONLY : intra_bgrp_comm, me_bgrp, root_bgrp
+  USE mp,        ONLY : mp_bcast
   USE io_global, ONLY: stdout
   !
   IMPLICIT NONE
@@ -39,7 +46,10 @@ SUBROUTINE elphon()
   ! the change of Vscf due to perturbations
   COMPLEX(DP), POINTER :: dvscfin(:,:,:), dvscfins (:,:,:)
 
-
+  INTEGER :: ntyp_, nat_, ibrav_, nspin_mag_, mu, nu, na, nb, nta, ntb, nqs_
+  REAL(DP) :: celldm_(6)
+  CHARACTER(LEN=3) :: atm(ntyp)
+   
   CALL start_clock ('elphon')
 
   if(dvscf_star%basis.eq.'cartesian') then
@@ -64,7 +74,11 @@ SUBROUTINE elphon()
      DO ipert = 1, npe
         CALL davcio_drho ( dvscfin(1,1,ipert),  lrdrho, iudvscf, &
                            imode0 + ipert,  -1 )
+        IF (okpaw .AND. me_bgrp==0) &
+             CALL davcio( int3_paw(:,:,ipert,:,:), lint3paw, &
+                                          iuint3paw, imode0 + ipert, - 1 )
      END DO
+     IF (okpaw) CALL mp_bcast(int3_paw, root_bgrp, intra_bgrp_comm)
      IF (doublegrid) THEN
         ALLOCATE (dvscfins (dffts%nnr, nspin_mag , npert(irr)) )
         DO is = 1, nspin_mag
@@ -76,7 +90,7 @@ SUBROUTINE elphon()
         dvscfins => dvscfin
      ENDIF
      CALL newdq (dvscfin, npert(irr))
-     CALL elphel (npert (irr), imode0, dvscfins)
+     CALL elphel (irr, npert (irr), imode0, dvscfins)
      !
      imode0 = imode0 + npe
      IF (doublegrid) DEALLOCATE (dvscfins)
@@ -91,8 +105,52 @@ SUBROUTINE elphon()
   ! now read the eigenvalues and eigenvectors of the dynamical matrix
   ! calculated in a previous run
   !
-  IF (.NOT.trans) CALL readmat (iudyn, ibrav, celldm, nat, ntyp, &
-       ityp, omega, amass, tau, xq, w2, dyn)
+  IF (.NOT.trans) THEN
+     IF (.NOT. xmldyn) THEN
+        CALL readmat (iudyn, ibrav, celldm, nat, ntyp, &
+                      ityp, omega, amass, tau, xq, w2, dyn)
+     ELSE
+        CALL read_dyn_mat_param(fildyn, ntyp_, nat_)
+        IF ( ntyp_ /= ntyp .OR. nat_ /= nat ) &
+           CALL errore('elphon','uncorrect nat or ntyp',1)
+          
+        CALL read_dyn_mat_header(ntyp, nat, ibrav_, nspin_mag_, &
+                 celldm_, at, bg, omega, atm, amass, tau, ityp, &
+                 m_loc, nqs_)
+
+        IF (ibrav_.NE.ibrav .OR. ABS ( celldm_ (1) - celldm (1) ) > 1.0d-5 &
+             .OR. (nspin_mag_ /= nspin_mag ) ) CALL errore ('elphon', &
+             'inconsistent data', 1)
+
+        CALL read_dyn_mat(nat,1,xq,dyn)
+        !
+        !  Diagonalize the dynamical matrix
+        !
+        DO mu = 1, 3*nat
+           na = (mu - 1) / 3 + 1
+           nta = ityp (na)
+           DO nu = 1, 3*nat
+              nb = (nu - 1) / 3 + 1
+              ntb = ityp (nb)
+              dyn (mu, nu) = dyn (mu, nu) / &
+                             sqrt (amass (nta)*amass (ntb)) / amu_ry
+           ENDDO
+        ENDDO
+        !
+        CALL cdiagh (3 * nat, dyn, 3 * nat, w2, dyn)
+        !
+        ! divide by sqrt(mass) to get displacements
+        !
+        DO nu = 1, 3 * nat
+           DO mu = 1, 3 * nat
+              na = (mu - 1) / 3 + 1
+              dyn (mu, nu) = dyn (mu, nu) / SQRT ( amu_ry * amass (ityp (na) ) )
+           ENDDO
+        ENDDO
+
+        CALL read_dyn_mat_tail(nat)
+     ENDIF
+  ENDIF
   !
   CALL stop_clock ('elphon')
   RETURN
@@ -190,7 +248,7 @@ SUBROUTINE readmat (iudyn, ibrav, celldm, nat, ntyp, ityp, omega, &
 END SUBROUTINE readmat
 !
 !-----------------------------------------------------------------------
-SUBROUTINE elphel (npe, imode0, dvscfins)
+SUBROUTINE elphel (irr, npe, imode0, dvscfins)
   !-----------------------------------------------------------------------
   !
   !      Calculation of the electron-phonon matrix elements el_ph_mat
@@ -198,35 +256,58 @@ SUBROUTINE elphel (npe, imode0, dvscfins)
   !      Original routine written by Francesco Mauri
   !
   USE kinds, ONLY : DP
-  USE fft_base, ONLY : dffts
+  USE fft_base, ONLY : dffts, tg_cgather
   USE wavefunctions_module,  ONLY: evc
   USE io_files, ONLY: iunigk
+  USE buffers,  ONLY : get_buffer
   USE klist, ONLY: xk
   USE lsda_mod, ONLY: lsda, current_spin, isk
   USE noncollin_module, ONLY : noncolin, npol, nspin_mag
   USE wvfct, ONLY: nbnd, npw, npwx, igk
+  USE buffers, ONLY : get_buffer
   USE uspp, ONLY : vkb
-  USE el_phon, ONLY : el_ph_mat
+  USE el_phon, ONLY : el_ph_mat, el_ph_mat_rec, el_ph_mat_rec_col, &
+                      comp_elph, done_elph
   USE modes, ONLY : u
   USE units_ph, ONLY : iubar, lrbar, lrwfc, iuwfc
   USE eqv,      ONLY : dvpsi, evq
-  USE qpoint,   ONLY : igkq, npwq, nksq, ikks, ikqs
-  USE control_ph, ONLY : trans, lgamma
-  USE mp_global, ONLY: intra_pool_comm
+  USE qpoint,   ONLY : igkq, npwq, nksq, ikks, ikqs, nksqtot
+  USE control_ph, ONLY : trans, lgamma, current_iq
+  USE ph_restart, ONLY : ph_writefile
+  USE spin_orb,   ONLY : domag
+  USE mp_bands,   ONLY: intra_bgrp_comm, ntask_groups
+  USE mp_pools,   ONLY: npool
   USE mp,        ONLY: mp_sum
 
   IMPLICIT NONE
   !
-  INTEGER :: npe, imode0
-  COMPLEX(DP) :: dvscfins (dffts%nnr, nspin_mag, npe)
+  INTEGER, INTENT(IN) :: irr, npe, imode0
+  COMPLEX(DP), INTENT(IN) :: dvscfins (dffts%nnr, nspin_mag, npe)
   ! LOCAL variables
   INTEGER :: nrec, ik, ikk, ikq, ipert, mode, ibnd, jbnd, ir, ig, &
-       ios
-  COMPLEX(DP) , ALLOCATABLE :: aux1 (:,:), elphmat (:,:,:)
+       ipol, ios, ierr
+  COMPLEX(DP) , ALLOCATABLE :: aux1 (:,:), elphmat (:,:,:), tg_dv(:,:), &
+       tg_psic(:,:), aux2(:,:)
+  INTEGER :: v_siz, incr
   COMPLEX(DP), EXTERNAL :: zdotc
   !
+  IF (.NOT. comp_elph(irr) .OR. done_elph(irr)) RETURN
+  IF ( ntask_groups > 1 ) dffts%have_task_groups=.TRUE.
+
   ALLOCATE (aux1    (dffts%nnr, npol))
   ALLOCATE (elphmat ( nbnd , nbnd , npe))
+  ALLOCATE( el_ph_mat_rec (nbnd,nbnd,nksq,npe) )
+  el_ph_mat_rec=(0.0_DP,0.0_DP)
+  ALLOCATE (aux2(npwx*npol, nbnd))
+  incr=1
+  IF ( dffts%have_task_groups ) THEN
+     !
+     v_siz =  dffts%tg_nnr * dffts%nogrp
+     ALLOCATE( tg_dv   ( v_siz, nspin_mag ) )
+     ALLOCATE( tg_psic( v_siz, npol ) )
+     incr = dffts%nogrp
+     !
+  ENDIF
   !
   !  Start the loops over the k-points
   !
@@ -257,10 +338,10 @@ SUBROUTINE elphel (npe, imode0, dvscfins)
      !
      IF (nksq.GT.1) THEN
         IF (lgamma) THEN
-           CALL davcio (evc, lrwfc, iuwfc, ikk, - 1)
+           CALL get_buffer(evc, lrwfc, iuwfc, ikk)
         ELSE
-           CALL davcio (evc, lrwfc, iuwfc, ikk, - 1)
-           CALL davcio (evq, lrwfc, iuwfc, ikq, - 1)
+           CALL get_buffer (evc, lrwfc, iuwfc, ikk)
+           CALL get_buffer (evq, lrwfc, iuwfc, ikq)
         ENDIF
      ENDIF
      !
@@ -270,7 +351,7 @@ SUBROUTINE elphel (npe, imode0, dvscfins)
         !  dvbare_q*psi_kpoint is read from file (if available) or recalculated
         !
         IF (trans) THEN
-           CALL davcio (dvpsi, lrbar, iubar, nrec, - 1)
+           CALL get_buffer (dvpsi, lrbar, iubar, nrec)
         ELSE
            mode = imode0 + ipert
            ! TODO : .false. or .true. ???
@@ -279,13 +360,37 @@ SUBROUTINE elphel (npe, imode0, dvscfins)
         !
         ! calculate dvscf_q*psi_k
         !
-        DO ibnd = 1, nbnd
-           CALL cft_wave (evc(1, ibnd), aux1, +1)
-           CALL apply_dpot(dffts%nnr, aux1, dvscfins(1,1,ipert), current_spin)
-           CALL cft_wave (dvpsi(1, ibnd), aux1, -1)
-        END DO
-        CALL adddvscf (ipert, ik)
+        IF ( ntask_groups > 1 ) dffts%have_task_groups=.TRUE.
+        IF ( dffts%have_task_groups ) THEN
+           IF (noncolin) THEN
+              CALL tg_cgather( dffts, dvscfins(:,1,ipert), tg_dv(:,1))
+              IF (domag) THEN
+                 DO ipol=2,4
+                    CALL tg_cgather( dffts, dvscfins(:,ipol,ipert), &
+                                                          tg_dv(:,ipol))
+                 ENDDO
+              ENDIF
+           ELSE
+              CALL tg_cgather( dffts, dvscfins(:,current_spin,ipert), &
+                                                            tg_dv(:,1))
+           ENDIF
+        ENDIF
+        aux2=(0.0_DP,0.0_DP)
+        DO ibnd = 1, nbnd, incr
+           IF ( dffts%have_task_groups ) THEN
+              CALL cft_wave_tg (evc, tg_psic, 1, v_siz, ibnd, nbnd )
+              CALL apply_dpot(v_siz, tg_psic, tg_dv, 1)
+              CALL cft_wave_tg (aux2, tg_psic, -1, v_siz, ibnd, nbnd)
+           ELSE
+              CALL cft_wave (evc(1, ibnd), aux1, +1)
+              CALL apply_dpot(dffts%nnr, aux1, dvscfins(1,1,ipert), current_spin)
+              CALL cft_wave (aux2(1, ibnd), aux1, -1)
+           ENDIF
+        ENDDO
+        dvpsi=dvpsi+aux2
+        dffts%have_task_groups=.FALSE.
 
+        CALL adddvscf (ipert, ik)
         !
         ! calculate elphmat(j,i)=<psi_{k+q,j}|dvscf_q*psi_{k,i}> for this pertur
         !
@@ -300,7 +405,7 @@ SUBROUTINE elphel (npe, imode0, dvscfins)
         ENDDO
      ENDDO
      !
-     CALL mp_sum (elphmat, intra_pool_comm)
+     CALL mp_sum (elphmat, intra_bgrp_comm)
      !
      !  save all e-ph matrix elements into el_ph_mat
      !
@@ -308,13 +413,32 @@ SUBROUTINE elphel (npe, imode0, dvscfins)
         DO jbnd = 1, nbnd
            DO ibnd = 1, nbnd
               el_ph_mat (ibnd, jbnd, ik, ipert + imode0) = elphmat (ibnd, jbnd, ipert)
+              el_ph_mat_rec (ibnd, jbnd, ik, ipert ) = elphmat (ibnd, jbnd, ipert)
            ENDDO
         ENDDO
      ENDDO
   ENDDO
   !
+  done_elph(irr)=.TRUE.
+  IF (npool>1) THEN
+     ALLOCATE(el_ph_mat_rec_col(nbnd,nbnd,nksqtot,npe))
+     CALL el_ph_collect(npe,el_ph_mat_rec,el_ph_mat_rec_col,nksqtot,nksq)
+   ELSE
+     el_ph_mat_rec_col => el_ph_mat_rec
+  ENDIF
+  CALL ph_writefile('el_phon',current_iq,irr,ierr)
+  IF (npool > 1) DEALLOCATE(el_ph_mat_rec_col)
+  DEALLOCATE(el_ph_mat_rec)
+  !
   DEALLOCATE (elphmat)
   DEALLOCATE (aux1)
+  DEALLOCATE (aux2)
+  IF ( ntask_groups > 1) dffts%have_task_groups=.TRUE.
+  IF ( dffts%have_task_groups ) THEN
+     DEALLOCATE( tg_dv )
+     DEALLOCATE( tg_psic )
+  ENDIF
+  dffts%have_task_groups=.FALSE.
   !
   RETURN
 END SUBROUTINE elphel
@@ -328,7 +452,7 @@ SUBROUTINE elphsum ( )
   !      New version by  Malgorzata Wierzbowska
   !
   USE kinds,       ONLY : DP
-  USE constants,   ONLY : pi, rytoev, degspin
+  USE constants,   ONLY : pi, rytoev, ry_to_cmm1, ry_to_ghz, degspin
   USE ions_base,   ONLY : nat, ityp, tau
   USE cell_base,   ONLY : at, bg
   USE lsda_mod,    ONLY: isk, nspin
@@ -338,21 +462,24 @@ SUBROUTINE elphsum ( )
   USE noncollin_module, ONLY: nspin_lsda, nspin_mag
   USE wvfct,       ONLY: nbnd, et
   USE parameters,  ONLY : npk
-  USE el_phon,     ONLY : el_ph_mat
+  USE el_phon,     ONLY : el_ph_mat, done_elph, el_ph_nsigma, el_ph_ngauss, &
+                          el_ph_sigma
   USE qpoint,      ONLY : xq, nksq
-  USE modes,       ONLY : u, minus_q, nsymq, rtau
+  USE modes,       ONLY : u, minus_q, nsymq, rtau, nirr
   USE dynmat,      ONLY : dyn, w2
   USE io_global,   ONLY : stdout, ionode, ionode_id
-  USE mp_global,   ONLY : my_pool_id, npool, kunit, intra_image_comm
+  USE xml_io_base, ONLY : create_directory
+  USE mp_pools,    ONLY : my_pool_id, npool, kunit
+  USE mp_images,   ONLY : intra_image_comm
   USE mp,          ONLY : mp_bcast
-  USE control_ph,  ONLY : lgamma, tmp_dir_phq, xmldyn
+  USE control_ph,  ONLY : lgamma, tmp_dir_phq, xmldyn, current_iq
   USE save_ph,     ONLY : tmp_dir_save
   USE io_files,    ONLY : prefix, tmp_dir, seqopn
   !
   IMPLICIT NONE
   ! epsw = 20 cm^-1, in Ry
-  REAL(DP), PARAMETER :: Rytocm1 = 109737.57990d0, RytoGHz = 3.289828D6, &
-       RytoTHz = RytoGHz/1000.d0, epsw = 20.d0 / Rytocm1, eps = 1.0d-6
+  REAL(DP), PARAMETER :: epsw = 20.d0 / ry_to_cmm1
+  REAL(DP), PARAMETER :: eps = 1.0d-6
   !
   INTEGER :: iuna2Fsave  = 40
   !
@@ -385,7 +512,8 @@ SUBROUTINE elphsum ( )
   INTEGER, ALLOCATABLE :: eqBZ(:), sBZ(:)
   REAL(DP) :: weight, wqa, w0g1, w0g2, degauss1, dosef, &
        ef1, lambda, gamma
-  REAL(DP) :: deg(10), effit(10), dosfit(10), etk, etq
+  REAL(DP), ALLOCATABLE :: deg(:), effit(:), dosfit(:)
+  REAL(DP) :: etk, etq
   REAL(DP), EXTERNAL :: dos_ef, efermig, w0gauss
   character(len=80) :: name
   LOGICAL  :: exst, xmldyn_save
@@ -396,18 +524,36 @@ SUBROUTINE elphsum ( )
   REAL(DP), ALLOCATABLE :: xk_collect(:,:), wk_collect(:)
   REAL(DP), POINTER :: wkfit_dist(:), etfit_dist(:,:)
   INTEGER :: nksfit_dist, rest, kunit_save
-  INTEGER :: nks_real, ispin, nksqtot
+  INTEGER :: nks_real, ispin, nksqtot, irr
+  CHARACTER(LEN=256) :: elph_dir
+  CHARACTER(LEN=6) :: int_to_char
   !
   !
+  !
+  !  If the electron phonon matrix elements have not been calculated for
+  !  all representations this routine exit
+  !
+  DO irr=1,nirr
+     IF (.NOT.done_elph(irr)) RETURN
+  ENDDO
+  elph_dir='elph_dir/'
+  IF (ionode) INQUIRE(file=TRIM(elph_dir), EXIST=exst)
+  CALL mp_bcast(exst, ionode_id, intra_image_comm) 
+  IF (.NOT.exst) CALL create_directory( elph_dir )
   WRITE (6, '(5x,"electron-phonon interaction  ..."/)')
   ngauss1 = 0
-  nsig = 10
+  nsig =el_ph_nsigma
 
   ALLOCATE(xk_collect(3,nkstot))
   ALLOCATE(wk_collect(nkstot))
+
+  ALLOCATE(deg(nsig))
+  ALLOCATE(effit(nsig))
+  ALLOCATE(dosfit(nsig))
+
   IF (npool==1) THEN
 !
-!  no pool, just copy old variable on the new ones
+!  no pool, just copy old variables on the new ones
 !
      nksqtot=nksq
      xk_collect(:,1:nks) = xk(:,1:nks)
@@ -425,7 +571,7 @@ SUBROUTINE elphsum ( )
      ENDIF
      ALLOCATE(el_ph_mat_collect(nbnd,nbnd,nksqtot,3*nat))
      CALL xk_wk_collect(xk_collect,wk_collect,xk,wk,nkstot,nks)
-     CALL el_ph_collect(el_ph_mat,el_ph_mat_collect,nksqtot,nksq)
+     CALL el_ph_collect(3*nat,el_ph_mat,el_ph_mat_collect,nksqtot,nksq)
   ENDIF
   !
   ! read eigenvalues for the dense grid
@@ -488,7 +634,7 @@ SUBROUTINE elphsum ( )
      ! recalculate Ef = effit and DOS at Ef N(Ef) = dosfit using dense grid
      ! for value "deg" of gaussian broadening
      !
-     deg(isig) = isig * 0.005d0
+     deg(isig) = isig * el_ph_sigma
      !
      effit(isig) = efermig &
           ( etfit_dist, nbnd, nksfit_dist, nelec, wkfit_dist, &
@@ -675,7 +821,7 @@ SUBROUTINE elphsum ( )
         else
            lamb(nu,isig) = 0.0d0
         endif
-        gam(nu,isig) = gam(nu,isig)*RytoGHz
+        gam(nu,isig) = gam(nu,isig)*ry_to_ghz
      enddo  !nu
   enddo  ! isig
   !
@@ -687,21 +833,27 @@ SUBROUTINE elphsum ( )
      enddo
   enddo
   ! Isaev: save files in suitable format for processing by lambda.x
-  write(name,'(A5,f9.6,A1,f9.6,A1,f9.6)') 'elph.',xq(1),'.',xq(2),'.',xq(3)
-  open(12,file=name, form='formatted', status='unknown')
+   name=TRIM(elph_dir)// 'elph.inp_lambda.' //TRIM(int_to_char(current_iq))
+                                             
+  IF (ionode) THEN
+     open(unit=12, file=TRIM(name), form='formatted', status='unknown', &
+                                    iostat=ios)
 
-  write(12, "(5x,3f14.6,2i6)") xq(1),xq(2),xq(3), nsig, 3*nat
-  write(12, "(6e14.6)") (w2(i), i=1,3*nat)
+     write(12, "(5x,3f14.6,2i6)") xq(1),xq(2),xq(3), nsig, 3*nat
+     write(12, "(6e14.6)") (w2(i), i=1,3*nat)
 
-  do isig= 1,nsig
-     WRITE (12, 9000) deg(isig), ngauss1
-     WRITE (12, 9005) dosfit(isig), effit(isig) * rytoev
-     do nu=1,3*nat
-        WRITE (12, 9010) nu, lamb(nu,isig), gam(nu,isig)
+     do isig= 1,nsig
+        WRITE (12, 9000) deg(isig), ngauss1
+        WRITE (12, 9005) dosfit(isig), effit(isig) * rytoev
+        do nu=1,3*nat
+           WRITE (12, 9010) nu, lamb(nu,isig), gam(nu,isig)
+        enddo
      enddo
-  enddo
-  close (unit=12,status='keep')
+     close (unit=12,status='keep')
+  ENDIF
   ! Isaev end
+  CALL mp_bcast(ios, ionode_id, intra_image_comm)
+  IF (ios /= 0) CALL errore('elphsum','problem opening file'//TRIM(name),1)
   deallocate (gam)
   deallocate (lamb)
   write(stdout,*)
@@ -711,17 +863,20 @@ SUBROUTINE elphsum ( )
   call star_q (xq, at, bg, nsym, s, invs, nq, sxq, isq, imq, .TRUE. )
   !
   do isig=1,nsig
-     write(name,"(A7,I2)") 'a2Fq2r.',50 + isig
+     name=TRIM(elph_dir)//'a2Fq2r.'// TRIM(int_to_char(50 + isig)) &
+                                  //'.'//TRIM(int_to_char(current_iq))
      if (ionode) then
         iuelph = 4
         open(iuelph, file=name, STATUS = 'unknown', FORM = 'formatted', &
-             POSITION='append')
+                     iostat=ios)
      else
         !
         ! this node doesn't write: unit 6 is redirected to /dev/null
         !
         iuelph =6
      end if
+     call mp_bcast(ios, ionode_id, intra_image_comm)
+     IF (ios /= 0) call errore('elphsum','opening output file '// TRIM(name),1)
      dyn22(:,:) = gf(:,:,isig)
      write(iuelph,*) deg(isig), effit(isig), dosfit(isig)
      IF ( imq == 0 ) THEN
@@ -737,6 +892,9 @@ SUBROUTINE elphsum ( )
      if (ionode) CLOSE( UNIT = iuelph, STATUS = 'KEEP' )
   enddo
   deallocate (gf)
+  DEALLOCATE( deg )
+  DEALLOCATE( effit )
+  DEALLOCATE( dosfit )
   DEALLOCATE(xk_collect)
   DEALLOCATE(wk_collect)
   IF (npool /= 1) DEALLOCATE(el_ph_mat_collect)
@@ -746,7 +904,7 @@ SUBROUTINE elphsum ( )
 9005 FORMAT(5x,'DOS =',f10.6,' states/spin/Ry/Unit Cell at Ef=', &
           &       f10.6,' eV')
 9006 FORMAT(5x,'double delta at Ef =',f10.6)
-9010 FORMAT(5x,'lambda(',i2,')=',f8.4,'   gamma=',f8.2,' GHz')
+9010 FORMAT(5x,'lambda(',i5,')=',f8.4,'   gamma=',f8.2,' GHz')
   !
   RETURN
 END SUBROUTINE elphsum
@@ -760,82 +918,76 @@ SUBROUTINE elphsum_simple
   !      Rewritten by Matteo Calandra
   !-----------------------------------------------------------------------
   USE kinds, ONLY : DP
-  USE constants, ONLY : pi, ry_to_cmm1, rytoev
-  USE ions_base, ONLY : nat, ityp, tau,amass,tau, ntyp => nsp, atm
-  USE cell_base, ONLY : at, bg, ibrav, celldm 
-  USE fft_base,  ONLY: dfftp
-  USE symm_base, ONLY : s, sr, irt, nsym, time_reversal, invs
+  USE constants, ONLY : pi, ry_to_cmm1, ry_to_ghz, rytoev
+  USE ions_base, ONLY : nat
+  USE cell_base, ONLY : at, bg
+  USE symm_base, ONLY : s, irt, nsym, invs
   USE klist, ONLY : xk, nelec, nks, wk
   USE wvfct, ONLY : nbnd, et
   USE el_phon, ONLY : el_ph_mat, el_ph_nsigma, el_ph_ngauss, el_ph_sigma
-  USE mp_global, ONLY : me_pool, root_pool, inter_pool_comm, npool, intra_pool_comm
-  USE io_global, ONLY : stdout
-  USE klist, only : degauss,ngauss
-  USE control_flags, ONLY : modenum, noinv
-  USE units_ph,       ONLY :iudyn
-  USE io_files,  ONLY : prefix
-  USE qpoint, ONLY : xq, nksq
+  USE mp_pools,  ONLY : inter_pool_comm, npool
+  USE mp_images, ONLY : intra_image_comm
+  USE qpoint, ONLY : xq, nksq, ikks, ikqs
+  USE output, ONLY : fildyn
   USE dynmat, ONLY : dyn, w2
-  USE modes, ONLY : u,rtau, nsymq,irotmq, minus_q
-  USE control_ph, only : lgamma
-  USE lsda_mod, only : isk,nspin, current_spin,lsda
-  USE mp,        ONLY: mp_sum
+  USE modes, ONLY : u, rtau, nsymq, irotmq, minus_q, nirr
+  USE control_ph, only : current_iq, qplot
+  USE lsda_mod, only : isk
+  USE el_phon,   ONLY : done_elph, gamma_disp
+  USE io_global, ONLY : stdout, ionode, ionode_id
+  USE mp,        ONLY: mp_sum, mp_bcast
   !
   IMPLICIT NONE
   REAL(DP), PARAMETER :: eps = 20_dp/ry_to_cmm1 ! eps = 20 cm^-1, in Ry
   !
   INTEGER :: ik, ikk, ikq, isig, ibnd, jbnd, ipert, jpert, nu, mu, &
-       vu, ngauss1, nsig, iuelph, ios, iuelphmat,icnt,i,j,rrho,nt,k
-  INTEGER :: na,nb,icar,jcar,iu_sym,nmodes
-  INTEGER :: iu_Delta_dyn,iu_analdyn,iu_nonanaldyn
-  INTEGER :: io_file_unit
-  !   for star_q
-  INTEGER :: nsymloc, sloc(3,3,48), invsloc(48), irtloc(48,nat), &
-             nqloc, isqloc(48), imqloc
-  REAL(DP) :: rtauloc(3,48,nat), sxqloc(3,48)
-  !   end of star_q definitions
-  REAL(DP) :: weight, w0g1, w0g2, w0gauss, wgauss,degauss1, dosef, &
-       ef1, phase_space, lambda, gamma, wg1, w0g,wgp,deltae
+       vu, ngauss1, nsig, iuelph, ios, irr
+  INTEGER :: nmodes
+  REAL(DP) :: weight, w0g1, w0g2, w0gauss, wgauss, degauss1, dosef, &
+       ef1, phase_space, lambda, gamma
   REAL(DP), EXTERNAL :: dos_ef, efermig
-  REAL(DP) xk_dummy(3)
-  COMPLEX(DP), allocatable :: phi(:,:,:,:),phi_nonanal(:,:,:,:)
-  COMPLEX(DP), allocatable :: dyn_mat_r(:,:),zz(:,:)
-  CHARACTER(len=20) :: char_deg
-  CHARACTER(len=1) :: char_ng
   character(len=80) :: filelph
   CHARACTER(len=256) ::  file_elphmat
   !
   COMPLEX(DP) :: el_ph_sum (3*nat,3*nat), dyn_corr(3*nat,3*nat)
 
   INTEGER, EXTERNAL :: find_free_unit
+  CHARACTER(LEN=6) :: int_to_char
+
+
+  DO irr=1,nirr
+     IF (.NOT.done_elph(irr)) RETURN
+  ENDDO
 
   nmodes=3*nat
 
-
-  write(filelph,'(A5,f9.6,A1,f9.6,A1,f9.6)') 'elph.',xq(1),'.',xq(2),'.',xq(3)
+  filelph=TRIM(fildyn)//'.elph.'//TRIM(int_to_char(current_iq))
 
   ! parallel case: only first node writes
-  IF ( me_pool /= root_pool ) THEN
-     iuelph = 0
-  ELSE
+  IF ( ionode ) THEN
      !
      iuelph = find_free_unit()
-     OPEN (unit = iuelph, file = filelph, status = 'unknown', err = &
+     OPEN (unit = iuelph, file = TRIM(filelph), status = 'unknown', err = &
           100, iostat = ios)
-100  CALL errore ('elphon', 'opening file '//filelph, ABS (ios) )
      REWIND (iuelph)
+  ELSE
+     iuelph = 0
      !
   END IF
+100 CONTINUE
+  CALL mp_bcast(ios,ionode_id,intra_image_comm)
+  CALL errore ('elphsum_simple', 'opening file '//filelph, ABS (ios) )
 
-  WRITE (iuelph, '(3f15.8,2i8)') xq, nsig, 3 * nat
-  WRITE (iuelph, '(6e14.6)') (w2 (nu) , nu = 1, nmodes)
+  IF (ionode) THEN
+     WRITE (iuelph, '(3f15.8,2i8)') xq, nsig, 3 * nat
+     WRITE (iuelph, '(6e14.6)') (w2 (nu) , nu = 1, nmodes)
+  ENDIF
   
 
   ngauss1=0
   DO isig = 1, el_ph_nsigma
      !     degauss1 = 0.01 * isig
      degauss1 = el_ph_sigma * isig
-     write(stdout,*) degauss1
      el_ph_sum(:,:) = (0.d0, 0.d0)
      phase_space = 0.d0
      !
@@ -859,18 +1011,11 @@ SUBROUTINE elphsum_simple
         !
         ! see subroutine elphel for the logic of indices
         !
-        IF (lgamma) THEN
-           ikk = ik
-           ikq = ik
-        ELSE
-           ikk = 2 * ik - 1
-           ikq = ikk + 1
-        ENDIF
+        ikk = ikks(ik)
+        ikq = ikqs(ik)
         DO ibnd = 1, nbnd
            w0g1 = w0gauss ( (ef1 - et (ibnd, ikk) ) / degauss1, ngauss1) &
                 / degauss1
-           xk_dummy(:)=xk(:,ikk)
-           call cryst_to_cart(1,xk_dummy,at,-1)
            DO jbnd = 1, nbnd
               w0g2 = w0gauss ( (ef1 - et (jbnd, ikq) ) / degauss1, ngauss1) &
                    / degauss1
@@ -895,9 +1040,7 @@ SUBROUTINE elphsum_simple
      !
      ! collect contributions from all pools (sum over k-points)
      !
-     
-!     CALL poolreduce (2 * 3 * nat * 3 * nat, el_ph_sum)
-!     CALL poolreduce (1, phase_space)
+
      call mp_sum ( el_ph_sum , inter_pool_comm )
      call mp_sum ( phase_space , inter_pool_comm )
 
@@ -908,25 +1051,24 @@ SUBROUTINE elphsum_simple
      CALL symdyn_munu_new (el_ph_sum, u, xq, s, invs, rtau, irt,  at, &
           bg, nsymq, nat, irotmq, minus_q)
      !
-     WRITE (6, 9000) degauss1, ngauss1
-     WRITE (6, 9005) dosef, ef1 * rytoev
-     WRITE (6, 9006) phase_space
-     IF (iuelph.NE.0) THEN
+     WRITE (stdout, *)
+     WRITE (stdout, 9000) degauss1, ngauss1
+     WRITE (stdout, 9005) dosef, ef1 * rytoev
+     WRITE (stdout, 9006) phase_space
+     IF (ionode) THEN
         WRITE (iuelph, 9000) degauss1, ngauss1
         WRITE (iuelph, 9005) dosef, ef1 * rytoev
      ENDIF
      
      DO nu = 1, nmodes
-        gamma = 0.0
+        gamma = 0.d0
         DO mu = 1, 3 * nat
            DO vu = 1, 3 * nat
               gamma = gamma + DBLE (CONJG (dyn (mu, nu) ) * el_ph_sum (mu, vu)&
                    * dyn (vu, nu) )
            ENDDO
         ENDDO
-        write(819+mu,*) gamma
         gamma = pi * gamma / 2.d0
-        write(6,*) 'gamma*pi/2=',gamma
         !
         ! the factor 2 comes from the factor sqrt(hbar/2/M/omega) that appears
         ! in the definition of the electron-phonon matrix element g
@@ -946,12 +1088,11 @@ SUBROUTINE elphsum_simple
            ! lambda(nu)= gamma(nu)/(pi N(Ef) \omega_{q,nu}^2)
            lambda = gamma / pi / w2 (nu) / dosef
         ELSE
-           lambda = 0.0
+           lambda = 0.d0
         ENDIF
-        ! 3.289828x10^6 is the conversion factor from Ry to GHz
-        WRITE (6, 9010) nu, lambda, gamma * 3.289828d6
-        IF (iuelph.NE.0) WRITE (iuelph, 9010) nu, lambda, gamma * &
-             3.289828d6
+        WRITE (stdout, 9010) nu, lambda, gamma * ry_to_gHz
+        IF (ionode) WRITE (iuelph, 9010) nu, lambda, gamma * ry_to_gHz
+        IF (qplot) gamma_disp(nu,isig,current_iq) = gamma * ry_to_gHz
      ENDDO
   ENDDO
   
@@ -960,10 +1101,10 @@ SUBROUTINE elphsum_simple
 9005 FORMAT(5x,'DOS =',f10.6,' states/spin/Ry/Unit Cell at Ef=', &
           &       f10.6,' eV')
 9006 FORMAT(5x,'double delta at Ef =',f10.6)
-9010 FORMAT(5x,'lambda(',i2,')=',f8.4,'   gamma=',f8.2,' GHz')
+9010 FORMAT(5x,'lambda(',i5,')=',f8.4,'   gamma=',f8.2,' GHz')
   !
   !
-  IF (iuelph.NE.0) CLOSE (unit = iuelph)
+  IF (ionode) CLOSE (unit = iuelph)
   RETURN
   
 
@@ -974,15 +1115,12 @@ SUBROUTINE elphsum_simple
 
 END SUBROUTINE elphsum_simple
    
-
-
-
 !-----------------------------------------------------------------------
 FUNCTION dos_ef (ngauss, degauss, ef, et, wk, nks, nbnd)
   !-----------------------------------------------------------------------
   !
   USE kinds, ONLY : DP
-  USE mp_global, ONLY : inter_pool_comm, intra_pool_comm
+  USE mp_pools, ONLY : inter_pool_comm
   USE mp,        ONLY : mp_sum
   IMPLICIT NONE
   REAL(DP) :: dos_ef
