@@ -17,18 +17,24 @@ SUBROUTINE ep_matrix_element_wannier()
   USE ions_base, ONLY : nat, ntyp => nsp, ityp, tau, amass
   USE gvecs, ONLY: doublegrid
   USE fft_base, ONLY : dfftp, dffts
-  USE noncollin_module, ONLY : nspin_mag
+  USE noncollin_module, ONLY : nspin_mag, noncolin
   USE dynmat, ONLY : dyn, w2
   USE qpoint, ONLY : xq, nksq, ikks
   USE modes,  ONLY : npert, nirr
   USE control_ph, ONLY : trans
   USE units_ph, ONLY : iudyn, lrdrho, iudvscf
   USE io_global, ONLY : stdout
-  USE mp_global, ONLY : me_pool, root_pool
+  USE mp_pools,  ONLY : me_pool, root_pool
   USE modes, ONLY : u
   USE klist, ONLY : xk
   USE wvfct, ONLY : npwx
   USE el_phon, ONLY: elph_mat, kpq, g_kpq, igqg, xk_gamma
+  USE phus,       ONLY : int3, int3_nc, int3_paw
+  USE uspp,                 ONLY: okvan
+  USE paw_variables, ONLY : okpaw
+  USE uspp_param, ONLY : nhm
+  USE lsda_mod, ONLY : nspin
+
   !
   IMPLICIT NONE
   !
@@ -87,6 +93,12 @@ SUBROUTINE ep_matrix_element_wannier()
   imode0 = 0
   DO irr = 1, nirr
      ALLOCATE (dvscfin (dfftp%nnr, nspin_mag , npert(irr)) )
+     IF (okvan) THEN
+        ALLOCATE (int3 ( nhm, nhm, npert(irr), nat, nspin_mag))
+        IF (okpaw) ALLOCATE (int3_paw (nhm, nhm, npert(irr), nat, nspin_mag))
+        IF (noncolin) ALLOCATE(int3_nc( nhm, nhm, npert(irr), nat, nspin))
+     ENDIF
+
 !     if(ascii_dvscf) then
 !        DO ipert = 1, npert(irr)
 !           dvscfin(1:dfftp%nnr,:,ipert)=dvrot(1:dfftp%nnr,:,imode0+ipert)
@@ -113,12 +125,22 @@ SUBROUTINE ep_matrix_element_wannier()
      imode0 = imode0 + npert (irr)
      IF (doublegrid) DEALLOCATE (dvscfins)
      DEALLOCATE (dvscfin)
+
+     IF (okvan) THEN
+       DEALLOCATE (int3)
+       IF (okpaw) DEALLOCATE (int3_paw)
+       IF (noncolin) DEALLOCATE(int3_nc)
+     ENDIF
+
   ENDDO
   !
   ! now read the eigenvalues and eigenvectors of the dynamical matrix
   ! calculated in a previous run
   !
-  IF (.NOT.trans) CALL readmat (iudyn, ibrav, celldm, nat, ntyp, &
+!  IF (.NOT.trans) CALL readmat (iudyn, ibrav, celldm, nat, ntyp, &
+!       ityp, omega, amass, tau, xq, w2, dyn)
+
+  IF (.NOT.trans) CALL readmat_findq (iudyn, ibrav, celldm, nat, ntyp, &
        ityp, omega, amass, tau, xq, w2, dyn)
   !
   
@@ -146,10 +168,11 @@ SUBROUTINE elphsum_wannier(q_index)
   USE klist, ONLY : xk, nelec
   USE wvfct, ONLY : nbnd, et
   USE el_phon
-  USE mp_global, ONLY : me_pool, root_pool, inter_pool_comm, npool, intra_pool_comm
+  USE mp_pools,  ONLY : me_pool, root_pool, inter_pool_comm, npool
+  USE mp_bands,  ONLY : intra_bgrp_comm
   USE io_global, ONLY : stdout,ionode
   USE io_files,  ONLY : prefix
-  USE qpoint, ONLY : xq, nksq
+  USE qpoint, ONLY : xq, nksq, ikks, ikqs
   USE dynmat, ONLY : dyn, w2
   USE modes, ONLY : u, gi, gimq, irgq, irotmq
   USE control_ph, only : lgamma
@@ -198,7 +221,7 @@ SUBROUTINE elphsum_wannier(q_index)
      iuelphmat = find_free_unit()
      OPEN (unit = iuelphmat, file = file_elphmat, status = 'unknown', err = &
           111, iostat = ios, form='unformatted')
-111  CALL errore ('elphon', 'opening file'//file_elphmat, ABS (ios) )
+111  CALL errore ('elphsum_wannier', 'opening file'//file_elphmat, ABS (ios) )
      REWIND (iuelphmat)
      xk_dummy(:)=xq(:)
      call cryst_to_cart(1,xk_dummy,at,-1)
@@ -213,13 +236,8 @@ SUBROUTINE elphsum_wannier(q_index)
      WRITE (iuelphmat) ((u(ipert,jpert),ipert=1,nmodes),jpert=1,nmodes)
      WRITE (iuelphmat) ((dyn(ipert,jpert),ipert=1,3*nat),jpert=1,3*nat)
      do ik=1,nksq
-        IF (lgamma) THEN
-           ikk = ik
-           ikq = ik
-        ELSE
-           ikk = 2 * ik - 1
-           ikq = ikk + 1
-        ENDIF
+        ikk=ikks(ik)
+        ikq=ikqs(ik) 
         xk_dummy(:)=xk(:,ikk)
         call cryst_to_cart(1,xk_dummy,at,-1)
         WRITE (iuelphmat) (xk_dummy(ipert),ipert=1,3)
@@ -231,7 +249,8 @@ SUBROUTINE elphsum_wannier(q_index)
                 ibnd=elph_nbnd_min,elph_nbnd_max)
         enddo
      enddo
-  
+
+
 
 
      !
@@ -311,16 +330,18 @@ SUBROUTINE elphel_refolded (npe, imode0, dvscfins)
   USE klist, ONLY: xk
   USE lsda_mod, ONLY: lsda, current_spin, isk
   USE noncollin_module, ONLY : noncolin, npol, nspin_mag
+  USE buffers, ONLY : get_buffer
   USE wvfct, ONLY: nbnd, npw, npwx, igk
   USE uspp, ONLY : vkb
   USE el_phon, ONLY : el_ph_mat, iunwfcwann, igqg, kpq, g_kpq, &
            xk_gamma, npwq_refolded, lrwfcr
   USE modes, ONLY : u
-  USE units_ph, ONLY : iubar, lrbar, lrwfc, iuwfc
+  USE units_ph, ONLY : iubar, lrbar
   USE eqv,      ONLY : dvpsi!, evq
   USE qpoint,   ONLY : igkq, npwq, nksq, ikks, ikqs
   USE control_ph, ONLY : trans, lgamma
-  USE mp_global, ONLY: intra_pool_comm, me_pool, root_pool
+  USE mp_bands,  ONLY: intra_bgrp_comm
+  USE mp_pools,  ONLY: me_pool, root_pool
   USE mp,        ONLY: mp_sum
   USE ions_base, ONLY : nat
   USE io_global, ONLY : stdout
@@ -363,7 +384,7 @@ SUBROUTINE elphel_refolded (npe, imode0, dvscfins)
      
      IF (nksq.GT.1) THEN
         READ (iunigk, err = 100, iostat = ios) npw, igk
-100     CALL errore ('elphel', 'reading igk', ABS (ios) )
+100     CALL errore ('elphel_refolded', 'reading igk', ABS (ios) )
      ENDIF
      !
      !  ik = counter of k-points with vector k
@@ -380,7 +401,7 @@ SUBROUTINE elphel_refolded (npe, imode0, dvscfins)
      IF (lsda) current_spin = isk (ikk)
      IF (.NOT.lgamma.AND.nksq.GT.1) THEN
         READ (iunigk, err = 200, iostat = ios) npwq, igkq
-200     CALL errore ('elphel', 'reading igkq', ABS (ios) )
+200     CALL errore ('elphel_refolded', 'reading igkq', ABS (ios) )
      ENDIF
 
      
@@ -423,7 +444,7 @@ SUBROUTINE elphel_refolded (npe, imode0, dvscfins)
         !  dvbare_q*psi_kpoint is read from file (if available) or recalculated
         !
         IF (trans) THEN
-           CALL davcio (dvpsi, lrbar, iubar, nrec, - 1)
+           CALL get_buffer (dvpsi, lrbar, iubar, nrec)
         ELSE
            mode = imode0 + ipert
            ! TODO : .false. or .true. ???
@@ -453,7 +474,7 @@ SUBROUTINE elphel_refolded (npe, imode0, dvscfins)
         ENDDO
      ENDDO
      !
-     CALL mp_sum (elphmat, intra_pool_comm)
+     CALL mp_sum (elphmat, intra_bgrp_comm)
      !
      !  save all e-ph matrix elements into el_ph_mat
      !
@@ -496,7 +517,7 @@ subroutine get_equivalent_kpq(xk,xq,kpq,g_kpq, igqg)
   USE qpoint, ONLY : nksq, ikks
   USE gvect, ONLY: g, gg
   USE qpoint, ONLY : nksq
-  USE mp_global, ONLY : intra_pool_comm
+  USE mp_bands, ONLY : intra_bgrp_comm
   USE mp, ONLY : mp_sum
   ! WARNING g_kpq mesh is an integer
   implicit none
@@ -591,7 +612,7 @@ subroutine get_equivalent_kpq(xk,xq,kpq,g_kpq, igqg)
 
   allocate(ig_check(nksq))
   ig_check=igqg
-  CALL mp_sum( ig_check, intra_pool_comm )
+  CALL mp_sum( ig_check, intra_bgrp_comm )
   do ik=1,nksq
      if(ig_check(ik).eq.0) &
           CALL errore('get_equivalent_kpq', &
@@ -669,12 +690,12 @@ subroutine calculate_and_apply_phase(ik, ikqg, igqg, npwq_refolded, g_kpq, xk_ga
   phase(:)=conjg(phase(:))
 
 
-  if(npwq_refolded.ne.npw_) call errore('elphel_refolded', 'Warning : npwq_refolded \= npw_',-1)
+  if(npwq_refolded.ne.npw_) call errore('calculate_and_apply_phase', 'Warning : npwq_refolded \= npw_',-1)
   
   do m=1,nbnd
      psi_scratch = (0.d0, 0.d0)
-!     psi_scratch(nls (igk_ (1:npw_) ) ) = evq (1:npw_, m)
-     psi_scratch(nls (igk_ (1:npw) ) ) = evq (1:npw, m)
+     psi_scratch(nls (igk_ (1:npw_) ) ) = evq (1:npw_, m)
+!     psi_scratch(nls (igk_ (1:npw) ) ) = evq (1:npw, m)
      CALL invfft ('Wave', psi_scratch, dffts)
      !     call cft3s (psic, nr1s, nr2s, nr3s, nrx1s, nrx2s, nrx3s, +2)
      psi_scratch(1:dffts%nnr) = psi_scratch(1:dffts%nnr) * phase(1:dffts%nnr)
@@ -691,3 +712,117 @@ subroutine calculate_and_apply_phase(ik, ikqg, igqg, npwq_refolded, g_kpq, xk_ga
   return
 end subroutine calculate_and_apply_phase
   
+
+!-----------------------------------------------------------------------
+SUBROUTINE readmat_findq (iudyn, ibrav, celldm, nat, ntyp, ityp, omega, &
+     amass, tau, q, w2, dyn)
+  !-----------------------------------------------------------------------
+  !
+  USE kinds, ONLY : DP
+  USE constants, ONLY : amu_ry
+  IMPLICIT NONE
+  ! Input
+  INTEGER :: iudyn, ibrav, nat, ntyp, ityp (nat)
+  REAL(DP) :: celldm (6), amass (ntyp), tau (3, nat), q (3), &
+       omega
+  ! output
+  REAL(DP) :: w2 (3 * nat)
+  COMPLEX(DP) :: dyn (3 * nat, 3 * nat)
+  ! local (control variables)
+  INTEGER :: ntyp_, nat_, ibrav_, ityp_
+  REAL(DP) :: celldm_ (6), amass_, tau_ (3), q_ (3)
+  ! local
+  REAL(DP) :: dynr (2, 3, nat, 3, nat), err_q(3)
+  CHARACTER(len=80) :: line
+  CHARACTER(len=3)  :: atm
+  INTEGER :: nt, na, nb, naa, nbb, nu, mu, i, j
+  LOGICAL :: lfound
+  
+  !
+  !
+  REWIND (iudyn)
+  READ (iudyn, '(a)') line
+  READ (iudyn, '(a)') line
+  READ (iudyn, * ) ntyp_, nat_, ibrav_, celldm_
+  IF ( ntyp.NE.ntyp_ .OR. nat.NE.nat_ .OR.ibrav_.NE.ibrav .OR. &
+       ABS ( celldm_ (1) - celldm (1) ) > 1.0d-5) &
+          CALL errore ('readmat', 'inconsistent data', 1)
+  DO nt = 1, ntyp
+     READ (iudyn, * ) i, atm, amass_
+     IF ( nt.NE.i .OR. ABS (amass_ - amu_ry*amass (nt) ) > 1.0d-5) &
+        CALL errore ( 'readmat', 'inconsistent data', 1 + nt)
+  ENDDO
+  DO na = 1, nat
+     READ (iudyn, * ) i, ityp_, tau_
+     IF (na.NE.i.OR.ityp_.NE.ityp (na) ) CALL errore ('readmat', &
+          'inconsistent data', 10 + na)
+  ENDDO
+
+  lfound=.false.
+
+  do while(.not.lfound)
+
+     READ (iudyn, '(a)') line
+     READ (iudyn, '(a)') line
+     READ (iudyn, '(a)') line
+     READ (iudyn, '(a)') line
+
+     READ (line (11:80), * ) (q_ (i), i = 1, 3)
+
+     err_q(1:3)=dabs(q_(1:3)-q(1:3))
+
+     if(err_q(1).lt.1.d-7.and.err_q(2).lt.1.d-7.and.err_q(3).lt.1.d-7) lfound=.true.
+
+
+
+     READ (iudyn, '(a)') line
+     DO na = 1, nat
+        DO nb = 1, nat
+           READ (iudyn, * ) naa, nbb
+           IF (na.NE.naa.OR.nb.NE.nbb) CALL errore ('readmat', 'error reading &
+                &file', nb)
+           READ (iudyn, * ) ( (dynr (1, i, na, j, nb), dynr (2, i, na, j, nb) &
+                , j = 1, 3), i = 1, 3)
+        ENDDO
+     ENDDO
+
+     if(lfound) then
+        !
+        ! divide the dynamical matrix by the (input) masses (in amu)
+        !
+        DO nb = 1, nat
+           DO j = 1, 3
+              DO na = 1, nat
+                 DO i = 1, 3
+                    dynr (1, i, na, j, nb) = dynr (1, i, na, j, nb) / SQRT (amass ( &
+                         ityp (na) ) * amass (ityp (nb) ) ) / amu_ry
+                    dynr (2, i, na, j, nb) = dynr (2, i, na, j, nb) / SQRT (amass ( &
+                         ityp (na) ) * amass (ityp (nb) ) ) / amu_ry
+                 ENDDO
+              ENDDO
+           ENDDO
+        ENDDO
+       !
+       ! solve the eigenvalue problem.
+       ! NOTA BENE: eigenvectors are overwritten on dyn
+       !
+       CALL cdiagh (3 * nat, dynr, 3 * nat, w2, dyn)
+       !
+       ! divide by sqrt(mass) to get displacements
+       !
+       DO nu = 1, 3 * nat
+          DO mu = 1, 3 * nat
+             na = (mu - 1) / 3 + 1
+             dyn (mu, nu) = dyn (mu, nu) / SQRT ( amu_ry * amass (ityp (na) ) )
+          ENDDO
+       ENDDO
+       !
+       !
+     endif
+  enddo
+
+
+
+  RETURN
+END SUBROUTINE readmat_findq
+
