@@ -148,7 +148,7 @@ SUBROUTINE find_mode_sym_new (u, w2, tau, nat, nsym, sr, irt, xq,    &
            trace(iclass,igroup)=trace(iclass,igroup) + &
                 zdotc(3*nat,z(1,nu_i),1,rmode,1)
         END DO
-!              write(6,*) igroup,iclass, trace(iclass,igroup)
+!              write(6,*) 'group,class',igroup, iclass, trace(iclass,igroup)
      END DO
   END DO
   !
@@ -312,13 +312,13 @@ next=0
 DO imode = 1, 3 * nat
    IF ( imode < next .OR. ABS(w1(imode)) < 1.d-3 ) CYCLE
    IF (num_rap_mode(imode) == 0)  THEN
-      WRITE(stdout,'(5x,"omega(",i3," -",i3,") = ",f12.1,2x,"[cm-1]",3x, "-->   ?")') imode, imode, w1(imode)
+      WRITE(stdout,'(5x,"freq (",i3," -",i3,") = ",f12.1,2x,"[cm-1]",3x, "-->   ?")') imode, imode, w1(imode)
    ELSE
       irap=num_rap_mode(imode)
       next = imode + NINT(DBLE(char_mat(irap,1)))
       cdum="   "
       IF (lir) cdum=TRIM(ir_ram(irap))
-      WRITE(stdout,'(5x,"omega(",i3," -",i3,") = ",f12.1,2x,"[cm-1]",3x,"--> ",a19)') &
+      WRITE(stdout,'(5x,"freq (",i3," -",i3,") = ",f12.1,2x,"[cm-1]",3x,"--> ",a19)') &
            imode, next-1, w1(imode), name_rap(irap)//" "//cdum
    ENDIF
 ENDDO
