@@ -17,7 +17,7 @@ MODULE io_dyn_mat
   !
   USE kinds,     ONLY : DP
   USE io_global, ONLY : ionode, ionode_id
-  USE mp_global, ONLY : intra_image_comm
+  USE mp_images, ONLY : intra_image_comm
   USE mp,        ONLY : mp_bcast
   !
   IMPLICIT NONE
@@ -59,8 +59,8 @@ MODULE io_dyn_mat
 
     INTEGER, INTENT(IN) :: ityp(nat)
 
-    INTEGER :: ierr, na, nt, kc
-    REAL(DP) :: aux(3,3)
+    INTEGER :: ierr, na, nt, kc, ibrav_
+    REAL(DP) :: aux(3,3), celldm_(6)
     REAL (DP), PARAMETER ::   convfact = BOHR_RADIUS_ANGS**2
 
 
@@ -71,7 +71,7 @@ MODULE io_dyn_mat
     END IF
     CALL mp_bcast( ierr, ionode_id, intra_image_comm )
     !
-    CALL errore( 'write_dyn_mat', 'no free units to write ', ierr )
+    CALL errore( 'write_dyn_mat_header', 'no free units to write ', ierr )
     IF ( ionode ) THEN
        !
        ! ... open XML descriptor
@@ -82,16 +82,19 @@ MODULE io_dyn_mat
     ENDIF
     CALL mp_bcast( ierr, ionode_id, intra_image_comm )
     !
-    CALL errore( 'write_dyn_mat', 'error opening the dyn mat file ', ierr )
+    CALL errore( 'write_dyn_mat_header', 'error opening the dyn mat file ', ierr )
     !
     IF (ionode) THEN
        CALL iotk_write_begin(iunout, "GEOMETRY_INFO" )
        !
        CALL iotk_write_dat(iunout, "NUMBER_OF_TYPES", ntyp )
        CALL iotk_write_dat(iunout, "NUMBER_OF_ATOMS", nat )
-       CALL iotk_write_dat(iunout, "BRAVAIS_LATTICE_INDEX", ibrav )
+       ibrav_=0
+       CALL iotk_write_dat(iunout, "BRAVAIS_LATTICE_INDEX", ibrav_ )
        CALL iotk_write_dat(iunout, "SPIN_COMPONENTS", nspin_mag )
-       CALL iotk_write_dat(iunout, "CELL_DIMENSIONS", celldm )
+       celldm_=0.0_DP
+       celldm_(1)=celldm(1)
+       CALL iotk_write_dat(iunout, "CELL_DIMENSIONS", celldm_ )
        CALL iotk_write_dat(iunout, "AT", at, COLUMNS=3 )
        CALL iotk_write_dat(iunout, "BG", bg, COLUMNS=3 )
        CALL iotk_write_dat(iunout, "UNIT_CELL_VOLUME_AU", omega )
@@ -271,7 +274,7 @@ MODULE io_dyn_mat
     ENDIF
     CALL mp_bcast( ierr, ionode_id, intra_image_comm )
     !
-    CALL errore( 'write_dyn_mat', 'error opening the dyn mat file ', ierr )
+    CALL errore( 'read_dyn_mat_param', 'error opening the dyn mat file ', ierr )
     !
     IF (ionode) THEN
        CALL iotk_scan_begin(iunout, "GEOMETRY_INFO" )
@@ -282,8 +285,8 @@ MODULE io_dyn_mat
        CALL iotk_scan_end(iunout, "GEOMETRY_INFO" )
     ENDIF
 
-    CALL mp_bcast(ntyp, ionode_id)
-    CALL mp_bcast(nat, ionode_id)
+    CALL mp_bcast(ntyp, ionode_id, intra_image_comm)
+    CALL mp_bcast(nat, ionode_id, intra_image_comm)
 
     RETURN
     END SUBROUTINE read_dyn_mat_param
@@ -380,23 +383,23 @@ MODULE io_dyn_mat
           ENDIF
        ENDIF
     ENDIF
-    CALL mp_bcast(ibrav,ionode_id)
-    CALL mp_bcast(nspin_mag,ionode_id)
-    CALL mp_bcast(celldm,ionode_id)
-    CALL mp_bcast(at,ionode_id)
-    CALL mp_bcast(bg,ionode_id)
-    CALL mp_bcast(omega,ionode_id)
-    CALL mp_bcast(atm,ionode_id)
-    CALL mp_bcast(amass,ionode_id)
-    CALL mp_bcast(ityp,ionode_id)
-    CALL mp_bcast(tau,ionode_id)
-    CALL mp_bcast(m_loc,ionode_id)
-    CALL mp_bcast(nqs,ionode_id)
-    IF (PRESENT(lrigid)) CALL mp_bcast(lrigid,ionode_id)
-    IF (PRESENT(epsil))  CALL mp_bcast(epsil,ionode_id)
-    IF (PRESENT(zstareu)) CALL mp_bcast(zstareu,ionode_id)
-    IF (PRESENT(lraman)) CALL mp_bcast(lraman,ionode_id)
-    IF (PRESENT(ramtns)) CALL mp_bcast(ramtns,ionode_id)
+    CALL mp_bcast(ibrav,ionode_id, intra_image_comm)
+    CALL mp_bcast(nspin_mag,ionode_id, intra_image_comm)
+    CALL mp_bcast(celldm,ionode_id, intra_image_comm)
+    CALL mp_bcast(at,ionode_id, intra_image_comm)
+    CALL mp_bcast(bg,ionode_id, intra_image_comm)
+    CALL mp_bcast(omega,ionode_id, intra_image_comm)
+    CALL mp_bcast(atm,ionode_id, intra_image_comm)
+    CALL mp_bcast(amass,ionode_id, intra_image_comm)
+    CALL mp_bcast(ityp,ionode_id, intra_image_comm)
+    CALL mp_bcast(tau,ionode_id, intra_image_comm)
+    CALL mp_bcast(m_loc,ionode_id, intra_image_comm)
+    CALL mp_bcast(nqs,ionode_id, intra_image_comm)
+    IF (PRESENT(lrigid)) CALL mp_bcast(lrigid,ionode_id, intra_image_comm)
+    IF (PRESENT(epsil))  CALL mp_bcast(epsil,ionode_id, intra_image_comm)
+    IF (PRESENT(zstareu)) CALL mp_bcast(zstareu,ionode_id, intra_image_comm)
+    IF (PRESENT(lraman)) CALL mp_bcast(lraman,ionode_id, intra_image_comm)
+    IF (PRESENT(ramtns)) CALL mp_bcast(ramtns,ionode_id, intra_image_comm)
 
     RETURN
     END SUBROUTINE read_dyn_mat_header
@@ -426,8 +429,8 @@ MODULE io_dyn_mat
 
        CALL iotk_scan_end(iunout, "DYNAMICAL_MAT_"//TRIM(iotk_index(iq)) )
     ENDIF
-    CALL mp_bcast(xq, ionode_id)
-    CALL mp_bcast(dyn, ionode_id)
+    CALL mp_bcast(xq, ionode_id, intra_image_comm)
+    CALL mp_bcast(dyn, ionode_id, intra_image_comm)
 
     RETURN
     END SUBROUTINE read_dyn_mat
@@ -462,8 +465,8 @@ MODULE io_dyn_mat
 
        CALL iotk_close_read( iunout )
     END IF
-    IF (PRESENT(omega)) CALL mp_bcast(omega, ionode_id)
-    IF (PRESENT(u)) CALL mp_bcast(u, ionode_id)
+    IF (PRESENT(omega)) CALL mp_bcast(omega, ionode_id, intra_image_comm)
+    IF (PRESENT(u)) CALL mp_bcast(u, ionode_id, intra_image_comm)
 
     RETURN
     END SUBROUTINE read_dyn_mat_tail
@@ -489,9 +492,9 @@ MODULE io_dyn_mat
 
        CALL iotk_scan_end( iunout, "INTERATOMIC_FORCE_CONSTANTS" )
     ENDIF
-    CALL mp_bcast(nr1, ionode_id)
-    CALL mp_bcast(nr2, ionode_id)
-    CALL mp_bcast(nr3, ionode_id)
+    CALL mp_bcast(nr1, ionode_id, intra_image_comm)
+    CALL mp_bcast(nr2, ionode_id, intra_image_comm)
+    CALL mp_bcast(nr3, ionode_id, intra_image_comm)
     RETURN
     END SUBROUTINE read_ifc_param
 
@@ -531,7 +534,7 @@ MODULE io_dyn_mat
        CALL iotk_scan_end( iunout, "INTERATOMIC_FORCE_CONSTANTS" )
        CALL iotk_close_read( iunout )
     ENDIF
-    CALL mp_bcast(phid,ionode_id)
+    CALL mp_bcast(phid,ionode_id, intra_image_comm)
 
     RETURN
     END SUBROUTINE read_ifc
