@@ -106,6 +106,10 @@ PROGRAM matdyn
   !                       frequencies  (default: .false.)
   !                NB: You cannot use this option with the symmetry
   !                analysis of the modes.
+  !     fd         (logical) if .t. the ifc come from the finite displacement calculation
+  !     na_ifc     (logical) add non analitic contributions to the interatomic force 
+  !                constants if finite displacement method is used (as in Wang et al.
+  !                Phys. Rev. B 85, 224303 (2012)) [to be used in conjunction with fd.x]
   !
   !  if (readtau) atom types and positions in the supercell follow:
   !     (tau(i,na),i=1,3), ityp(na)
@@ -127,15 +131,18 @@ PROGRAM matdyn
   !
   USE kinds,      ONLY : DP
   USE mp,         ONLY : mp_bcast
-  USE mp_global,  ONLY :  mp_startup, mp_global_end
+  USE mp_world,   ONLY : world_comm
+  USE mp_global,  ONLY : mp_startup, mp_global_end
   USE environment, ONLY : environment_start, environment_end
   USE io_global,  ONLY : ionode, ionode_id, stdout
   USE io_dyn_mat, ONLY : read_dyn_mat_param, read_dyn_mat_header, &
                          read_ifc_param, read_ifc
-  USE cell_base,  ONLY : at, bg
+  USE cell_base,  ONLY : at, bg, celldm
   USE constants,  ONLY : RY_TO_THZ, RY_TO_CMM1, amu_ry
   USE symm_base,  ONLY : set_sym
   USE rap_point_group,  ONLY : code_group
+  USE bz_form,    ONLY : transform_label_coord
+  USE parser,     ONLY : read_line
 
   USE ifconstants, ONLY : frc, atm, zeu, tau_blk, ityp_blk, m_loc
   !
@@ -153,9 +160,9 @@ PROGRAM matdyn
   CHARACTER(LEN=256) :: flfrc, flfrq, flvec, fltau, fldos, filename, fldyn, fleig
   CHARACTER(LEN=10)  :: asr
   LOGICAL :: dos, has_zstar, q_in_cryst_coord, eigen_similarity
-  COMPLEX(DP), ALLOCATABLE :: dyn(:,:,:,:), dyn_blk(:,:,:,:)
+  COMPLEX(DP), ALLOCATABLE :: dyn(:,:,:,:), dyn_blk(:,:,:,:), frc_ifc(:,:,:,:)
   COMPLEX(DP), ALLOCATABLE :: z(:,:)
-  REAL(DP), ALLOCATABLE:: tau(:,:), q(:,:), w2(:,:), freq(:,:)
+  REAL(DP), ALLOCATABLE:: tau(:,:), q(:,:), w2(:,:), freq(:,:), wq(:)
   INTEGER, ALLOCATABLE:: tetra(:,:), ityp(:), itau_blk(:)
   REAL(DP) ::     omega,alat, &! cell parameters and volume
                   at_blk(3,3), bg_blk(3,3),  &! original cell
@@ -173,10 +180,10 @@ PROGRAM matdyn
 
   INTEGER :: nspin_mag, nqs, ios
   !
-  LOGICAL :: readtau, la2F, xmlifc, lo_to_split
+  LOGICAL :: readtau, la2F, xmlifc, lo_to_split, na_ifc, fd
   !
-  REAL(DP) :: qhat(3), qh, DeltaE, Emin=0._dp, Emax, E, DOSofE(1)
-  REAL(DP) :: celldm(6), delta, pathL
+  REAL(DP) :: qhat(3), qh, DeltaE, Emin=0._dp, Emax, E, DOSofE(1), qq
+  REAL(DP) :: delta, pathL
   REAL(DP), ALLOCATABLE :: xqaux(:,:)
   INTEGER, ALLOCATABLE :: nqb(:)
   INTEGER :: n, i, j, it, nq, nqx, na, nb, ndos, iout, nqtot, iout_dyn, iout_eig
@@ -188,12 +195,22 @@ PROGRAM matdyn
   ! .... variables for band plotting based on similarity of eigenvalues
   COMPLEX(DP), ALLOCATABLE :: tmp_z(:,:)
   REAL(DP), ALLOCATABLE :: abs_similarity(:,:), tmp_w2(:)
-  INTEGER :: location(1)
+  COMPLEX(DP), ALLOCATABLE :: f_of_q(:,:,:,:)
+  INTEGER :: location(1), isig
+  CHARACTER(LEN=6) :: int_to_char
   LOGICAL, ALLOCATABLE :: mask(:)
+  INTEGER            :: npk_label, nch
+  CHARACTER(LEN=3), ALLOCATABLE :: letter(:)
+  INTEGER, ALLOCATABLE :: label_list(:)
+  LOGICAL :: tend, terr
+  CHARACTER(LEN=256) :: input_line, buffer
+  CHARACTER(LEN=10) :: point_label_type
+  CHARACTER(len=80) :: k_points = 'tpiba'
   !
   NAMELIST /input/ flfrc, amass, asr, flfrq, flvec, fleig, at, dos,  &
        &           fldos, nk1, nk2, nk3, l1, l2, l3, ntyp, readtau, fltau, &
-       &           la2F, ndos, DeltaE, q_in_band_form, q_in_cryst_coord, fldyn
+       &           la2F, ndos, DeltaE, q_in_band_form, q_in_cryst_coord, &
+       &           eigen_similarity, fldyn, na_ifc, fd, point_label_type
   !
   CALL mp_startup()
   CALL environment_start('MATDYN')
@@ -230,37 +247,43 @@ PROGRAM matdyn
      q_in_band_form=.FALSE.
      eigen_similarity=.FALSE.
      q_in_cryst_coord = .FALSE.
+     na_ifc=.FALSE.
+     fd=.FALSE.
+     point_label_type='SC'
      !
      !
      IF (ionode) READ (5,input,IOSTAT=ios)
-     CALL mp_bcast(ios, ionode_id) 
+     CALL mp_bcast(ios, ionode_id, world_comm) 
      CALL errore('matdyn', 'reading input namelist', ABS(ios))
-     CALL mp_bcast(dos,ionode_id)
-     CALL mp_bcast(deltae,ionode_id)
-     CALL mp_bcast(ndos,ionode_id)
-     CALL mp_bcast(nk1,ionode_id)
-     CALL mp_bcast(nk2,ionode_id)
-     CALL mp_bcast(nk3,ionode_id)
-     CALL mp_bcast(asr,ionode_id)
-     CALL mp_bcast(readtau,ionode_id)
-     CALL mp_bcast(flfrc,ionode_id)
-     CALL mp_bcast(fldos,ionode_id)
-     CALL mp_bcast(flfrq,ionode_id)
-     CALL mp_bcast(flvec,ionode_id)
-     CALL mp_bcast(fleig,ionode_id)
-     CALL mp_bcast(fldyn,ionode_id)
-     CALL mp_bcast(fltau,ionode_id)
-     CALL mp_bcast(amass,ionode_id)
-     CALL mp_bcast(amass_blk,ionode_id)
-     CALL mp_bcast(at,ionode_id)
-     CALL mp_bcast(ntyp,ionode_id)
-     CALL mp_bcast(l1,ionode_id)
-     CALL mp_bcast(l2,ionode_id)
-     CALL mp_bcast(l3,ionode_id)
-     CALL mp_bcast(la2f,ionode_id)
-     CALL mp_bcast(q_in_band_form,ionode_id)
-     CALL mp_bcast(eigen_similarity,ionode_id)
-     CALL mp_bcast(q_in_cryst_coord,ionode_id)
+     CALL mp_bcast(dos,ionode_id, world_comm)
+     CALL mp_bcast(deltae,ionode_id, world_comm)
+     CALL mp_bcast(ndos,ionode_id, world_comm)
+     CALL mp_bcast(nk1,ionode_id, world_comm)
+     CALL mp_bcast(nk2,ionode_id, world_comm)
+     CALL mp_bcast(nk3,ionode_id, world_comm)
+     CALL mp_bcast(asr,ionode_id, world_comm)
+     CALL mp_bcast(readtau,ionode_id, world_comm)
+     CALL mp_bcast(flfrc,ionode_id, world_comm)
+     CALL mp_bcast(fldos,ionode_id, world_comm)
+     CALL mp_bcast(flfrq,ionode_id, world_comm)
+     CALL mp_bcast(flvec,ionode_id, world_comm)
+     CALL mp_bcast(fleig,ionode_id, world_comm)
+     CALL mp_bcast(fldyn,ionode_id, world_comm)
+     CALL mp_bcast(fltau,ionode_id, world_comm)
+     CALL mp_bcast(amass,ionode_id, world_comm)
+     CALL mp_bcast(amass_blk,ionode_id, world_comm)
+     CALL mp_bcast(at,ionode_id, world_comm)
+     CALL mp_bcast(ntyp,ionode_id, world_comm)
+     CALL mp_bcast(l1,ionode_id, world_comm)
+     CALL mp_bcast(l2,ionode_id, world_comm)
+     CALL mp_bcast(l3,ionode_id, world_comm)
+     CALL mp_bcast(na_ifc,ionode_id, world_comm)
+     CALL mp_bcast(fd,ionode_id, world_comm)
+     CALL mp_bcast(la2f,ionode_id, world_comm)
+     CALL mp_bcast(q_in_band_form,ionode_id, world_comm)
+     CALL mp_bcast(eigen_similarity,ionode_id, world_comm)
+     CALL mp_bcast(q_in_cryst_coord,ionode_id, world_comm)
+     CALL mp_bcast(point_label_type,ionode_id, world_comm)
 
      !
      ! read force constants
@@ -286,6 +309,7 @@ PROGRAM matdyn
         CALL readfc ( flfrc, nr1, nr2, nr3, epsil, nat_blk, &
             ibrav, alat, at_blk, ntyp_blk, &
             amass_blk, omega_blk, has_zstar)
+        print*,'alat', alat
      ENDIF
      !
      CALL recips ( at_blk(1,1),at_blk(1,2),at_blk(1,3),  &
@@ -378,50 +402,88 @@ PROGRAM matdyn
         ! read q-point list
         !
         IF (ionode) READ (5,*) nq
-        CALL mp_bcast(nq, ionode_id)
+        CALL mp_bcast(nq, ionode_id, world_comm)
         ALLOCATE ( q(3,nq) )
         ALLOCATE( tetra(1,1) )
         IF (.NOT.q_in_band_form) THEN
            DO n = 1,nq
               IF (ionode) READ (5,*) (q(i,n),i=1,3)
            END DO
-           CALL mp_bcast(q, ionode_id)
+           CALL mp_bcast(q, ionode_id, world_comm)
+           !
+           IF (q_in_cryst_coord)  CALL cryst_to_cart(nq,q,bg,+1)
         ELSE
-           ALLOCATE(nqb(nq))
-           ALLOCATE(xqaux(3,nq))
-           DO n = 1,nq
-              IF (ionode) READ (5,*) (q(i,n),i=1,3), nqb(n)
-           END DO
-           CALL mp_bcast(q, ionode_id)
-           CALL mp_bcast(nqb, ionode_id)
+           ALLOCATE( nqb(nq) )
+           ALLOCATE( xqaux(3,nq) )
+           ALLOCATE( letter(nq) )
+           ALLOCATE( label_list(nq) )
+           npk_label=0
+           DO n = 1, nq
+              CALL read_line( input_line, end_of_file = tend, error = terr )
+              IF (tend) CALL errore('matdyn','Missing lines',1)
+              IF (terr) CALL errore('matdyn','Error reading q points',1)
+              DO j=1,256   ! loop over all characters of input_line
+                 IF ( (ICHAR(input_line(j:j)) < 58 .AND. &   ! a digit
+                       ICHAR(input_line(j:j)) > 47)      &
+                   .OR.ICHAR(input_line(j:j)) == 43 .OR. &   ! the + sign
+                       ICHAR(input_line(j:j)) == 45 .OR. &   ! the - sign
+                       ICHAR(input_line(j:j)) == 46 ) THEN   ! a dot .
+!
+!   This is a digit, therefore this line contains the coordinates of the
+!   k point. We read it and exit from the loop on characters
+!
+                     READ(input_line,*) xqaux(1,n), xqaux(2,n), xqaux(3,n), &
+                                                    nqb(n)
+                     EXIT
+                 ELSEIF ((ICHAR(input_line(j:j)) < 123 .AND. &
+                          ICHAR(input_line(j:j)) > 64))  THEN
+!
+!   This is a letter, not a space character. We read the next three 
+!   characters and save them in the letter array, save also which k point
+!   it is
+!
+                    npk_label=npk_label+1
+                    READ(input_line(j:),'(a3)') letter(npk_label)
+                    label_list(npk_label)=n
+!
+!  now we remove the letters from input_line and read the number of points
+!  of the line. The next two line should account for the case in which
+!  there is only one space between the letter and the number of points.
+!
+                    nch=3
+                    IF ( ICHAR(input_line(j+1:j+1))==32 .OR. &
+                         ICHAR(input_line(j+2:j+2))==32 ) nch=2
+                    buffer=input_line(j+nch:)
+                    READ(buffer,*,err=20,iostat=ios) nqb(n)
+20                  IF (ios /=0) CALL errore('matdyn',&
+                                      'problem reading number of points',1)
+                    EXIT
+                 ENDIF
+              ENDDO
+           ENDDO
+           IF (q_in_cryst_coord) k_points='crystal'
+           IF ( npk_label > 0 ) &
+              CALL transform_label_coord(ibrav, celldm, xqaux, letter, &
+                   label_list, npk_label, nq, k_points, point_label_type )
+
+           DEALLOCATE(letter)
+           DEALLOCATE(label_list)
+
+           CALL mp_bcast(xqaux, ionode_id, world_comm)
+           CALL mp_bcast(nqb, ionode_id, world_comm)
+           IF (q_in_cryst_coord)  CALL cryst_to_cart(nq,xqaux,bg,+1)
            nqtot=SUM(nqb(1:nq-1))+1
            DO i=1,nq-1
               IF (nqb(i)==0) nqtot=nqtot+1
            ENDDO
-           xqaux(:,1:nq)=q(:,1:nq)
            DEALLOCATE(q)
            ALLOCATE(q(3,nqtot))
-           nqtot=0
-           DO i=1,nq-1
-              IF (nqb(i)>0) THEN
-                 delta=1.0_DP/nqb(i)
-                 DO j=0,nqb(i)-1
-                    nqtot=nqtot+1
-                    q(:,nqtot)=xqaux(:,i)+delta*j*(xqaux(:,i+1)-xqaux(:,i))
-                 ENDDO
-              ELSE
-                 nqtot=nqtot+1
-                 q(:,nqtot) = xqaux(:,i)
-              ENDIF
-           ENDDO
-           nqtot=nqtot+1
-           q(:,nqtot)=xqaux(:,nq)
+           ALLOCATE(wq(nqtot))
+           CALL generate_k_along_lines(nq, xqaux, nqb, q, wq, nqtot)
            nq=nqtot
            DEALLOCATE(xqaux)
            DEALLOCATE(nqb)
         END IF
-        !
-        IF (q_in_cryst_coord)  CALL cryst_to_cart(nq,q,bg,+1)
         ! 
      END IF
      !
@@ -452,10 +514,10 @@ PROGRAM matdyn
      END IF
 
      ALLOCATE ( dyn(3,3,nat,nat), dyn_blk(3,3,nat_blk,nat_blk) )
-     ALLOCATE ( z(3*nat,3*nat), w2(3*nat,nq) )
+     ALLOCATE ( z(3*nat,3*nat), w2(3*nat,nq), f_of_q(3,3,nat,nat) )
      ALLOCATE ( tmp_w2(3*nat), abs_similarity(3*nat,3*nat), mask(3*nat) )
 
-     if(la2F.and.ionode) open(300,file='dyna2F',status='unknown')
+     if(la2F.and.ionode) open(unit=300,file='dyna2F',status='unknown')
      IF (xmlifc) CALL set_sym(nat, tau, ityp, nspin_mag, m_loc, 6, 6, 6 )
 
      ALLOCATE(num_rap_mode(3*nat,nq))
@@ -467,12 +529,24 @@ PROGRAM matdyn
         dyn(:,:,:,:) = (0.d0, 0.d0)
 
         lo_to_split=.FALSE.
+        f_of_q(:,:,:,:)=CMPLX(0.d0,0.d0)
+
+        IF(na_ifc) THEN
+
+           qq=sqrt(q(1,n)**2+q(2,n)**2+q(3,n)**3)
+           if(qq == 0.0) qq=1.0
+           qhat(1)=q(1,n)/qq
+           qhat(2)=q(2,n)/qq
+           qhat(3)=q(3,n)/qq
+
+           CALL nonanal_ifc (nat,nat_blk,itau_blk,epsil,qhat,zeu,omega,dyn, &
+                           nr1, nr2, nr3,f_of_q)
+        END IF
+
         CALL setupmat (q(1,n), dyn, nat, at, bg, tau, itau_blk, nsc, alat, &
              dyn_blk, nat_blk, at_blk, bg_blk, tau_blk, omega_blk,  &
-             epsil, zeu, frc, nr1,nr2,nr3, has_zstar, rws, nrws)
-        !
-        ! the following lines recognize as q=0 all q=G reciprocal latt. vectors
-        !
+             epsil, zeu, frc, nr1,nr2,nr3, has_zstar, rws, nrws, na_ifc,f_of_q,fd)
+
         qhat(1) = q(1,n)*at(1,1)+q(2,n)*at(2,1)+q(3,n)*at(3,1)
         qhat(2) = q(1,n)*at(1,2)+q(2,n)*at(2,2)+q(3,n)*at(3,2)
         qhat(3) = q(1,n)*at(1,3)+q(2,n)*at(2,3)+q(3,n)*at(3,3)
@@ -568,7 +642,7 @@ PROGRAM matdyn
         endif
         !
 
-        IF (ionode.and.iout.ne.0) CALL writemodes(nax,nat,q(1,n),w2(1,n),z,iout)
+        IF (ionode.and.iout.ne.0) CALL writemodes(nat,q(1,n),w2(1,n),z,iout)
 
         !
      END DO  !nq
@@ -648,7 +722,7 @@ PROGRAM matdyn
            !
            !WRITE (2, '(F15.10,F15.2,F15.6,F20.5)') &
            !     E, E*RY_TO_CMM1, E*RY_TO_THZ, 0.5d0*DOSofE(1)
-           IF (ionode) WRITE (2, '(E12.4,E12.4)') E, 0.5d0*DOSofE(1)
+           IF (ionode) WRITE (2, '(ES12.4,ES12.4)') E, 0.5d0*DOSofE(1)
         END DO
         IF (ionode) CLOSE(unit=2)
      END IF  !dos
@@ -657,6 +731,14 @@ PROGRAM matdyn
      !    for a2F
      !
      IF(la2F) THEN
+         !
+         IF (.NOT. dos) THEN
+            DO isig=1,10
+               OPEN (unit=200+isig,file='elph.gamma.'//&
+                  TRIM(int_to_char(isig)), status='unknown',form='formatted')
+               WRITE(200+isig, '(" &plot nbnd=",i4,", nks=",i4," /")') 3*nat, nq
+            END DO
+         END IF
          !
          ! convert frequencies to Ry
          !
@@ -667,7 +749,13 @@ PROGRAM matdyn
          call a2Fdos (nat, nq, nr1, nr2, nr3, ibrav, at, bg, tau, alat, &
                            nsc, nat_blk, at_blk, bg_blk, itau_blk, omega_blk, &
                            rws, nrws, dos, Emin, DeltaE, ndos, &
-                           ntetra, tetra, asr, q, freq)
+                           ntetra, tetra, asr, q, freq,fd)
+         !
+         IF (.NOT.dos) THEN
+            DO isig=1,10
+               CLOSE(UNIT=200+isig)
+            ENDDO
+         ENDIF
      END IF
      DEALLOCATE ( freq)
      DEALLOCATE(num_rap_mode)
@@ -689,22 +777,24 @@ SUBROUTINE readfc ( flfrc, nr1, nr2, nr3, epsil, nat,    &
   !
   USE kinds,      ONLY : DP
   USE ifconstants,ONLY : tau => tau_blk, ityp => ityp_blk, frc, zeu
+  USE cell_base,  ONLY : celldm
   USE io_global,  ONLY : ionode, ionode_id, stdout
   USE mp,         ONLY : mp_bcast 
+  USE mp_world,   ONLY : world_comm 
   USE constants,  ONLY : amu_ry
   !
   IMPLICIT NONE
   ! I/O variable
-  CHARACTER(LEN=256) flfrc
-  INTEGER ibrav, nr1,nr2,nr3,nat, ntyp
-  REAL(DP) alat, at(3,3), epsil(3,3)
-  LOGICAL has_zstar
+  CHARACTER(LEN=256) :: flfrc
+  INTEGER :: ibrav, nr1,nr2,nr3,nat, ntyp
+  REAL(DP) :: alat, at(3,3), epsil(3,3)
+  LOGICAL :: has_zstar
   ! local variables
-  INTEGER i, j, na, nb, m1,m2,m3
-  INTEGER ibid, jbid, nabid, nbbid, m1bid,m2bid,m3bid
-  REAL(DP) amass(ntyp), amass_from_file, celldm(6), omega
-  INTEGER nt
-  CHARACTER(LEN=3) atm
+  INTEGER :: i, j, na, nb, m1,m2,m3
+  INTEGER :: ibid, jbid, nabid, nbbid, m1bid,m2bid,m3bid
+  REAL(DP) :: amass(ntyp), amass_from_file, omega
+  INTEGER :: nt
+  CHARACTER(LEN=3) :: atm
   !
   !
   IF (ionode) OPEN (unit=1,file=flfrc,status='old',form='formatted')
@@ -717,12 +807,12 @@ SUBROUTINE readfc ( flfrc, nr1, nr2, nr3, epsil, nat,    &
         read(1,*) ((at(i,j),i=1,3),j=1,3)
      end if
   ENDIF
-  CALL mp_bcast(ntyp, ionode_id)
-  CALL mp_bcast(nat, ionode_id)
-  CALL mp_bcast(ibrav, ionode_id)
-  CALL mp_bcast(celldm, ionode_id)
+  CALL mp_bcast(ntyp, ionode_id, world_comm)
+  CALL mp_bcast(nat, ionode_id, world_comm)
+  CALL mp_bcast(ibrav, ionode_id, world_comm)
+  CALL mp_bcast(celldm, ionode_id, world_comm)
   IF (ibrav==0) THEN
-     CALL mp_bcast(at, ionode_id)
+     CALL mp_bcast(at, ionode_id, world_comm)
   ENDIF
   !
   CALL latgen(ibrav,celldm,at(1,1),at(1,2),at(1,3),omega)
@@ -734,9 +824,9 @@ SUBROUTINE readfc ( flfrc, nr1, nr2, nr3, epsil, nat,    &
   !
   DO nt = 1,ntyp
      IF (ionode) READ(1,*) i,atm,amass_from_file
-     CALL mp_bcast(i,ionode_id)
-     CALL mp_bcast(atm,ionode_id)
-     CALL mp_bcast(amass_from_file,ionode_id)
+     CALL mp_bcast(i,ionode_id, world_comm)
+     CALL mp_bcast(atm,ionode_id, world_comm)
+     CALL mp_bcast(amass_from_file,ionode_id, world_comm)
      IF (i.NE.nt) CALL errore ('readfc','wrong data read',nt)
      IF (amass(nt).EQ.0.d0) THEN
         amass(nt) = amass_from_file/amu_ry
@@ -749,35 +839,35 @@ SUBROUTINE readfc ( flfrc, nr1, nr2, nr3, epsil, nat,    &
   !
   DO na=1,nat
      IF (ionode) READ(1,*) i,ityp(na),(tau(j,na),j=1,3)
-     CALL mp_bcast(i,ionode_id)
+     CALL mp_bcast(i,ionode_id, world_comm)
      IF (i.NE.na) CALL errore ('readfc','wrong data read',na)
   END DO
-  CALL mp_bcast(ityp,ionode_id)
-  CALL mp_bcast(tau,ionode_id)
+  CALL mp_bcast(ityp,ionode_id, world_comm)
+  CALL mp_bcast(tau,ionode_id, world_comm)
   !
   !  read macroscopic variable
   !
   IF (ionode) READ (1,*) has_zstar
-  CALL mp_bcast(has_zstar,ionode_id)
+  CALL mp_bcast(has_zstar,ionode_id, world_comm)
   IF (has_zstar) THEN
      IF (ionode) READ(1,*) ((epsil(i,j),j=1,3),i=1,3)
-     CALL mp_bcast(epsil,ionode_id)
+     CALL mp_bcast(epsil,ionode_id, world_comm)
      IF (ionode) THEN
         DO na=1,nat
            READ(1,*)
            READ(1,*) ((zeu(i,j,na),j=1,3),i=1,3)
         END DO
      ENDIF
-     CALL mp_bcast(zeu,ionode_id)
+     CALL mp_bcast(zeu,ionode_id, world_comm)
   ELSE
      zeu  (:,:,:) = 0.d0
      epsil(:,:) = 0.d0
   END IF
   !
   IF (ionode) READ (1,*) nr1,nr2,nr3
-  CALL mp_bcast(nr1,ionode_id)
-  CALL mp_bcast(nr2,ionode_id)
-  CALL mp_bcast(nr3,ionode_id)
+  CALL mp_bcast(nr1,ionode_id, world_comm)
+  CALL mp_bcast(nr2,ionode_id, world_comm)
+  CALL mp_bcast(nr3,ionode_id, world_comm)
   !
   !  read real-space interatomic force constants
   !
@@ -788,10 +878,10 @@ SUBROUTINE readfc ( flfrc, nr1, nr2, nr3, epsil, nat,    &
         DO na=1,nat
            DO nb=1,nat
               IF (ionode) READ (1,*) ibid, jbid, nabid, nbbid
-              CALL mp_bcast(ibid,ionode_id)
-              CALL mp_bcast(jbid,ionode_id)
-              CALL mp_bcast(nabid,ionode_id)
-              CALL mp_bcast(nbbid,ionode_id)
+              CALL mp_bcast(ibid,ionode_id, world_comm)
+              CALL mp_bcast(jbid,ionode_id, world_comm)
+              CALL mp_bcast(nabid,ionode_id, world_comm)
+              CALL mp_bcast(nbbid,ionode_id, world_comm)
               IF(i .NE.ibid  .OR. j .NE.jbid .OR.                   &
                  na.NE.nabid .OR. nb.NE.nbbid)                      &
                  CALL errore  ('readfc','error in reading',1)
@@ -799,7 +889,7 @@ SUBROUTINE readfc ( flfrc, nr1, nr2, nr3, epsil, nat,    &
                           frc(m1,m2,m3,i,j,na,nb),                  &
                            m1=1,nr1),m2=1,nr2),m3=1,nr3)
                
-              CALL mp_bcast(frc(:,:,:,i,j,na,nb),ionode_id)
+              CALL mp_bcast(frc(:,:,:,i,j,na,nb),ionode_id, world_comm)
            END DO
         END DO
      END DO
@@ -811,7 +901,7 @@ SUBROUTINE readfc ( flfrc, nr1, nr2, nr3, epsil, nat,    &
 END SUBROUTINE readfc
 !
 !-----------------------------------------------------------------------
-SUBROUTINE frc_blk(dyn,q,tau,nat,nr1,nr2,nr3,frc,at,bg,rws,nrws)
+SUBROUTINE frc_blk(dyn,q,tau,nat,nr1,nr2,nr3,frc,at,bg,rws,nrws,f_of_q,fd)
   !-----------------------------------------------------------------------
   ! calculates the dynamical matrix at q from the (short-range part of the)
   ! force constants
@@ -822,14 +912,16 @@ SUBROUTINE frc_blk(dyn,q,tau,nat,nr1,nr2,nr3,frc,at,bg,rws,nrws)
   !
   IMPLICIT NONE
   INTEGER nr1, nr2, nr3, nat, n1, n2, n3, &
-          ipol, jpol, na, nb, m1, m2, m3, nint, i,j, nrws
-  COMPLEX(DP) dyn(3,3,nat,nat)
+          ipol, jpol, na, nb, m1, m2, m3, nint, i,j, nrws, nax
+  COMPLEX(DP) dyn(3,3,nat,nat), f_of_q(3,3,nat,nat)
   REAL(DP) frc(nr1,nr2,nr3,3,3,nat,nat), tau(3,nat), q(3), arg, &
                at(3,3), bg(3,3), r(3), weight, r_ws(3),  &
-               total_weight, rws(0:3,nrws)
+               total_weight, rws(0:3,nrws), alat
   REAL(DP), EXTERNAL :: wsweight
   REAL(DP),SAVE,ALLOCATABLE :: wscache(:,:,:,:,:)
+  REAL(DP), ALLOCATABLE :: ttt(:,:,:,:,:), tttx(:,:)
   LOGICAL,SAVE :: first=.true.
+  LOGICAL :: fd
   !
   FIRST_TIME : IF (first) THEN
     first=.false.
@@ -844,6 +936,7 @@ SUBROUTINE frc_blk(dyn,q,tau,nat,nr1,nr2,nr3,frc,at,bg,rws,nrws)
                    DO i=1, 3
                       r(i) = n1*at(i,1)+n2*at(i,2)+n3*at(i,3)
                       r_ws(i) = r(i) + tau(i,na)-tau(i,nb)
+                      if (fd) r_ws(i) = r(i) + tau(i,nb)-tau(i,na)
                    END DO
                    wscache(n3,n2,n1,nb,na) = wsweight(r_ws,rws,nrws)
                 ENDDO
@@ -853,6 +946,10 @@ SUBROUTINE frc_blk(dyn,q,tau,nat,nr1,nr2,nr3,frc,at,bg,rws,nrws)
     ENDDO
   ENDIF FIRST_TIME
   !
+  ALLOCATE(ttt(3,nat,nr1,nr2,nr3))
+  ALLOCATE(tttx(3,nat*nr1*nr2*nr3))
+  ttt(:,:,:,:,:)=0.d0
+
   DO na=1, nat
      DO nb=1, nat
         total_weight=0.0d0
@@ -877,15 +974,21 @@ SUBROUTINE frc_blk(dyn,q,tau,nat,nr1,nr2,nr3,frc,at,bg,rws,nrws)
                     IF(m2.LE.0) m2=m2+nr2
                     m3 = MOD(n3+1,nr3)
                     IF(m3.LE.0) m3=m3+nr3
+                 !   write(*,'(6i4)') n1,n2,n3,m1,m2,m3
                     !
                     ! FOURIER TRANSFORM
                     !
+                    do i=1,3
+                       ttt(i,na,m1,m2,m3)=tau(i,na)+m1*at(i,1)+m2*at(i,2)+m3*at(i,3)
+                       ttt(i,nb,m1,m2,m3)=tau(i,nb)+m1*at(i,1)+m2*at(i,2)+m3*at(i,3)
+                    end do
+
                     arg = tpi*(q(1)*r(1) + q(2)*r(2) + q(3)*r(3))
                     DO ipol=1, 3
                        DO jpol=1, 3
                           dyn(ipol,jpol,na,nb) =                 &
                                dyn(ipol,jpol,na,nb) +            &
-                               frc(m1,m2,m3,ipol,jpol,na,nb)     &
+                               (frc(m1,m2,m3,ipol,jpol,na,nb)+f_of_q(ipol,jpol,na,nb))     &
                                *CMPLX(COS(arg),-SIN(arg),kind=DP)*weight
                        END DO
                     END DO
@@ -901,13 +1004,36 @@ SUBROUTINE frc_blk(dyn,q,tau,nat,nr1,nr2,nr3,frc,at,bg,rws,nrws)
      END DO
   END DO
   !
+!  alat=10.2
+!  nax=0
+!  DO n1=1,nr1
+!     DO n2=1,nr2
+!        DO n3=1,nr3
+!           do na=1,nat
+!              nax=nax+1
+!              do i=1,3
+!                 tttx(i,nax)=ttt(i,na,n1,n2,n3)*alat*0.529177
+!              end do
+!           end do
+!        end do
+!     end do
+!  end do
+!
+!  do nb=1,nat
+!     write(6,'(3(f15.9,1x))') tau(1,nb),tau(2,nb),tau(3,nb)
+!  enddo
+!  print*, '========='
+!  do nb=1,nat*nr1*nr2*nr3
+!     write(6,'(3(f15.9,1x))') tttx(1,nb),tttx(2,nb),tttx(3,nb)
+!  enddo
+! 
   RETURN
 END SUBROUTINE frc_blk
 !
 !-----------------------------------------------------------------------
 SUBROUTINE setupmat (q,dyn,nat,at,bg,tau,itau_blk,nsc,alat, &
      &         dyn_blk,nat_blk,at_blk,bg_blk,tau_blk,omega_blk, &
-     &                 epsil,zeu,frc,nr1,nr2,nr3,has_zstar,rws,nrws)
+     &                 epsil,zeu,frc,nr1,nr2,nr3,has_zstar,rws,nrws,na_ifc,f_of_q,fd)
   !-----------------------------------------------------------------------
   ! compute the dynamical matrix (the analytic part only)
   !
@@ -923,16 +1049,16 @@ SUBROUTINE setupmat (q,dyn,nat,at,bg,tau,itau_blk,nsc,alat, &
                   epsil(3,3), zeu(3,3,nat_blk), rws(0:3,nrws),   &
                   frc(nr1,nr2,nr3,3,3,nat_blk,nat_blk)
   REAL(DP) :: tau_blk(3,nat_blk), at_blk(3,3), bg_blk(3,3), omega_blk
-  COMPLEX(DP) dyn_blk(3,3,nat_blk,nat_blk)
+  COMPLEX(DP) dyn_blk(3,3,nat_blk,nat_blk), f_of_q(3,3,nat,nat)
   COMPLEX(DP) ::  dyn(3,3,nat,nat)
-  LOGICAL has_zstar
+  LOGICAL :: has_zstar, na_ifc, fd
   !
   ! local variables
   !
   REAL(DP) :: arg
   COMPLEX(DP) :: cfac(nat)
   INTEGER :: i,j,k, na,nb, na_blk, nb_blk, iq
-  REAL(DP) qp(3), qbid(3,nsc) ! automatic array
+  REAL(DP) :: qp(3), qbid(3,nsc) ! automatic array
   !
   !
   CALL q_gen(nsc,qbid,at_blk,bg_blk,at,bg)
@@ -945,10 +1071,10 @@ SUBROUTINE setupmat (q,dyn,nat,at,bg,tau,itau_blk,nsc,alat, &
      !
      dyn_blk(:,:,:,:) = (0.d0,0.d0)
      CALL frc_blk (dyn_blk,qp,tau_blk,nat_blk,              &
-          &              nr1,nr2,nr3,frc,at_blk,bg_blk,rws,nrws)
-     IF (has_zstar) &
-          CALL rgd_blk(nr1,nr2,nr3,nat_blk,dyn_blk,qp,tau_blk,   &
-                       epsil,zeu,bg_blk,omega_blk,+1.d0)
+          &              nr1,nr2,nr3,frc,at_blk,bg_blk,rws,nrws,f_of_q,fd)
+      IF (has_zstar .and. .not.na_ifc) &
+           CALL rgd_blk(nr1,nr2,nr3,nat_blk,dyn_blk,qp,tau_blk,   &
+                        epsil,zeu,bg_blk,omega_blk,+1.d0)
      !
      DO na=1,nat
         na_blk = itau_blk(na)
@@ -1793,6 +1919,7 @@ SUBROUTINE read_tau &
   USE kinds,      ONLY : DP
   USE io_global,  ONLY : ionode_id, ionode
   USE mp,         ONLY : mp_bcast
+  USE mp_world,   ONLY : world_comm
   !
   IMPLICIT NONE
   !
@@ -1807,8 +1934,8 @@ SUBROUTINE read_tau &
   !
   DO na=1,nat
      IF (ionode) READ(5,*) (tau(i,na),i=1,3), ityp(na)
-     CALL mp_bcast(tau(:,na),ionode_id)
-     CALL mp_bcast(ityp(na),ionode_id)
+     CALL mp_bcast(tau(:,na),ionode_id, world_comm)
+     CALL mp_bcast(ityp(na),ionode_id, world_comm)
      IF (ityp(na).LE.0 .OR. ityp(na) .GT. ntyp) &
           CALL errore('read_tau',' wrong atomic type', na)
      DO na_blk=1,nat_blk
@@ -1904,25 +2031,27 @@ END SUBROUTINE gen_qpoints
 SUBROUTINE a2Fdos &
      (nat, nq, nr1, nr2, nr3, ibrav, at, bg, tau, alat, &
      nsc, nat_blk, at_blk, bg_blk, itau_blk, omega_blk, rws, nrws, &
-     dos, Emin, DeltaE, ndos, ntetra, tetra, asr, q, freq )
+     dos, Emin, DeltaE, ndos, ntetra, tetra, asr, q, freq,fd )
   !-----------------------------------------------------------------------
   !
   USE kinds,      ONLY : DP
   USE io_global,   ONLY : ionode, ionode_id
   USE mp,          ONLY : mp_bcast
-  USE ifconstants
+  USE mp_world,    ONLY : world_comm
+  USE mp_images,   ONLY : intra_image_comm
+  USE ifconstants, ONLY : zeu, tau_blk
   USE constants,  ONLY : pi, RY_TO_THZ
   !
   IMPLICIT NONE
   !
   INTEGER, INTENT(in) :: nat, nq, nr1, nr2, nr3, ibrav, ndos, ntetra, &
        tetra(4, ntetra)
-  LOGICAL, INTENT(in) :: dos
+  LOGICAL, INTENT(in) :: dos,fd
   CHARACTER(LEN=*), INTENT(IN) :: asr
   REAL(DP), INTENT(in) :: freq(3*nat,nq), q(3,nq), at(3,3), bg(3,3), &
        tau(3,nat), alat, Emin, DeltaE
   !
-  INTEGER, INTENT(in) :: nsc, nat_blk, itau_blk, nrws
+  INTEGER, INTENT(in) :: nsc, nat_blk, itau_blk(nat), nrws
   REAL(DP), INTENT(in) :: rws(0:3,nrws), at_blk(3,3), bg_blk(3,3), omega_blk
   !
   REAL(DP), ALLOCATABLE    :: gamma(:,:), frcg(:,:,:,:,:,:,:)
@@ -1932,40 +2061,44 @@ SUBROUTINE a2Fdos &
                               deg(10), fermi(10), E
   real(DP), parameter      :: eps_w2 = 0.0000001d0
   integer                  :: isig, ifn, n, m, na, nb, nc, nu, nmodes, &
-                              i,j,k, ngauss, jsig, p1, p2, p3, filea2F
-  character(len=14)        :: name
+                              i,j,k, ngauss, jsig, p1, p2, p3, filea2F, ios
+  character(len=256)        :: name
+  character(len=256)        :: elph_dir
   real(DP), external       :: dos_gam
   CHARACTER(LEN=6)         :: int_to_char
   !
   !
   nmodes = 3*nat
-  IF (ionode) THEN
+  elph_dir='elph_dir/'
   do isig=1,10
      filea2F = 60 + isig
-     write(name,"(A10,I2)") 'a2Fmatdyn.',filea2F
-     open(filea2F, file=name, STATUS = 'unknown')
-     READ(filea2F,*) deg(isig), fermi(isig), dos_ee(isig)
-  enddo
-  ENDIF 
-  call mp_bcast(deg, ionode_id)
-  call mp_bcast(fermi, ionode_id)
-  call mp_bcast(dos_ee, ionode_id)
+     name= TRIM(elph_dir) // 'a2Fmatdyn.' // TRIM(int_to_char(filea2F))
+     IF (ionode) open(unit=filea2F, file=TRIM(name), &
+                                STATUS = 'unknown', IOSTAT=ios)
+      CALL mp_bcast(ios, ionode_id, intra_image_comm)
+      IF (ios /= 0) CALL errore('a2Fdos','problem opening file'//TRIM(name),1)
+      IF (ionode) &
+            READ(filea2F,*) deg(isig), fermi(isig), dos_ee(isig)
+  ENDDO
+  call mp_bcast(deg, ionode_id, intra_image_comm)
+  call mp_bcast(fermi, ionode_id, intra_image_comm)
+  call mp_bcast(dos_ee, ionode_id, intra_image_comm)
   !
   IF (ionode) THEN
-  IF(dos) then
-     open(400,file='lambda',status='unknown')
-     write(400,*)
-     write(400,*) ' Electron-phonon coupling constant, lambda '
-     write(400,*)
-  ELSE
-     open (20,file='gam.lines' ,status='unknown')
-     write(20,*)
-     write(20,*) ' Gamma lines for all modes [THz] '
-     write(20,*)
-     write(6,*)
-     write(6,*) ' Gamma lines for all modes [Rydberg] '
-     write(6,*)
-  ENDIF
+     IF(dos) then
+        open(unit=400,file='lambda',status='unknown')
+        write(400,*)
+        write(400,*) ' Electron-phonon coupling constant, lambda '
+        write(400,*)
+     ELSE
+        open (unit=20,file='gam.lines' ,status='unknown')
+        write(20,*)
+        write(20,*) ' Gamma lines for all modes [THz] '
+        write(20,*)
+        write(6,*)
+        write(6,*) ' Gamma lines for all modes [Rydberg] '
+        write(6,*)
+     ENDIF
   ENDIF
   !
   ALLOCATE ( frcg(nr1,nr2,nr3,3,3,nat,nat) )
@@ -1981,7 +2114,7 @@ SUBROUTINE a2Fdos &
         CALL set_asr (asr, nr1, nr2, nr3, frcg, zeu, nat_blk, ibrav, tau_blk)
      endif
      !
-     IF (ionode) open(300,file='dyna2F',status='old')
+     IF (ionode) open(unit=300,file='dyna2F',status='old')
      !
      do n = 1 ,nq
         gam(:,:,:,:) = (0.d0, 0.d0)
@@ -1991,12 +2124,12 @@ SUBROUTINE a2Fdos &
               read(300,*) (z(na,m),m=1,nmodes)
            end do ! na
         ENDIF
-        CALL mp_bcast(z, ionode_id)
+        CALL mp_bcast(z, ionode_id, world_comm)
         
         !
         CALL setgam (q(1,n), gam, nat, at, bg, tau, itau_blk, nsc, alat, &
              gam_blk, nat_blk, at_blk,bg_blk,tau_blk, omega_blk, &
-             frcg, nr1,nr2,nr3, rws, nrws)
+             frcg, nr1,nr2,nr3, rws, nrws, fd)
         !
         ! here multiply dyn*gam*dyn for gamma and divide by w2 for lambda at given q
         !
@@ -2082,6 +2215,15 @@ SUBROUTINE a2Fdos &
            write( 6,'(3x,i5)') n
            write(20,'(9F8.4)')  (gamma(i,n)*RY_TO_THZ,i=1,3*nat)
            write( 6,'(6F12.9)') (gamma(i,n),i=1,3*nat)
+!
+!   write also in a format that can be read by plotband
+!
+           WRITE(200+isig, '(10x,3f10.6)')  q(1,n), q(2,n), q(3,n)
+!
+!     output in GHz
+!
+           WRITE(200+isig, '(6f10.4)') (gamma(nu,n)*RY_TO_THZ*1000.0_DP, &
+                                        nu=1,3*nat)
         end do
      endif
      !
@@ -2101,7 +2243,7 @@ END SUBROUTINE a2Fdos
 !-----------------------------------------------------------------------
 subroutine setgam (q, gam, nat, at,bg,tau,itau_blk,nsc,alat, &
      &             gam_blk, nat_blk,    at_blk,bg_blk,tau_blk,omega_blk, &
-     &             frcg, nr1,nr2,nr3, rws,nrws)
+     &             frcg, nr1,nr2,nr3, rws,nrws, fd)
   !-----------------------------------------------------------------------
   ! compute the dynamical matrix (the analytic part only)
   !
@@ -2116,8 +2258,9 @@ subroutine setgam (q, gam, nat, at,bg,tau,itau_blk,nsc,alat, &
   real(DP)       :: q(3), tau(3,nat), at(3,3), bg(3,3), alat, rws(0:3,nrws)
   real(DP)       :: tau_blk(3,nat_blk), at_blk(3,3), bg_blk(3,3), omega_blk, &
                     frcg(nr1,nr2,nr3,3,3,nat_blk,nat_blk)
-  COMPLEX(DP)  :: gam_blk(3,3,nat_blk,nat_blk)
+  COMPLEX(DP)  :: gam_blk(3,3,nat_blk,nat_blk),f_of_q(3,3,nat,nat)
   COMPLEX(DP) ::  gam(3,3,nat,nat)
+  LOGICAL :: fd
   !
   ! local variables
   !
@@ -2129,6 +2272,7 @@ subroutine setgam (q, gam, nat, at,bg,tau,itau_blk,nsc,alat, &
   !
   call q_gen(nsc,qbid,at_blk,bg_blk,at,bg)
   !
+  f_of_q=(0.0_DP,0.0_DP)
   do iq=1,nsc
      !
      do k=1,3
@@ -2137,7 +2281,7 @@ subroutine setgam (q, gam, nat, at,bg,tau,itau_blk,nsc,alat, &
      !
      gam_blk(:,:,:,:) = (0.d0,0.d0)
      CALL frc_blk (gam_blk,qp,tau_blk,nat_blk,              &
-                   nr1,nr2,nr3,frcg,at_blk,bg_blk,rws,nrws)
+                   nr1,nr2,nr3,frcg,at_blk,bg_blk,rws,nrws,f_of_q,fd)
      !
      do na=1,nat
         na_blk = itau_blk(na)
@@ -2301,6 +2445,7 @@ subroutine readfg ( ifn, nr1, nr2, nr3, nat, frcg )
   USE kinds,       ONLY : DP
   USE io_global,   ONLY : ionode, ionode_id, stdout
   USE mp,          ONLY : mp_bcast
+  USE mp_world,    ONLY : world_comm
   implicit none
   ! I/O variable
   integer, intent(in) ::  nr1,nr2,nr3, nat
@@ -2311,9 +2456,9 @@ subroutine readfg ( ifn, nr1, nr2, nr3, nat, frcg )
   !
   !
   IF (ionode) READ (ifn,*) m1, m2, m3
-  CALL mp_bcast(m1, ionode_id)
-  CALL mp_bcast(m2, ionode_id)
-  CALL mp_bcast(m3, ionode_id)
+  CALL mp_bcast(m1, ionode_id, world_comm)
+  CALL mp_bcast(m2, ionode_id, world_comm)
+  CALL mp_bcast(m3, ionode_id, world_comm)
   if ( m1 /= nr1 .or. m2 /= nr2 .or. m3 /= nr3) &
        call errore('readfg','inconsistent nr1, nr2, nr3 read',1)
   do i=1,3
@@ -2321,10 +2466,10 @@ subroutine readfg ( ifn, nr1, nr2, nr3, nat, frcg )
         do na=1,nat
            do nb=1,nat
               IF (ionode) read (ifn,*) ibid, jbid, nabid, nbbid
-              CALL mp_bcast(ibid, ionode_id)
-              CALL mp_bcast(jbid, ionode_id)
-              CALL mp_bcast(nabid, ionode_id)
-              CALL mp_bcast(nbbid, ionode_id)
+              CALL mp_bcast(ibid, ionode_id, world_comm)
+              CALL mp_bcast(jbid, ionode_id, world_comm)
+              CALL mp_bcast(nabid, ionode_id, world_comm)
+              CALL mp_bcast(nbbid, ionode_id, world_comm)
               
               if(i.ne.ibid.or.j.ne.jbid.or.na.ne.nabid.or.nb.ne.nbbid)  then
                   write(stdout,*) i,j,na,nb,'  <>  ', ibid, jbid, nabid, nbbid
@@ -2334,7 +2479,7 @@ subroutine readfg ( ifn, nr1, nr2, nr3, nat, frcg )
                                  frcg(m1,m2,m3,i,j,na,nb), &
                                  m1=1,nr1),m2=1,nr2),m3=1,nr3)
               endif
-              CALL mp_bcast(frcg(:,:,:,i,j,na,nb), ionode_id)
+              CALL mp_bcast(frcg(:,:,:,i,j,na,nb), ionode_id, world_comm)
            end do
         end do
      end do
@@ -2364,11 +2509,12 @@ SUBROUTINE find_representations_mode_q ( nat, ntyp, xq, w2, u, tau, ityp, &
   CHARACTER(15), INTENT(OUT) :: name_rap_mode(3*nat)
   INTEGER, INTENT(OUT) :: num_rap_mode(3*nat)
   REAL(DP) :: gi (3, 48), gimq (3), sr_is(3,3,48), rtau(3,48,nat)
-  INTEGER :: irotmq, nsymq, nsym_is, isym, i
+  INTEGER :: irotmq, nsymq, nsym_is, isym, i, ierr
   LOGICAL :: minus_q, search_sym, sym(48), magnetic_sym
 !
 !  find the small group of q
 !
+  time_reversal=.TRUE.
   IF (.NOT.time_reversal) minus_q=.FALSE.
 
   sym(1:nsym)=.true.
@@ -2397,9 +2543,9 @@ SUBROUTINE find_representations_mode_q ( nat, ntyp, xq, w2, u, tau, ityp, &
      CALL prepare_sym_analysis(nsymq,sr,t_rev,magnetic_sym)
      sym (1:nsym) = .TRUE.
      CALL sgam_ph_new (at, bg, nsym, s, irt, tau, rtau, nat)
-     CALL find_mode_sym (u, w2, at, bg, tau, nat, nsymq, sr, irt, xq, rtau, &
-                         amass, ntyp, ityp, 1, .FALSE., .FALSE., &
-                         nspin_mag, name_rap_mode, num_rap_mode)
+     CALL find_mode_sym_new (u, w2, tau, nat, nsymq, sr, irt, xq,    &
+             rtau, amass, ntyp, ityp, 1, .FALSE., .FALSE., num_rap_mode, ierr)
+
   ENDIF
   RETURN
   END SUBROUTINE find_representations_mode_q
