@@ -13,7 +13,7 @@ SUBROUTINE openfilq()
   ! ... calculation.
   !
   USE kinds,           ONLY : DP
-  USE control_flags,   ONLY : modenum
+  USE control_flags,   ONLY : io_level, modenum
   USE units_ph,        ONLY : iuwfc, iudwf, iubar, iucom, iudvkb3, &
                               iudrhous, iuebar, iudrho, iudyn, iudvscf, &
                               lrwfc, lrdwf, lrbar, lrcom, lrdvkb3, &
@@ -36,8 +36,9 @@ SUBROUTINE openfilq()
   USE noncollin_module,ONLY : npol, nspin_mag
   USE paw_variables,   ONLY : okpaw
   USE control_flags,   ONLY : twfcollect
-  USE mp_global,       ONLY : me_pool
+  USE mp_bands,        ONLY : me_bgrp
   USE io_global,       ONLY : ionode,stdout
+  USE buffers,         ONLY : open_buffer
   USE ramanm,          ONLY : lraman, elop, iuchf, iud2w, iuba2, lrchf, lrd2w, lrba2
   USE acfdtest,        ONLY : acfdt_is_active, acfdt_num_der
   USE input_parameters,ONLY : nk1, nk2, nk3
@@ -51,12 +52,12 @@ SUBROUTINE openfilq()
   ! integer variable for I/O control
   CHARACTER (len=256) :: filint, fildvscf_rot
   ! the name of the file
-  LOGICAL :: exst
-  ! logical variable to check file existe
+  LOGICAL :: exst, exst_mem
+  ! logical variable to check file exists
+  ! logical variable to check file exists in memory
   !
   REAL(DP) :: edum(1,1), wdum(1,1)
   INTEGER :: ndr, ierr, iq_dummy
-  INTEGER, EXTERNAL :: find_free_unit
   !
   !
   IF (LEN_TRIM(prefix) == 0) CALL errore ('openfilq', 'wrong prefix', 1)
@@ -79,9 +80,9 @@ SUBROUTINE openfilq()
   ENDIF
 !!!!!!!!!!!!!!!!!!!!!!!! END OF ACFDT TEST !!!!!!!!!!!!!!!!
   iuwfc = 20
-  lrwfc = 2 * nbnd * npwx * npol
-  CALL diropn (iuwfc, 'wfc', lrwfc, exst)
-  IF (.NOT.exst.and..not.all_done) THEN
+  lrwfc = nbnd * npwx * npol
+  CALL open_buffer (iuwfc, 'wfc', lrwfc, io_level, exst_mem, exst, tmp_dir)
+  IF (.NOT.exst.AND..NOT.exst_mem.and..not.all_done) THEN
      CALL errore ('openfilq', 'file '//trim(prefix)//'.wfc not found', 1)
   END IF
   IF (elph_mat) then
@@ -102,16 +103,16 @@ SUBROUTINE openfilq()
   !    The file with deltaV_{bare} * psi
   !
   iubar = 21
-  lrbar = 2 * nbnd * npwx * npol
-  CALL diropn (iubar, 'bar', lrbar, exst)
+  lrbar = nbnd * npwx * npol
+  CALL open_buffer (iubar, 'bar', lrbar, io_level, exst_mem, exst, tmp_dir)
   IF (ext_recover.AND..NOT.exst) &
      CALL errore ('openfilq','file '//trim(prefix)//'.bar not found', 1)
   !
   !    The file with the solution delta psi
   !
   iudwf = 22
-  lrdwf = 2 * nbnd * npwx * npol
-  CALL diropn (iudwf, 'dwf', lrdwf, exst)
+  lrdwf =  nbnd * npwx * npol
+  CALL open_buffer (iudwf, 'dwf', lrdwf, io_level, exst_mem, exst, tmp_dir)
   IF (ext_recover.AND..NOT.exst) &
      CALL errore ('openfilq','file '//trim(prefix)//'.dwf not found', 1)
   !
@@ -119,8 +120,8 @@ SUBROUTINE openfilq()
   !
   IF (okvan) THEN
      iudrhous = 25
-     lrdrhous = 2 * dfftp%nnr * nspin_mag
-     CALL diropn (iudrhous, 'prd', lrdrhous, exst)
+     lrdrhous =  dfftp%nnr * nspin_mag
+     CALL open_buffer (iudrhous, 'prd', lrdrhous, io_level, exst_mem, exst, tmp_dir)
      IF (ext_recover.AND..NOT.exst) &
         CALL errore ('openfilq','file '//trim(prefix)//'.prd not found', 1)
   ENDIF
@@ -164,29 +165,25 @@ SUBROUTINE openfilq()
   !
 400 IF (trim(fildvscf).NE.' ') THEN
      iudvscf = 27
-     IF ( me_pool == 0 ) THEN
-           IF(trim(dvscf_star%ext).NE.' ' .and. elph_mat) THEN
-              fildvscf_rot = dfile_name(xq, at, TRIM(dvscf_star%ext), &
+     IF ( me_bgrp == 0 ) THEN
+        IF (trim(dvscf_star%ext).NE.' ' .and. elph_mat) THEN
+           fildvscf_rot = dfile_name(xq, at, TRIM(dvscf_star%ext), &
                    TRIM(dvscf_star%dir)//prefix, &
                    generate=.false., index_q=iq_dummy, equiv=.false. )
               
-              WRITE(stdout,'(5x,5a)') "Opening dvscf file '",TRIM(fildvscf_rot), &
+           WRITE(stdout,'(5x,5a)') "Opening dvscf file '",TRIM(fildvscf_rot), &
                    "' (for reading) in directory '",trim(dvscf_star%dir),"'"
               
-              CALL diropn (iudvscf, fildvscf_rot, lrdrho, exst, dvscf_star%dir)
-           ELSE
-              CALL diropn (iudvscf, fildvscf, lrdrho, exst )
-           ENDIF
-           IF (okpaw) THEN
-              filint=TRIM(fildvscf)//'_paw'
-              lint3paw = 2 * nhm * nhm * 3 * nat * nspin_mag
-              iuint3paw=34
-              !           IF(dvscf_dir.NE.' ') then
-              !              CALL diropn (iuint3paw, filint, lint3paw, exst, dvscf_dir)
-              !           ELSE
-              CALL diropn (iuint3paw, filint, lint3paw, exst)
-              !           ENDIF
-           ENDIF
+           CALL diropn (iudvscf, fildvscf_rot, lrdrho, exst, dvscf_star%dir)
+        ELSE
+           CALL diropn (iudvscf, fildvscf, lrdrho, exst )
+        ENDIF
+        IF (okpaw) THEN
+           filint=TRIM(fildvscf)//'_paw'
+           lint3paw = 2 * nhm * nhm * nat * nspin_mag
+           iuint3paw=34
+           CALL diropn (iuint3paw, filint, lint3paw, exst)
+        ENDIF
         END IF
      END IF
   !
@@ -196,8 +193,8 @@ SUBROUTINE openfilq()
   !
   IF (okvan .AND. (epsil .OR. zue)) THEN
      iucom = 28
-     lrcom = 2 * nbnd * npwx * npol
-     CALL diropn (iucom, 'com', lrcom, exst)
+     lrcom = nbnd * npwx * npol
+     CALL open_buffer (iucom, 'com', lrcom, io_level, exst_mem, exst, tmp_dir)
      IF (ext_recover.AND..NOT.exst) &
          CALL errore ('openfilq', 'file '//trim(prefix)//'.com not found', 1)
   !
@@ -212,8 +209,8 @@ SUBROUTINE openfilq()
   ENDIF
   IF (epsil .OR. zue) THEN
      iuebar = 30
-     lrebar = 2 * nbnd * npwx * npol
-     CALL diropn (iuebar, 'ebar', lrebar, exst)
+     lrebar =  nbnd * npwx * npol
+     CALL open_buffer (iuebar, 'ebar', lrebar, io_level, exst_mem, exst, tmp_dir)
      IF (ext_recover.AND..NOT.exst) &
         CALL errore ('openfilq','file '//trim(prefix)//'.ebar not found', 1)
   ENDIF
