@@ -39,6 +39,7 @@ SUBROUTINE phq_init()
   USE gvect,                ONLY : g, ngm
   USE klist,                ONLY : xk
   USE lsda_mod,             ONLY : lsda, current_spin, isk
+  USE buffers,              ONLY : get_buffer
   USE io_global,            ONLY : stdout
   USE io_files,             ONLY : iunigk
   USE atom,                 ONLY : msh, rgrid
@@ -56,11 +57,12 @@ SUBROUTINE phq_init()
   USE units_ph,             ONLY : lrwfc, iuwfc
   USE qpoint,               ONLY : xq, igkq, npwq, nksq, eigqts, ikks, ikqs
 
-  USE mp_global,           ONLY : intra_pool_comm
+  USE mp_bands,            ONLY : intra_bgrp_comm
   USE mp,                  ONLY : mp_sum
   USE acfdtest,            ONLY : acfdt_is_active, acfdt_num_der
   USE el_phon,             ONLY : elph_mat, iunwfcwann, npwq_refolded, &
                            kpq,g_kpq,igqg,xk_gamma, lrwfcr
+  USE wannier_gw,           ONLY : l_head
   !
   IMPLICIT NONE
   !
@@ -81,7 +83,6 @@ SUBROUTINE phq_init()
   COMPLEX(DP), ALLOCATABLE :: aux1(:,:)
     ! used to compute alphap
   COMPLEX(DP), EXTERNAL :: zdotc
-
   !
   !
   IF (all_done) RETURN
@@ -193,7 +194,7 @@ SUBROUTINE phq_init()
        call read_wfc_rspace_and_fwfft( evc , ik , lrwfcr , iunwfcwann , npw , igk )
 !       CALL davcio (evc, lrwfc, iunwfcwann, ik, - 1)
     else
-       CALL davcio( evc, lrwfc, iuwfc, ikk, -1 )
+       CALL get_buffer( evc, lrwfc, iuwfc, ikk )
     endif
      !
      ! ... e) we compute the becp terms which are used in the rest of
@@ -228,15 +229,15 @@ SUBROUTINE phq_init()
   IF (acfdt_is_active) THEN
      ! ACFDT -test always read calculated wcf from non_scf calculation
      IF(acfdt_num_der) then 
-       CALL davcio( evq, lrwfc, iuwfc, ikq, -1 )
+       CALL get_buffer( evq, lrwfc, iuwfc, ikq )
      ELSE
        IF ( .NOT. lgamma ) &
-          CALL davcio( evq, lrwfc, iuwfc, ikq, -1 )
+          CALL get_buffer( evq, lrwfc, iuwfc, ikq )
      ENDIF
   ELSE
      ! this is the standard treatment
      IF ( .NOT. lgamma .and..not. elph_mat )then 
-        CALL davcio( evq, lrwfc, iuwfc, ikq, -1 )
+        CALL get_buffer( evq, lrwfc, iuwfc, ikq )
      ELSEIF(.NOT. lgamma .and. elph_mat) then
         !
         ! I read the wavefunction in real space and fwfft it
@@ -273,9 +274,7 @@ SUBROUTINE phq_init()
      END DO
      !
   END DO
-#ifdef __MPI
-     CALL mp_sum ( eprec, intra_pool_comm )
-#endif
+  CALL mp_sum ( eprec, intra_bgrp_comm )
   !
   DEALLOCATE( aux1 )
      
@@ -283,7 +282,7 @@ SUBROUTINE phq_init()
   CALL dvanqq()
   CALL drho()
   !
-  IF ( ( epsil .OR. zue ) .AND. okvan ) THEN
+  IF ( ( epsil .OR. zue .OR. l_head) .AND. okvan ) THEN
      CALL compute_qdipol(dpqq)
      IF (lspinorb) CALL compute_qdipol_so(dpqq, dpqq_so)
      CALL qdipol_cryst()
