@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2011 PWSCF group
+! Copyright (C) 2001-2013 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -19,17 +19,18 @@ SUBROUTINE phq_readin()
   USE kinds,         ONLY : DP
   USE parameters,    ONLY : nsx
   USE ions_base,     ONLY : nat, ntyp => nsp
-  USE io_global,     ONLY : ionode_id
   USE mp,            ONLY : mp_bcast
+  USE mp_world,      ONLY : world_comm
   USE ions_base,     ONLY : amass, atm
   USE input_parameters, ONLY : max_seconds, nk1, nk2, nk3, k1, k2, k3
   USE start_k,       ONLY : reset_grid
   USE klist,         ONLY : xk, nks, nkstot, lgauss, two_fermi_energies, lgauss
   USE ktetra,        ONLY : ltetra
-  USE control_flags, ONLY : gamma_only, tqr, restart, lkpoint_dir
+  USE control_flags, ONLY : gamma_only, tqr, restart, lkpoint_dir, io_level
   USE uspp,          ONLY : okvan
   USE fixed_occ,     ONLY : tfixed_occ
   USE lsda_mod,      ONLY : lsda, nspin
+  USE fft_base,      ONLY : dffts
   USE spin_orb,      ONLY : domag
   USE cellmd,        ONLY : lmovecell
   USE run_info, ONLY : title
@@ -39,26 +40,31 @@ SUBROUTINE phq_readin()
                             nmix_ph, ldisp, recover, lrpa, lnoloc, start_irr, &
                             last_irr, start_q, last_q, current_iq, tmp_dir_ph, &
                             ext_recover, ext_restart, u_from_file, ldiag, &
-                            search_sym, lqdir, electron_phonon
-  USE save_ph,       ONLY : tmp_dir_save
+                            search_sym, lqdir, electron_phonon, tmp_dir_phq, &
+                            rec_code_read, qplot, only_init, only_wfc, &
+                            low_directory_check
+
+  USE save_ph,       ONLY : tmp_dir_save, save_ph_input_variables
   USE gamma_gamma,   ONLY : asr
   USE qpoint,        ONLY : nksq, xq
   USE partial,       ONLY : atomo, nat_todo, nat_todo_input
   USE output,        ONLY : fildyn, fildvscf, fildrho
-  USE disp,          ONLY : nq1, nq2, nq3
-  USE io_files,      ONLY : tmp_dir, prefix
+  USE disp,          ONLY : nq1, nq2, nq3, x_q, wq, nqs, lgamma_iq
+  USE io_files,      ONLY : outdir, tmp_dir, prefix
   USE noncollin_module, ONLY : i_cons, noncolin
   USE ldaU,          ONLY : lda_plus_u
   USE control_flags, ONLY : iverbosity, modenum, twfcollect
-  USE io_global,     ONLY : ionode, stdout
-  USE mp_global,     ONLY : nproc_pool, nproc_pool_file, &
-                            nimage, my_image_id,    &
-                            nproc_image_file, nproc_image, npool, &
-                            get_ntask_groups, ntask_groups_file,  &
-                            nproc_bgrp_file
+  USE io_global,     ONLY : meta_ionode, meta_ionode_id, ionode, ionode_id, stdout
+  USE mp_images,     ONLY : nimage, my_image_id, intra_image_comm,   &
+                            me_image, nproc_image
+  USE mp_global,     ONLY : nproc_pool_file, &
+                            nproc_bgrp_file, nproc_image_file
+  USE mp_pools,      ONLY : nproc_pool, npool 
+  USE mp_bands,      ONLY : nproc_bgrp, ntask_groups
   USE paw_variables, ONLY : okpaw
   USE ramanm,        ONLY : eth_rps, eth_ns, lraman, elop, dek
-  USE freq_ph,       ONLY : fpol, fiu, nfs, nfsmax
+  USE freq_ph,       ONLY : fpol, fiu, nfs
+  USE cryst_ph,      ONLY : magnetic_sym
   USE ph_restart,    ONLY : ph_readfile
   USE xml_io_base,   ONLY : create_directory
   USE el_phon,       ONLY : elph,elph_mat,elph_simple,elph_nbnd_min, elph_nbnd_max, &
@@ -69,7 +75,7 @@ SUBROUTINE phq_readin()
   !
   CHARACTER(LEN=256), EXTERNAL :: trimcheck
   !
-  INTEGER :: ios, ipol, iter, na, it, ierr
+  INTEGER :: ios, ipol, iter, na, it, ierr, ierr1
     ! integer variable for I/O control
     ! counter on polarizations
     ! counter on iterations
@@ -77,17 +83,22 @@ SUBROUTINE phq_readin()
     ! counter on types
   REAL(DP) :: amass_input(nsx)
     ! save masses read from input here
-  CHARACTER (LEN=256) :: outdir
+  CHARACTER (LEN=256) :: filename
   !
   CHARACTER(LEN=80)          :: card
   CHARACTER(LEN=1), EXTERNAL :: capital
   CHARACTER(LEN=6) :: int_to_char
   INTEGER                    :: i
   LOGICAL                    :: nogg
+  LOGICAL      :: q2d, q_in_band_form
   INTEGER, EXTERNAL  :: atomic_number
   REAL(DP), EXTERNAL :: atom_weight
   LOGICAL, EXTERNAL  :: imatches
   LOGICAL, EXTERNAL  :: has_xml
+  LOGICAL :: exst, parallelfs
+  REAL(DP), ALLOCATABLE :: xqaux(:,:)
+  INTEGER, ALLOCATABLE :: wqaux(:)
+  INTEGER :: nqaux, iq
   !
   NAMELIST / INPUTPH / tr2_ph, amass, alpha_mix, niter_ph, nmix_ph,  &
                        nat_todo, iverbosity, outdir, epsil,  &
@@ -98,9 +109,10 @@ SUBROUTINE phq_readin()
                        fpol, asr, lrpa, lnoloc, start_irr, last_irr, &
                        start_q, last_q, nogg, ldiag, search_sym, lqdir, &
                        nk1, nk2, nk3, k1, k2, k3, &
-                       drho_star, dvscf_star, &
+                       drho_star, dvscf_star, only_init, only_wfc, &
                        elph_nbnd_min, elph_nbnd_max, el_ph_ngauss,el_ph_nsigma, el_ph_sigma,  &
-                       electron_phonon
+                       electron_phonon, &
+                       q_in_band_form, q2d, qplot, low_directory_check
 
   ! tr2_ph       : convergence threshold
   ! amass        : atomic masses
@@ -144,8 +156,9 @@ SUBROUTINE phq_readin()
   ! ik1, ik2, ik3: when specified in input it uses for the phonon run
   !                a different mesh than that used for the charge density.
   !
-  ! dvscf_star%open : if .true. write in dvscf_star%dir the dvscf_q' for all q' in the
-  !                   star of q with suffix dvscf_star%ext. The dvscf_q' is written in the basis dvscf_star%basis;
+  ! dvscf_star%open : if .true. write in dvscf_star%dir the dvscf_q
+  !                   'for all q' in the star of q with suffix dvscf_star%ext. 
+  !                   The dvscf_q' is written in the basis dvscf_star%basis;
   !                   if dvscf_star%pat is .true. also save a pattern file.
   ! dvscf_star%dir, dvscf_star%ext, dvscf_star%basis : see dvscf_star%open
   ! drho_star%open  : like dvscf_star%open but for drho_q
@@ -158,9 +171,18 @@ SUBROUTINE phq_readin()
   ! el_ph_nsigma, 
   ! el_ph_sigma  :  if (elph_mat=.true.) it defines the kind and the val-ue of the
   !                 smearing to be used in the eph coupling calculation.
-  !             
-
-  IF (ionode) THEN
+  ! qplot, : if true a list of q points is given in input
+  ! q_in_band_form: if true the input list of q points defines paths 
+  ! q2d, : if .true. the q list define a mesh in a square.
+  ! low_directory_check : if .true. only the requested representations
+  !                       are searched on file
+  ! 
+  ! Note: meta_ionode is a single processor that reads the input
+  !       (ionode is also a single processor but per image)
+  !       Data read from input is subsequently broadcast to all processors
+  !       from ionode_id (using the default communicator world_comm)
+  !
+  IF (meta_ionode) THEN
   !
   ! ... Input from file ?
   !
@@ -172,17 +194,17 @@ SUBROUTINE phq_readin()
   !
   ENDIF
   !
-  CALL mp_bcast(ios, ionode_id )
+  CALL mp_bcast(ios, meta_ionode_id, world_comm )
   CALL errore( 'phq_readin', 'reading title ', ABS( ios ) )
-  CALL mp_bcast(title, ionode_id )
+  CALL mp_bcast(title, meta_ionode_id, world_comm  )
   !
   ! Rewind the input if the title is actually the beginning of inputph namelist
   !
   IF( imatches("&inputph", title) ) THEN
     WRITE(*, '(6x,a)') "Title line not specified: using 'default'."
     title='default'
-    IF (ionode) REWIND(5, iostat=ios)
-    CALL mp_bcast(ios, ionode_id )
+    IF (meta_ionode) REWIND(5, iostat=ios)
+    CALL mp_bcast(ios, meta_ionode_id, world_comm  )
     CALL errore('phq_readin', 'Title line missing from input.', abs(ios))
   ENDIF
   !
@@ -216,8 +238,10 @@ SUBROUTINE phq_readin()
   elop         = .FALSE.
   max_seconds  =  1.E+7_DP
   reduce_io    = .FALSE.
-  CALL get_env( 'ESPRESSO_TMPDIR', outdir )
-  IF ( TRIM( outdir ) == ' ' ) outdir = './'
+  IF ( TRIM(outdir) == './') THEN
+     CALL get_env( 'ESPRESSO_TMPDIR', outdir )
+     IF ( TRIM( outdir ) == ' ' ) outdir = './'
+  ENDIF
   prefix       = 'pwscf'
   fildyn       = 'matdyn'
   fildrho      = ' '
@@ -236,6 +260,12 @@ SUBROUTINE phq_readin()
   last_q       =-1000
   ldiag        =.FALSE.
   lqdir        =.FALSE.
+  qplot        =.FALSE.
+  q_in_band_form=.FALSE.
+  q2d         = .FALSE.
+  only_init  = .FALSE.
+  only_wfc    = .FALSE.
+  low_directory_check=.FALSE.
   search_sym   =.TRUE.
   nk1       = 0
   nk2       = 0
@@ -262,21 +292,23 @@ SUBROUTINE phq_readin()
   !
   ! ...  reading the namelist inputph
   !
-  IF (ionode) READ( 5, INPUTPH, IOSTAT = ios )
-  !
-  CALL mp_bcast(ios, ionode_id)
+  IF (meta_ionode) READ( 5, INPUTPH, ERR=30, IOSTAT = ios )
+30  CALL mp_bcast(ios, meta_ionode_id, world_comm )
   CALL errore( 'phq_readin', 'reading inputph namelist', ABS( ios ) )
   !
-  IF (ionode) tmp_dir = trimcheck (outdir)
-
+  ! ...  broadcast all input variables
+  !
+  CALL bcast_ph_input ( )
+  CALL mp_bcast(nogg, meta_ionode_id, world_comm  )
+  CALL mp_bcast(q2d, meta_ionode_id, world_comm  )
+  CALL mp_bcast(q_in_band_form, meta_ionode_id, world_comm  )
+  !
+  tmp_dir = trimcheck (outdir)
   drho_star%dir=trimcheck(drho_star%dir)
   dvscf_star%dir=trimcheck(dvscf_star%dir)
   ! filename for the star must always be automatically generated:
   IF(drho_star%ext(1:5)/='auto:')  drho_star%ext  = 'auto:'//drho_star%ext
   IF(dvscf_star%ext(1:5)/='auto:') dvscf_star%ext = 'auto:'//dvscf_star%ext
-
-  CALL bcast_ph_input ( )
-  CALL mp_bcast(nogg, ionode_id )
   !
   ! ... Check all namelist variables
   !
@@ -288,6 +320,8 @@ SUBROUTINE phq_readin()
      IF (alpha_mix (iter) .LT.0.D0.OR.alpha_mix (iter) .GT.1.D0) CALL &
           errore ('phq_readin', ' Wrong alpha_mix ', iter)
   ENDDO
+  IF (qplot.AND..NOT.ldisp) CALL errore('phq_readin', &
+                                        'qplot requires ldisp=.true.',1)
   IF (niter_ph.LT.1.OR.niter_ph.GT.maxter) CALL errore ('phq_readin', &
        ' Wrong niter_ph ', 1)
   IF (nmix_ph.LT.1.OR.nmix_ph.GT.5) CALL errore ('phq_readin', ' Wrong &
@@ -297,11 +331,14 @@ SUBROUTINE phq_readin()
   IF (fildyn.EQ.' ') CALL errore ('phq_readin', ' Wrong fildyn ', 1)
   IF (max_seconds.LT.0.1D0) CALL errore ('phq_readin', ' Wrong max_seconds', 1)
 
+  IF (only_init.AND.only_wfc) CALL errore('phq_readin', &
+                        'only_init or only_wfc can be .true.', 1)
+
   IF (modenum < 0) CALL errore ('phq_readin', ' Wrong modenum ', 1)
   IF (dek <= 0.d0) CALL errore ( 'phq_readin', ' Wrong dek ', 1)
   !
   SELECT CASE( trim( electron_phonon ) )
-  CASE( 'simple' )
+  CASE( 'simple'  )
      elph=.true.
      elph_mat=.false.
      elph_simple=.true. 
@@ -319,17 +356,22 @@ SUBROUTINE phq_readin()
      elph_mat=.false.
      elph_simple=.false.
   END SELECT
+  IF (elph.AND.qplot) &
+     CALL errore('phq_readin', 'qplot and elph not implemented',1)
+
+  IF (ldisp.AND.only_init.AND.(.NOT.lqdir)) &
+     CALL errore('phq_readin', &
+                 'only_init=.TRUE. requires lqdir=.TRUE. or data are lost',1)
 
   epsil = epsil .OR. lraman .OR. elop
 
   IF (modenum /= 0) search_sym=.FALSE.
   
-  if(elph_simple.or.elph_mat) then
+  IF (elph_mat) THEN
      trans=.false.
-  else
+  ELSEIF (.NOT. elph) THEN
      trans = trans .OR. ldisp
-  endif
-   
+  ENDIF
   !
   ! Set default value for fildrho and fildvscf if they are required
   IF ( (lraman.OR.elop.OR.drho_star%open) .AND. fildrho == ' ') fildrho = 'drho'
@@ -343,17 +385,36 @@ SUBROUTINE phq_readin()
   !
   !    reads the q point (just if ldisp = .false.)
   !
-  IF (ionode) THEN
-     IF (.NOT. ldisp) &
-        READ (5, *, iostat = ios) (xq (ipol), ipol = 1, 3)
+  IF (meta_ionode) THEN
+     IF (qplot) THEN
+        READ (5, *, iostat = ios) nqaux
+     ELSE
+        IF (.NOT. ldisp) READ (5, *, iostat = ios) (xq (ipol), ipol = 1, 3)
+     ENDIF
   END IF
-  CALL mp_bcast(ios, ionode_id)
+  CALL mp_bcast(ios, meta_ionode_id, world_comm )
   CALL errore ('phq_readin', 'reading xq', ABS (ios) )
-  CALL mp_bcast(xq, ionode_id )
+  IF (qplot) THEN
+     CALL mp_bcast(nqaux, meta_ionode_id, world_comm )
+     ALLOCATE(xqaux(3,nqaux))
+     ALLOCATE(wqaux(nqaux))
+     IF (meta_ionode) THEN
+        DO iq=1, nqaux
+           READ (5, *, iostat = ios) (xqaux (ipol,iq), ipol = 1, 3), wqaux(iq)
+        ENDDO
+     ENDIF
+     CALL mp_bcast(ios, meta_ionode_id, world_comm )
+     CALL errore ('phq_readin', 'reading xq', ABS (ios) )
+     CALL mp_bcast(xqaux, meta_ionode_id, world_comm )
+     CALL mp_bcast(wqaux, meta_ionode_id, world_comm )
+  ELSE
+     CALL mp_bcast(xq, meta_ionode_id, world_comm  )
+  ENDIF
+  
   IF (.NOT.ldisp) THEN
      lgamma = xq (1) .EQ.0.D0.AND.xq (2) .EQ.0.D0.AND.xq (3) .EQ.0.D0
-     IF ( (epsil.OR.zue) .AND..NOT.lgamma) CALL errore ('phq_readin', &
-          'gamma is needed for elec.field', 1)
+     IF ( (epsil.OR.zue.or.lraman.or.elop) .AND..NOT.lgamma) &
+                CALL errore ('phq_readin', 'gamma is needed for elec.field', 1)
   ENDIF
   IF (zue.AND..NOT.trans) CALL errore ('phq_readin', 'trans must be &
        &.t. for Zue calc.', 1)
@@ -372,7 +433,7 @@ SUBROUTINE phq_readin()
      IF ( .NOT. epsil) CALL errore ('phq_readin', &
                                     'fpol=.TRUE. needs epsil=.TRUE.', 1 )
      nfs=0
-     IF (ionode) THEN
+     IF (meta_ionode) THEN
         READ (5, *, iostat = ios) card
         IF ( TRIM(card)=='FREQUENCIES'.OR. &
              TRIM(card)=='frequencies'.OR. &
@@ -380,12 +441,12 @@ SUBROUTINE phq_readin()
            READ (5, *, iostat = ios) nfs
         ENDIF
      ENDIF
-     CALL mp_bcast(ios, ionode_id )
+     CALL mp_bcast(ios, meta_ionode_id, world_comm  )
      CALL errore ('phq_readin', 'reading number of FREQUENCIES', ABS(ios) )
-     CALL mp_bcast(nfs, ionode_id )
-     if (nfs > nfsmax) call errore('phq_readin','Too many frequencies',1)
+     CALL mp_bcast(nfs, meta_ionode_id, world_comm  )
      if (nfs < 1) call errore('phq_readin','Too few frequencies',1)
-     IF (ionode) THEN
+     ALLOCATE(fiu(nfs))
+     IF (meta_ionode) THEN
         IF ( TRIM(card) == 'FREQUENCIES' .OR. &
              TRIM(card) == 'frequencies' .OR. &
              TRIM(card) == 'Frequencies' ) THEN
@@ -394,14 +455,14 @@ SUBROUTINE phq_readin()
            END DO
         END IF
      END IF
-     CALL mp_bcast(ios, ionode_id)
+     CALL mp_bcast(ios, meta_ionode_id, world_comm )
      CALL errore ('phq_readin', 'reading FREQUENCIES card', ABS(ios) )
-     CALL mp_bcast(fiu, ionode_id )
+     CALL mp_bcast(fiu, meta_ionode_id, world_comm  )
   ELSE
-     nfs=0
+     nfs=1
+     ALLOCATE(fiu(1))
      fiu=0.0_DP
   END IF
-
   !
   !
   !   Here we finished the reading of the input file.
@@ -414,31 +475,91 @@ SUBROUTINE phq_readin()
   !
   tmp_dir_save=tmp_dir
   tmp_dir_ph= TRIM (tmp_dir) // '_ph' // TRIM(int_to_char(my_image_id)) //'/'
-  CALL create_directory(tmp_dir_ph)
+  CALL check_tempdir ( tmp_dir_ph, exst, parallelfs )
+  tmp_dir_phq=tmp_dir_ph
 
   ext_restart=.FALSE.
   ext_recover=.FALSE.
+  rec_code_read=-1000
   IF (recover) THEN
-     CALL ph_readfile('init',ierr)
-     IF (ierr /= 0 ) THEN
+!
+!    With a recover run we read here the mesh of q points, the current iq,
+!    and the current frequency
+!
+     CALL ph_readfile('init', 0, 0, ierr)
+     CALL ph_readfile('status_ph', 0, 0, ierr1)
+!
+!   If some error occured here, we cannot recover the run
+!
+     IF (ierr /= 0 .OR. ierr1 /= 0 ) THEN
+        write(stdout,'(5x,"Run is not recoverable starting from scratch")')
         recover=.FALSE.
         goto 1001
      ENDIF
-     tmp_dir=tmp_dir_ph
-     CALL check_restart_recover(ext_recover, ext_restart)
-     tmp_dir=tmp_dir_save
-     IF (ldisp) lgamma = (current_iq==1)
 !
-!  If there is a restart or a recover file ph.x has saved its own data-file
-!  and we read the initial information from that file
+!   We check if the bands and the information on the pw run are in the directory
+!   written by the phonon code for the current q point. If the file exists
+!   we read from there, otherwise use the information in outdir.
 !
-     IF ((ext_recover.OR.ext_restart).AND..NOT.lgamma) &
-                                                      tmp_dir=tmp_dir_ph
+     IF (lqdir) THEN
+        tmp_dir_phq= TRIM (tmp_dir_ph) //TRIM(prefix)//&
+                          & '.q_' // TRIM(int_to_char(current_iq))//'/'
+        CALL check_restart_recover(ext_recover, ext_restart)
+        IF (.NOT.ext_recover.AND..NOT.ext_restart) tmp_dir_phq=tmp_dir_ph
+     ENDIF
+     !
+     filename=TRIM(tmp_dir_phq)//TRIM(prefix)//'.save/data-file.xml'
+     IF (ionode) inquire (file =TRIM(filename), exist = exst)
+     !
+     CALL mp_bcast( exst, ionode_id, intra_image_comm )
+     !
+     !  If this file exist we use it to recover the pw.x informations
+     !
+     IF (exst) tmp_dir=tmp_dir_phq
      u_from_file=.true.
   ENDIF
 1001 CONTINUE
 
+  IF (qplot.AND..NOT.recover) THEN
+     IF (q2d) THEN
+        nqs=wqaux(2)*wqaux(3)
+        ALLOCATE(x_q(3,nqs))
+        ALLOCATE(wq(nqs))
+        CALL generate_k_in_plane(nqaux, xqaux, wqaux, x_q, wq, nqs)
+     ELSEIF (q_in_band_form) THEN
+        nqs=SUM(wqaux(1:nqaux-1))+1
+        DO i=1,nqaux-1
+           IF (wqaux(i)==0) nqs=nqs+1
+        ENDDO
+        ALLOCATE(x_q(3,nqs))
+        ALLOCATE(wq(nqs))
+        CALL generate_k_along_lines(nqaux, xqaux, wqaux, x_q, wq, nqs)
+     ELSE
+        nqs=nqaux
+        ALLOCATE(x_q(3,nqs))
+        ALLOCATE(wq(nqs))
+        wq(:)=wqaux(:)
+        x_q(:,1:nqs)=xqaux(:,1:nqs)  
+     ENDIF
+     DEALLOCATE(xqaux)
+     DEALLOCATE(wqaux)
+     ALLOCATE(lgamma_iq(nqs))
+     DO iq=1, nqs
+        lgamma_iq(iq)= ( ABS(x_q(1,iq)) .LT. 1.0e-10_dp ) .AND. &
+                       ( ABS(x_q(2,iq)) .LT. 1.0e-10_dp ) .AND. &
+                       ( ABS(x_q(3,iq)) .LT. 1.0e-10_dp )
+     ENDDO
+     WRITE(stdout, '(//5x,"Dynamical matrices for q-points given in input")') 
+     WRITE(stdout, '(5x,"(",i4,"q-points):")') nqs
+     WRITE(stdout, '(5x,"  N         xq(1)         xq(2)         xq(3) " )')
+     DO iq = 1, nqs
+        WRITE(stdout, '(5x,i3, 3f14.9)') iq, x_q(1,iq), x_q(2,iq), x_q(3,iq)
+     END DO
+  ENDIF
+
   CALL read_file ( )
+
+  magnetic_sym=noncolin .AND. domag
   !
   ! init_start_grid returns .true. if a new k-point grid is set from values
   ! read from input (this happens if nk1*nk2*nk3, else it returns .false.,
@@ -471,6 +592,8 @@ SUBROUTINE phq_readin()
   IF (lmovecell) CALL errore('phq_readin', &
       'The phonon code is not working after vc-relax',1)
 
+  IF (reduce_io) io_level=0
+
   IF (nproc_image /= nproc_image_file .and. .not. twfcollect)  &
      CALL errore('phq_readin',&
      'pw.x run with a different number of processors. Use wf_collect=.true.',1)
@@ -478,21 +601,22 @@ SUBROUTINE phq_readin()
   IF (nproc_pool /= nproc_pool_file .and. .not. twfcollect)  &
      CALL errore('phq_readin',&
      'pw.x run with a different number of pools. Use wf_collect=.true.',1)
+  !
+  !   Task groups not used in phonon. Activated only in some places
+  !
+  IF (ntask_groups > 1) dffts%have_task_groups=.FALSE.
 
-  IF (get_ntask_groups() > 1) &
-     CALL errore('phq_readin','task_groups not available in phonon',1)
-
-  IF (nproc_bgrp_file /= nproc_pool_file) &
-     CALL errore('phq_readin','band parallelization not available in phonon',1)
-
-  IF (elph.and.nimage>1) CALL errore('phq_readin',&
-       'el-ph with image parallelization is not yet available',1)
-
+  IF (nproc_bgrp_file /= nproc_bgrp .AND. .NOT. twfcollect) &
+     CALL errore('phq_readin','pw.x run with different band parallelization',1)
+  
   if(elph_mat.and.fildvscf.eq.' ') call errore('phq_readin',&
        'el-ph with wannier requires fildvscf',1)
 
   IF(elph_mat.and.npool.ne.1) call errore('phq_readin',&
        'el-ph with wannier : pools not implemented',1)
+
+  IF(elph.and.nimage>1) call errore('phq_readin',&
+       'el-ph with images not implemented',1)
   
   IF (elph.OR.fildvscf /= ' ') lqdir=.TRUE.
 
@@ -564,8 +688,8 @@ SUBROUTINE phq_readin()
   !
   IF (elph.AND..NOT.lgauss) CALL errore ('phq_readin', 'Electron-&
        &phonon only for metals', 1)
-!  IF (elph.AND.fildvscf.EQ.' ') CALL errore ('phq_readin', 'El-ph needs &
-!       &a DeltaVscf file', 1)
+  IF (elph.AND.fildvscf.EQ.' ') CALL errore ('phq_readin', 'El-ph needs &
+       &a DeltaVscf file', 1)
   !   There might be other variables in the input file which describe
   !   partial computation of the dynamical matrix. Read them here
   !
@@ -574,12 +698,10 @@ SUBROUTINE phq_readin()
   IF ( nat_todo < 0 .OR. nat_todo > nat ) &
      CALL errore ('phq_readin', 'nat_todo is wrong', 1)
   IF (nat_todo.NE.0) THEN
-     IF (ionode) &
-     READ (5, *, iostat = ios) (atomo (na), na = 1, &
-          nat_todo)
-     CALL mp_bcast(ios, ionode_id )
+     IF (meta_ionode) READ (5, *, iostat = ios) (atomo (na), na = 1, nat_todo)
+     CALL mp_bcast(ios, meta_ionode_id, world_comm  )
      CALL errore ('phq_readin', 'reading atoms', ABS (ios) )
-     CALL mp_bcast(atomo, ionode_id )
+     CALL mp_bcast(atomo, meta_ionode_id, world_comm  )
   ENDIF
   nat_todo_input=nat_todo
 
@@ -595,8 +717,11 @@ SUBROUTINE phq_readin()
   IF (modenum > 0 .OR. lraman ) lgamma_gamma=.FALSE.
   IF (.NOT.lgamma_gamma) asr=.FALSE.
   !
-  IF (ldisp .AND. (nq1 .LE. 0 .OR. nq2 .LE. 0 .OR. nq3 .LE. 0)) &
+  IF ((ldisp.AND..NOT.qplot) .AND. &
+                  (nq1 .LE. 0 .OR. nq2 .LE. 0 .OR. nq3 .LE. 0)) &
        CALL errore('phq_readin','nq1, nq2, and nq3 must be greater than 0',1)
+
+  CALL save_ph_input_variables()
   !
   RETURN
   !
