@@ -50,21 +50,21 @@ subroutine phq_setup
   !
   !
   USE kinds,         ONLY : DP
-  USE ions_base,     ONLY : tau, nat, ntyp => nsp, ityp, amass
+  USE ions_base,     ONLY : tau, nat, ntyp => nsp, ityp
   USE cell_base,     ONLY : at, bg
   USE io_global,     ONLY : stdout, ionode
   USE io_files,      ONLY : tmp_dir
   USE ener,          ONLY : ef, ef_up, ef_dw
   USE klist,         ONLY : xk, lgauss, degauss, ngauss, nks, nelec, nelup, &
-                            neldw, two_fermi_energies, wk
-  USE ktetra,        ONLY : ltetra, tetra
+                            neldw, two_fermi_energies, wk, nkstot
+  USE ktetra,        ONLY : ltetra
   USE lsda_mod,      ONLY : nspin, lsda, starting_magnetization, isk
   USE scf,           ONLY : v, vrs, vltot, rho, rho_core, kedtau
   USE fft_base,      ONLY : dfftp
   USE gvect,         ONLY : ngm
   USE gvecs,       ONLY : doublegrid
   USE symm_base,     ONLY : nrot, nsym, s, ftau, irt, t_rev, time_reversal, &
-                            sname, sr, invs, inverse_s, copy_sym
+                            sr, invs, inverse_s
   USE uspp_param,    ONLY : upf
   USE spin_orb,      ONLY : domag
   USE constants,     ONLY : degspin, pi
@@ -77,27 +77,27 @@ subroutine phq_setup
                             trans, epsil, lgamma, recover, where_rec, alpha_pv,&
                             nbnd_occ, flmixdpot, reduce_io, rec_code_read, &
                             done_epsil, zeu, done_zeu, current_iq, u_from_file
-  USE el_phon,       ONLY : elph
+  USE el_phon,       ONLY : elph, comp_elph, done_elph
   USE output,        ONLY : fildrho
   USE modes,         ONLY : u, npertx, npert, gi, gimq, nirr, &
                             t, tmq, irotmq, minus_q, invsymq, &
-                            nsymq, nmodes, rtau, name_rap_mode, num_rap_mode
+                            nsymq, nmodes, rtau, num_rap_mode
   USE dynmat,        ONLY : dyn, dyn_rec, dyn00
   USE efield_mod,    ONLY : epsilon, zstareu
-  USE qpoint,        ONLY : xq
+  USE qpoint,        ONLY : xq, xk_col
   USE partial,       ONLY : comp_irr, atomo, nat_todo, all_comp, &
                             done_irr
   USE gamma_gamma,   ONLY : has_equivalent, asr, nasr, n_diff_sites, &
                             equiv_atoms, n_equiv_atoms, with_symmetry
   USE ph_restart,    ONLY : ph_writefile, ph_readfile
-  USE control_flags, ONLY : iverbosity, modenum, noinv
-  USE disp,          ONLY : comp_irr_iq
+  USE control_flags, ONLY : modenum, noinv
+  USE grid_irr_iq,   ONLY : comp_irr_iq
   USE funct,         ONLY : dmxc, dmxc_spin, dmxc_nc, dft_is_gradient
   USE ramanm,        ONLY : lraman, elop, ramtns, eloptns, done_lraman, &
                             done_elop
 
   USE mp,            ONLY : mp_max, mp_min
-  USE mp_global,     ONLY : inter_pool_comm, nimage
+  USE mp_pools,      ONLY : inter_pool_comm, npool
   !
   USE acfdtest,      ONLY : acfdt_is_active, acfdt_num_der
 
@@ -121,7 +121,8 @@ subroutine phq_setup
   real(DP) :: auxdmuxc(4,4)
   real(DP), allocatable :: wg_up(:,:), wg_dw(:,:)
 
-  logical :: sym (48), magnetic_sym, is_symmorphic
+  logical :: sym (48), magnetic_sym
+  LOGICAL :: symmorphic_or_nzb
   ! the symmetry operations
   integer, allocatable :: ifat(:)
   integer :: ierr
@@ -131,11 +132,14 @@ subroutine phq_setup
   !
   IF (dft_is_gradient().and.(lraman.or.elop)) call errore('phq_setup', &
      'third order derivatives not implemented with GGA', 1)
+
+  IF (nsymq==0) CALL errore('phq_setup', &
+                 'The small group of q is no more calculated in phq_setup',1)
   !
   !  read the displacement patterns
   !
   IF (u_from_file) THEN
-     CALL ph_readfile('data_u',ierr)
+     CALL ph_readfile('data_u',current_iq,0,ierr)
      IF (ierr /= 0) CALL errore('phq_setup', 'problem with modes file',1)
   ENDIF
   !
@@ -320,6 +324,12 @@ subroutine phq_setup
   time_reversal = .NOT. noinv .AND. .NOT. magnetic_sym
 
   nmodes = 3 * nat
+
+  IF (npool > 1) THEN
+     CALL xk_collect( xk_col, xk, nkstot, nks )
+  ELSE
+     xk_col(:,1:nks) = xk(:,1:nks)
+  ENDIF
   !
   !   The small group of q may be known. At a given q it is calculated
   !   by set_nscf, at gamma it coincides with the point group and we
@@ -352,20 +362,15 @@ subroutine phq_setup
   !    if minus_q is true calculate also irotmq and the G associated to Sq=-g+G
   !
   CALL set_giq (xq,s,nsymq,nsym,irotmq,minus_q,gi,gimq)
-  is_symmorphic=.NOT.(ANY(ftau(:,1:nsymq) /= 0))
-  IF (.NOT.is_symmorphic) THEN
-     DO isym=1,nsymq
-        search_sym=( search_sym.and.(abs(gi(1,isym))<1.d-8).and.  &
-                                    (abs(gi(2,isym))<1.d-8).and.  &
-                                    (abs(gi(3,isym))<1.d-8) )
-     END DO
-  ENDIF
+
+  search_sym = search_sym .AND. symmorphic_or_nzb()
+
   num_rap_mode=-1
   IF (search_sym) CALL prepare_sym_analysis(nsymq,sr,t_rev,magnetic_sym)
 
   IF (.NOT.u_from_file) THEN
      CALL find_irrep()
-     CALL ph_writefile('data_u',0)
+     CALL ph_writefile('data_u',current_iq,0,ierr)
   ENDIF
   CALL find_irrep_sym()
 
@@ -375,7 +380,7 @@ subroutine phq_setup
      ALLOCATE(with_symmetry(3*nat))
      ALLOCATE(n_equiv_atoms(nat))
      ALLOCATE(equiv_atoms(nat,nat))
-     CALL find_equiv_sites (nat,nat,nsym,irt,has_equivalent,n_diff_sites, &
+     CALL find_equiv_sites (nat,nsym,irt,has_equivalent,n_diff_sites, &
                        n_equiv_atoms,equiv_atoms)
 
      IF (n_diff_sites .LE. 0 .OR. n_diff_sites .GT. nat)            &
@@ -423,100 +428,49 @@ subroutine phq_setup
   !
   !  9) set the variables needed for the partial computation:
   !     nat_todo, atomo, comp_irr
-  ALLOCATE(ifat(nat))
-  comp_irr = 0
-  comp_irr(0)=1
-  IF (nat_todo==0.AND.modenum==0) THEN
-     !
-     !  Case 1)  The partial computation option is not used, make all
-     !           representation between start_irr and last_irr
-     !
-     IF (start_irr <= last_irr_eff) comp_irr(start_irr: last_irr_eff) = 1
-     !
-  ELSEIF (nat_todo /= 0) THEN
-     !
-     !   Case 2) Sets the atoms which must be computed: the requested
-     !   atoms and all the symmetry related atoms
-     !
-     ifat = 0
-     DO na = 1, nat_todo
-        IF(atomo(na)>nat .or. atomo(na)<1) &
-        CALL errore('phq_setup', 'one of atoms to do (nat_todo) is < 0 or > nat', 1)
-        ifat (atomo (na) ) = 1
-        DO isym = 1, nsymq
-           ifat (irt (isym, atomo (na) ) ) = 1
-        ENDDO
-     ENDDO
-     !
-     !  Find the irreducible representations where the required atoms moves
-     !
-     imode0 = 0
-     do irr = 1, nirr
-        do ipert = 1, npert (irr)
-           mu = imode0 + ipert
-           do na = 1, nat
-              if (ifat (na) == 1 .and. comp_irr (irr) == 0) then
-                 do ipol = 1, 3
-                    nu = 3 * (na - 1) + ipol
-                    if (abs (u (nu, mu) ) > 1.d-6)  comp_irr (irr) = 1
-                 enddo
-              endif
-           enddo
-        enddo
-        imode0 = imode0 + npert (irr)
-     enddo
-  ELSEIF (modenum /= 0) THEN
-     comp_irr(modenum)=1
-  ELSE
-     call errore('phq_setup','nat_todo or nrap wrong',1)
-  ENDIF
+
+  DO irr=0,nirr
+     comp_irr(irr)=comp_irr_iq(irr,current_iq)
+     IF (elph .AND. irr>0) comp_elph(irr)=comp_irr(irr)
+  ENDDO
   !
   !  The gamma_gamma case needs a different treatment
   !
   if (lgamma_gamma) then
      with_symmetry=1
-     comp_irr = 0
-     comp_irr(0)=1
+     comp_irr = .FALSE.
+     comp_irr(0)=.TRUE.
      do na=1,nat
         if (has_equivalent(na)==0) then
             do ipol=1,3
-               comp_irr(3*(na-1)+ipol)=1
+               comp_irr(3*(na-1)+ipol)=.TRUE.
                with_symmetry(3*(na-1)+ipol)=0
             enddo
         endif
      enddo
      if (nasr>0) then
         do ipol=1,3
-           comp_irr(3*(nasr-1)+ipol)=0
+           comp_irr(3*(nasr-1)+ipol)=.FALSE.
            with_symmetry(3*(nasr-1)+ipol)=0
         enddo
      endif
      IF (start_irr <= last_irr_eff) THEN
         DO irr=1,start_irr-1
-           comp_irr(irr) = 0
+           comp_irr(irr) = .FALSE.
         ENDDO
         DO irr=last_irr_eff+1,3*nat
-           comp_irr(irr) = 0
+           comp_irr(irr) = .FALSE.
         ENDDO
      ENDIF
   endif
-!
-!  In this case the number of irreducible representations to compute
-!  has been done elsewhere
-!
-  IF (nimage > 1 ) THEN
-     DO irr=0,nirr
-        comp_irr(irr)=comp_irr_iq(irr,current_iq)
-     ENDDO
-  ENDIF
-
   !
   !  Compute how many atoms moves and set the list atomo
   !
+  ALLOCATE(ifat(nat))
   ifat = 0
   imode0 = 0
   DO irr = 1, nirr
-     if (comp_irr (irr) .eq.1) then
+     if (comp_irr (irr)) then
         do ipert = 1, npert (irr)
            do na = 1, nat
               do ipol = 1, 3
@@ -542,12 +496,13 @@ subroutine phq_setup
   !
   all_comp=.true.
   DO irr=1,nirr
-     IF (comp_irr(irr)==0) all_comp=.false.
+     IF (.NOT.comp_irr(irr)) all_comp=.false.
   ENDDO
   all_comp = all_comp.OR.lgamma_gamma
   all_done = .FALSE.
   npertx = 0
-  done_irr = 0
+  done_irr = .FALSE.
+  IF (elph) done_elph = .FALSE.
   DO irr = 1, nirr
      npertx = max (npertx, npert (irr) )
   ENDDO
@@ -564,7 +519,7 @@ subroutine phq_setup
 
   where_rec='phq_setup.'
   rec_code=-40
-  CALL ph_writefile('data',0)
+  CALL ph_writefile('status_ph',current_iq,0,ierr)
 
   CALL stop_clock ('phq_setup')
   RETURN
