@@ -33,11 +33,12 @@ subroutine phq_summary
   USE run_info, ONLY : title
   USE gamma_gamma,   ONLY : with_symmetry, nasr
   USE control_ph,    ONLY : lgamma_gamma, lnoloc, lrpa, zue, epsil, ldisp, &
-                            nmix_ph, alpha_mix, tr2_ph, zeu
+                            nmix_ph, alpha_mix, tr2_ph, zeu, search_sym
   USE freq_ph,       ONLY : fpol, nfs, fiu
   USE partial,       ONLY : atomo, nat_todo, all_comp, done_irr, comp_irr
   USE modes,         ONLY : u, npert, irotmq, minus_q, nsymq, nirr, &
                             name_rap_mode
+  USE rap_point_group, ONLY : gname
   USE qpoint,        ONLY : xq
   USE ramanm,        ONLY : lraman, elop
   USE control_flags, ONLY : iverbosity
@@ -116,11 +117,11 @@ subroutine phq_summary
   !    description of the atoms inside the unit cell
   !
   WRITE( stdout, '(/, 5x,"Atoms inside the unit cell: ")')
-  WRITE( stdout, '(/,3x,"Cartesian axes")')
+  WRITE( stdout, '(/,5x,"Cartesian axes")')
   WRITE( stdout, '(/,5x,"site n.  atom      mass ", &
        &                "          positions (alat units)")')
 
-  WRITE( stdout, '(7x,i2,5x,a6,f8.4,"   tau(",i2, &
+  WRITE( stdout, '(4x,i5,2x,a6,f8.4,"   tau(",i5, &
        &                              ") = (",3f11.5,"  )")')  &
        &(na, atm (ityp (na) ) , amass (ityp (na) ), na,  &
        &(tau (ipol, na) , ipol = 1, 3) , na = 1, nat)
@@ -240,6 +241,11 @@ subroutine phq_summary
   endif
 
   CALL print_ps_info ( )
+  IF (search_sym.AND.iverbosity==1) THEN
+     CALL write_group_info(.true.)
+  ELSE IF (search_sym) THEN
+     WRITE(stdout,'(/,5x,"Mode symmetry, ",a11," point group:")') gname  
+  ENDIF
 
   IF (lgamma_gamma) &
         WRITE(stdout,'(/5x,"k=gamma and q=gamma tricks are used")')
@@ -272,24 +278,24 @@ subroutine phq_summary
   WRITE( stdout, '(5x,"There are ",i3," irreducible representations")') nirr
   imode0 = 0
   DO irr = 1, nirr
-     IF (done_irr (irr) .eq.1) then
+     IF (done_irr (irr)) then
         WRITE( stdout, '(/, 5x,"Representation ",i5,i7, &
              &                  " modes -",a,"  Done")') irr, npert (irr),&
                       TRIM( name_rap_mode(imode0+1) )
-     ELSEIF (comp_irr (irr) .eq.1) then
+     ELSEIF (comp_irr (irr)) then
         WRITE( stdout, '(/, 5x,"Representation ",i5,i7, &
              &             " modes -",a,"  To be done")') irr, npert (irr), &
                       TRIM( name_rap_mode(imode0+1) )
-     ELSEIF (comp_irr (irr) .eq.0) THEN
+     ELSEIF (.NOT.comp_irr (irr)) THEN
         IF (lgamma_gamma) THEN
            IF ((irr-1)/3+1==nasr) THEN
               WRITE( stdout, '(/, 5x,"Representation ",i5,i7, &
                  &     " modes - Calculated using asr")') irr, npert (irr)
-              done_irr(irr) = 1
+              done_irr(irr) = .TRUE.
            ELSEIF (with_symmetry(irr)==1) THEN
               WRITE( stdout, '(/, 5x,"Representation ",i5,i7, &
                  &     " modes - Calculated using symmetry")') irr, npert (irr)
-              done_irr(irr) = 1
+              done_irr(irr) = .TRUE.
            ELSE
               WRITE( stdout, '(/, 5x,"Representation ",i5,i7, &
                 &     " modes - Not done in this run")') irr, npert (irr)
@@ -300,7 +306,7 @@ subroutine phq_summary
                       TRIM( name_rap_mode(imode0+1) )
         ENDIF
      ENDIF
-     IF (iverbosity == 1) CALL write_modes(irr,imode0)
+     IF (iverbosity == 1) CALL write_modes_out(irr,imode0)
      imode0 = imode0 + npert(irr)
   ENDDO
   if (.not.all_comp) then
