@@ -13,7 +13,7 @@ SUBROUTINE phqscf
   !     This subroutine is the main driver of the self consistent cycle
   !     which gives as output the change of the wavefunctions and the
   !     change of the self-consistent potential due to a phonon of
-  !     a fixed q or to an electric field.
+  !     fixed q.
   !
   USE kinds, ONLY : DP
   USE ions_base, ONLY : nat
@@ -24,7 +24,7 @@ SUBROUTINE phqscf
   USE efield_mod, ONLY : zstarue0, zstarue0_rec
   USE control_ph, ONLY : zue, convt, rec_code
   USE partial,    ONLY : done_irr, comp_irr
-  USE modes,      ONLY : nirr, npert, npertx
+  USE modes,      ONLY : nirr, npert
   USE phus,       ONLY : int3, int3_nc, int3_paw
   USE uspp_param, ONLY : nhm
   USE eqv,        ONLY : drhoscfs
@@ -32,7 +32,8 @@ SUBROUTINE phqscf
   USE noncollin_module, ONLY : noncolin, nspin_mag
   USE recover_mod, ONLY : write_rec
 
-  USE mp_global,  ONLY : inter_pool_comm, intra_pool_comm
+  USE mp_pools,  ONLY : inter_pool_comm
+  USE mp_bands,  ONLY : intra_bgrp_comm
   USE mp,         ONLY : mp_sum
 
   IMPLICIT NONE
@@ -46,9 +47,6 @@ SUBROUTINE phqscf
   REAL(DP) :: tcpu, get_clock
   ! timing variables
 
-  LOGICAL :: exst
-  ! used to test the recover file
-
   EXTERNAL get_clock
   ! the change of density due to perturbations
 
@@ -58,7 +56,7 @@ SUBROUTINE phqscf
   !    of the wavefunctions
   !
   DO irr = 1, nirr
-     IF ( (comp_irr (irr) == 1) .AND. (done_irr (irr) == 0) ) THEN
+     IF ( (comp_irr (irr)) .AND. (.NOT.done_irr (irr)) ) THEN
         npe=npert(irr)
         ALLOCATE (drhoscfs( dfftp%nnr , nspin_mag, npe))
         imode0 = 0
@@ -95,15 +93,15 @@ SUBROUTINE phqscf
            IF (zue) CALL add_zstar_ue (imode0, npe )
            IF (zue.AND. okvan) CALL add_zstar_ue_us(imode0, npe )
            IF (zue) THEN
-#ifdef __MPI
-              call mp_sum ( zstarue0_rec, intra_pool_comm )
+
+              call mp_sum ( zstarue0_rec, intra_bgrp_comm )
               call mp_sum ( zstarue0_rec, inter_pool_comm )
-#endif
+
               zstarue0(:,:)=zstarue0(:,:)+zstarue0_rec(:,:)
            END IF
            !
            WRITE( stdout, '(/,5x,"Convergence has been achieved ")')
-           done_irr (irr) = 1
+           done_irr (irr) = .TRUE.
         ELSE
            WRITE( stdout, '(/,5x,"No convergence has been achieved ")')
            CALL stop_smoothly_ph (.FALSE.)
