@@ -60,6 +60,7 @@ PROGRAM q2r
   !
   USE kinds,      ONLY : DP
   USE mp,         ONLY : mp_bcast
+  USE mp_world,   ONLY : world_comm
   USE mp_global,  ONLY : mp_startup, mp_global_end
   USE dynamicalq, ONLY : phiq, tau, ityp, zeu
   USE fft_scalar, ONLY : cfft3d
@@ -116,13 +117,13 @@ PROGRAM q2r
      !
   IF (ionode)  READ ( 5, input, IOSTAT =ios )
  
-  CALL mp_bcast(ios, ionode_id)
+  CALL mp_bcast(ios, ionode_id, world_comm)
   CALL errore('q2r','error reading input namelist', abs(ios))
 
-  CALL mp_bcast(fildyn, ionode_id)
-  CALL mp_bcast(flfrc, ionode_id)
-  CALL mp_bcast(zasr, ionode_id)
-  CALL mp_bcast(la2f, ionode_id)
+  CALL mp_bcast(fildyn, ionode_id, world_comm)
+  CALL mp_bcast(flfrc, ionode_id, world_comm)
+  CALL mp_bcast(zasr, ionode_id, world_comm)
+  CALL mp_bcast(la2f, ionode_id, world_comm)
      !
      ! check input
      !
@@ -148,11 +149,11 @@ PROGRAM q2r
         CLOSE (unit=1, status='keep')
      END IF
   ENDIF
-  CALL mp_bcast(nr1, ionode_id)
-  CALL mp_bcast(nr2, ionode_id)
-  CALL mp_bcast(nr3, ionode_id)
-  CALL mp_bcast(nfile, ionode_id)
-  CALL mp_bcast(lnogridinfo, ionode_id)
+  CALL mp_bcast(nr1, ionode_id, world_comm)
+  CALL mp_bcast(nr2, ionode_id, world_comm)
+  CALL mp_bcast(nr3, ionode_id, world_comm)
+  CALL mp_bcast(nfile, ionode_id, world_comm)
+  CALL mp_bcast(lnogridinfo, ionode_id, world_comm)
      !
      IF (nr1 < 1 .OR. nr1 > 1024) CALL errore ('q2r',' nr1 wrong or missing',1)
      IF (nr2 < 1 .OR. nr2 > 1024) CALL errore ('q2r',' nr2 wrong or missing',1)
@@ -178,7 +179,7 @@ PROGRAM q2r
      DO ifile=1,nfile
         IF (lnogridinfo) THEN
            IF (ionode) READ(5,'(a)') filin
-           call mp_bcast(filin, ionode_id)
+           call mp_bcast(filin, ionode_id, world_comm)
         ELSE
            filin = TRIM(fildyn) // TRIM( int_to_char( ifile ) )
         END IF
@@ -208,7 +209,7 @@ PROGRAM q2r
         ELSE
            IF (ionode) &
            OPEN (unit=1, file=filin,status='old',form='formatted',iostat=ierr)
-           CALL mp_bcast(ierr, ionode_id)
+           CALL mp_bcast(ierr, ionode_id, world_comm)
            IF (ierr /= 0) CALL errore('q2r','file '//TRIM(filin)//' missing!',1)
            CALL read_dyn_from_file (nqs, q, epsil, lrigid,  &
                 ntyp, nat, ibrav, celldm, at, atm, amass)
@@ -344,7 +345,7 @@ PROGRAM q2r
      ENDIF
      resi = SUM ( ABS (AIMAG ( phid ) ) )
      IF (resi > eps12) THEN
-        WRITE (stdout,"(/5x,' fft-check warning: sum of imaginary terms = ',e12.7)") resi
+        WRITE (stdout,"(/5x,' fft-check warning: sum of imaginary terms = ',es12.6)") resi
      ELSE
         WRITE (stdout,"(/5x,' fft-check success (sum of imaginary terms < 10^-12)')")
      END IF
@@ -370,7 +371,9 @@ SUBROUTINE gammaq2r( nqtot, nat, nr1, nr2, nr3, at )
   USE kinds, ONLY : DP
   USE fft_scalar, ONLY : cfft3d
   USE io_global, ONLY : ionode, ionode_id, stdout
+  USE mp_images, ONLY : intra_image_comm
   USE mp,        ONLY : mp_bcast
+  USE mp_world,  ONLY : world_comm
   !
   IMPLICIT NONE
   INTEGER, INTENT(IN) :: nqtot, nat, nr1, nr2, nr3
@@ -385,7 +388,11 @@ SUBROUTINE gammaq2r( nqtot, nat, nr1, nr2, nr3, at )
   LOGICAL :: lq
   REAL(DP) :: deg, ef, dosscf
   REAL(DP) :: q(3,48), xq, resi
-  character(len=14) :: name
+  character(len=256) :: name
+  CHARACTER(LEN=256) :: elph_dir
+  CHARACTER(LEN=6) :: int_to_char
+  LOGICAL :: exst
+  INTEGER :: ios
 
   !
   ALLOCATE (gaminp(3,3,nat,nat,48), gamout(nr1*nr2*nr3,3,3,nat,nat) )
@@ -397,25 +404,34 @@ SUBROUTINE gammaq2r( nqtot, nat, nr1, nr2, nr3, at )
   nr(1) = nr1
   nr(2) = nr2
   nr(3) = nr3
+  elph_dir='elph_dir/'
+!  IF (ionode) INQUIRE(FILE=TRIM(elph_dir), EXIST=exst)
+!  CALL mp_bcast(exst, ionode_id, intra_image_comm)
+!  IF (.NOT. exst) CALL errore('gammaq2r','elph_dir directory not exists',1)
   !
   DO isig=1, nsig
      filea2F = 50 + isig
-     write(name,"(A7,I2)") 'a2Fq2r.',filea2F
-     IF (ionode) open(filea2F, file=name, STATUS = 'old', FORM = 'formatted')
      nc = 0
+     DO count_q=1,nqtot
+        name= TRIM(elph_dir) // 'a2Fq2r.' // TRIM(int_to_char(filea2F)) &
+                          // '.' // TRIM(int_to_char(count_q))
+        IF (ionode) open(unit=filea2F, file=name, STATUS = 'old', &
+                                 FORM = 'formatted', IOSTAT=ios)
+        CALL mp_bcast(ios, ionode_id, intra_image_comm)
+        IF (ios /= 0) CALL errore('gammaq2r','problem opening file' &
+                                   //TRIM(name), 1)
      !
      ! to pass to matdyn, for each isig, we read: degauss, Fermi energy and DOS
      !
-     DO count_q=1,nqtot
         !
         IF (ionode) THEN
            READ(filea2F,*) deg, ef, dosscf
            READ(filea2F,*) nstar
         ENDIF
-        CALL mp_bcast(deg, ionode_id)
-        CALL mp_bcast(ef, ionode_id)
-        CALL mp_bcast(dosscf, ionode_id)
-        CALL mp_bcast(nstar, ionode_id)
+        CALL mp_bcast(deg, ionode_id, world_comm)
+        CALL mp_bcast(ef, ionode_id, world_comm)
+        CALL mp_bcast(dosscf, ionode_id, world_comm)
+        CALL mp_bcast(nstar, ionode_id, world_comm)
         !
         CALL read_gamma ( nstar, nat, filea2F, q, gaminp )
         !
@@ -432,13 +448,13 @@ SUBROUTINE gammaq2r( nqtot, nat, nr1, nr2, nr3, at )
               m(ipol)= mod(iq,nr(ipol)) + 1
               if (m(ipol) < 1) m(ipol) = m(ipol) + nr(ipol)
            end do !ipol
-           IF (.NOT.lq) CALL errore('init','q not allowed',1)
+           IF (.NOT.lq) CALL errore('gammaq2r','q not allowed',1)
            !
            if(nc(m(1),m(2),m(3)) == 0) then
               nc(m(1),m(2),m(3)) = 1
               CALL TRASL( gamout, gaminp, nq, nr1, nr2, nr3, nat, m(1), m(2), m(3) )
            else
-              call errore('init',' nc already filled: wrong q grid or wrong nr',1)
+              call errore('gammaq2r',' nc already filled: wrong q grid or wrong nr',1)
            end if
         enddo ! stars for given q-point
      ENDDO ! q-points
@@ -449,7 +465,7 @@ SUBROUTINE gammaq2r( nqtot, nat, nr1, nr2, nr3, at )
         write (stdout,'(" Broadening = ",F10.3)') deg
         write (stdout,'(5x,a,i4)') ' q-space grid ok, #points = ',nq_log
      else
-        call errore('init',' missing q-point(s)!',1)
+        call errore('gammaq2r',' missing q-point(s)!',1)
      end if
      do j1=1,3
         do j2=1,3
@@ -466,9 +482,9 @@ SUBROUTINE gammaq2r( nqtot, nat, nr1, nr2, nr3, at )
      IF (ionode) close(filea2F)
      !
      filea2F = 60 + isig
-     write(name,"(A10,I2)") 'a2Fmatdyn.',filea2F
+     name = TRIM(elph_dir) // 'a2Fmatdyn.'// TRIM(int_to_char(filea2F)) 
      IF (ionode) THEN
-     open(filea2F, file=name, STATUS = 'unknown')
+     open(unit=filea2F, file=name, STATUS = 'unknown')
      !
      WRITE(filea2F,*) deg, ef, dosscf
      write(filea2F,'(3i4)') nr1, nr2, nr3
@@ -498,7 +514,7 @@ SUBROUTINE gammaq2r( nqtot, nat, nr1, nr2, nr3, at )
      resi = SUM ( ABS ( AIMAG( gamout ) ) )
 
      IF (resi > eps12) THEN
-        WRITE (stdout,"(/5x,' fft-check warning: sum of imaginary terms = ',e12.7)") resi
+        WRITE (stdout,"(/5x,' fft-check warning: sum of imaginary terms = ',es12.6)") resi
      ELSE
         WRITE (stdout,"(/5x,' fft-check success (sum of imaginary terms < 10^-12)')")
      END IF
@@ -516,6 +532,7 @@ subroutine read_gamma (nqs, nat, ifn, xq, gaminp)
   USE kinds, ONLY : DP
   USE io_global, ONLY : ionode, ionode_id, stdout
   USE mp,        ONLY : mp_bcast
+  USE mp_world,  ONLY : world_comm
   implicit none
   !
   ! I/O variables
@@ -538,18 +555,18 @@ subroutine read_gamma (nqs, nat, ifn, xq, gaminp)
      !     write(*,*) 'xq    ',iq,(xq(i,iq),i=1,3)
         READ(ifn,*)
      END IF
-     CALL mp_bcast(xq(:,iq), ionode_id)
+     CALL mp_bcast(xq(:,iq), ionode_id, world_comm)
      do na=1,nat
         do nb=1,nat
            IF (ionode) read(ifn,*) i,j
-           CALL mp_bcast(i, ionode_id)
-           CALL mp_bcast(j, ionode_id)
+           CALL mp_bcast(i, ionode_id, world_comm)
+           CALL mp_bcast(j, ionode_id, world_comm)
            if (i.ne.na) call errore('read_gamma','wrong na read',na)
            if (j.ne.nb) call errore('read_gamma','wrong nb read',nb)
            do i=1,3
               IF (ionode) read (ifn,*) (phir(j),phii(j),j=1,3)
-              CALL mp_bcast(phir, ionode_id)
-              CALL mp_bcast(phii, ionode_id)
+              CALL mp_bcast(phir, ionode_id, world_comm)
+              CALL mp_bcast(phii, ionode_id, world_comm)
               do j = 1,3
                  gaminp(i,j,na,nb,iq) = CMPLX(phir(j),phii(j),kind=DP)
               end do
