@@ -17,12 +17,6 @@ subroutine set_irr_new (xq, u, npert, nirr, eigen)
 !     give a basis for the irreducible representations of the
 !     small group of q.
 !
-!     Furthermore it computes:
-!     1) the small group of q
-!     2) the possible G vectors associated to every symmetry operation
-!     3) the matrices which represent the small group of q on the
-!        pattern basis.
-!
 !     Original routine was from C. Bungaro.
 !     Revised Oct. 1995 by Andrea Dal Corso.
 !     April 1997: parallel stuff added (SdG)
@@ -41,12 +35,11 @@ subroutine set_irr_new (xq, u, npert, nirr, eigen)
   USE control_flags, ONLY : iverbosity
   USE random_numbers, ONLY : randy
   USE rap_point_group, ONLY : name_rap
-  
-#ifdef __MPI
+
   use mp, only: mp_bcast
   use io_global, only : ionode_id
-  use mp_global, only : intra_image_comm
-#endif
+  use mp_images, only : intra_image_comm
+
   implicit none
 !
 !   first the dummy variables
@@ -110,17 +103,7 @@ subroutine set_irr_new (xq, u, npert, nirr, eigen)
 !
 !     We copy the dynamical matrix in a bidimensional array
 !
-     do na = 1, nat
-        do nb = 1, nat
-           do ipol = 1, 3
-              imode = ipol + 3 * (na - 1)
-              do jpol = 1, 3
-                 jmode = jpol + 3 * (nb - 1)
-                 phi (imode, jmode) = wdyn (ipol, jpol, na, nb)
-              enddo
-           enddo
-        enddo
-     enddo
+     CALL compact_dyn(nat, phi, wdyn)
 !
 !   Diagonalize
 !
@@ -150,7 +133,7 @@ subroutine set_irr_new (xq, u, npert, nirr, eigen)
         npert=1
         do imode=1,3*nat
            WRITE( stdout, '(2x,"autoval = ", e10.4)') eigen(imode)
-           CALL write_modes(imode,imode-1)
+           CALL write_modes_out(imode,imode-1)
         end do
      end if
 
@@ -287,7 +270,6 @@ subroutine set_irr_new (xq, u, npert, nirr, eigen)
 !      nsymq=1
 !      minus_q=.false.
 
-#ifdef __MPI
 !
 ! parallel stuff: first node broadcasts everything to all nodes
 !
@@ -302,6 +284,6 @@ subroutine set_irr_new (xq, u, npert, nirr, eigen)
   call mp_bcast (minus_q, ionode_id, intra_image_comm)
   call mp_bcast (num_rap_mode, ionode_id, intra_image_comm)
   call mp_bcast (name_rap_mode, ionode_id, intra_image_comm)
-#endif
+
   return
 end subroutine set_irr_new
