@@ -23,7 +23,7 @@ subroutine set_irr_sym_new ( t, tmq, npertx )
   USE modes,     ONLY : nsymq, u, irotmq, nirr, npert, rtau, minus_q
   USE control_flags, ONLY : modenum
   USE mp,        ONLY : mp_bcast
-  USE mp_global, ONLY : intra_image_comm
+  USE mp_images, ONLY : intra_image_comm
   USE io_global, ONLY : ionode_id
   implicit none
 !
@@ -161,12 +161,10 @@ subroutine set_irr_sym_new ( t, tmq, npertx )
      enddo
   enddo
 
-#ifdef __MPI
 !
 ! parallel stuff: first node broadcasts everything to all nodes
 !
   call mp_bcast (t, ionode_id, intra_image_comm)
   call mp_bcast (tmq, ionode_id, intra_image_comm)
-#endif
   return
 end subroutine set_irr_sym_new
