@@ -23,6 +23,7 @@ subroutine solve_e2
   USE fft_base,              ONLY : dfftp, dffts
   USE wvfct,                 ONLY : npw, npwx, nbnd, igk, g2kin, et
   USE io_files,  ONLY: prefix, iunigk
+  USE buffers,   ONLY: get_buffer
   USE ions_base, ONLY: nat
   USE uspp,      ONLY: okvan, nkb, vkb
   USE uspp_param,ONLY : nhm
@@ -36,7 +37,8 @@ subroutine solve_e2
   USE recover_mod, ONLY : read_rec, write_rec
 
   USE check_stop, ONLY: check_stop_now
-  USE mp_global,  ONLY : inter_pool_comm, intra_pool_comm
+  USE mp_pools,   ONLY : inter_pool_comm
+  USE mp_bands,   ONLY : intra_bgrp_comm
   USE mp,         ONLY : mp_sum
   implicit none
 
@@ -119,7 +121,7 @@ subroutine solve_e2
         !
         ! reads unperturbed wavefuctions psi_k in G_space, for all bands
         !
-        if (nksq.gt.1) call davcio (evc, lrwfc, iuwfc, ik, -1)
+        if (nksq.gt.1) call get_buffer(evc, lrwfc, iuwfc, ik)
         npwq = npw
         call init_us_2 (npw, igk, xk (1, ik), vkb)
         !
@@ -170,9 +172,7 @@ subroutine solve_e2
                             dbecsum (1, 1), dpsi)
            enddo   ! on perturbations
         enddo      ! on k points
-#ifdef __MPI
-     call mp_sum ( dbecsum, intra_pool_comm )
-#endif
+     call mp_sum ( dbecsum, intra_bgrp_comm )
      if (doublegrid) then
         do is = 1, nspin
            do ipol = 1, 6
@@ -188,18 +188,12 @@ subroutine solve_e2
      !   After the loop over the perturbations we have the change of the pote
      !   for all the modes, and we symmetrize this potential
      !
-#ifdef __MPI
      call mp_sum ( dvscfout, inter_pool_comm )
-#endif
      do ipol = 1, 6
         call dv_of_drho (0, dvscfout (1, 1, ipol), .false.)
      enddo
 
-#ifdef __MPI
      call psyme2(dvscfout)
-#else
-     call syme2(dvscfout)
-#endif
      !
      ! Mixing with the old potential
      !
@@ -219,8 +213,8 @@ subroutine solve_e2
      write (6, "(//,5x,' iter # ',i3, &
           &      '   av.it.: ',f5.1)") iter, avg_iter / (6.d0 * nksq)
      dr2 = dr2 / 6
-     write (6, "(5x,' thresh=',e10.3, ' alpha_mix = ',f6.3, &
-          &      ' |ddv_scf|^2 = ',e10.3 )") thresh, alpha_mix (kter), dr2
+     write (6, "(5x,' thresh=',es10.3, ' alpha_mix = ',f6.3, &
+          &      ' |ddv_scf|^2 = ',es10.3 )") thresh, alpha_mix (kter), dr2
      !
      CALL flush_unit( stdout )
      !
