@@ -23,6 +23,7 @@ subroutine solve_e_fpol ( iw )
   USE ions_base,             ONLY : nat
   USE io_global,             ONLY : stdout, ionode
   USE io_files,              ONLY : prefix, iunigk, diropn
+  USE buffers,               ONLY : get_buffer, save_buffer
   USE check_stop,            ONLY : check_stop_now
   USE wavefunctions_module,  ONLY : evc
   USE cell_base,             ONLY : tpiba2
@@ -44,7 +45,8 @@ subroutine solve_e_fpol ( iw )
   USE qpoint,                ONLY : nksq, npwq, igkq
   USE units_ph,              ONLY : lrdwf, iudwf, lrwfc, iuwfc, iudrho, &
                                     lrdrho
-  USE mp_global,             ONLY : inter_pool_comm, intra_pool_comm
+  USE mp_pools,              ONLY : inter_pool_comm
+  USE mp_bands,              ONLY : intra_bgrp_comm
   USE mp,                    ONLY : mp_sum
 
   implicit none
@@ -160,7 +162,7 @@ subroutine solve_e_fpol ( iw )
         !
         ! reads unperturbed wavefuctions psi_k in G_space, for all bands
         !
-        if (nksq.gt.1) call davcio (evc, lrwfc, iuwfc, ik, - 1)
+        if (nksq.gt.1) call get_buffer(evc, lrwfc, iuwfc, ik)
         npwq = npw
         call init_us_2 (npw, igk, xk (1, ik), vkb)
         !
@@ -207,9 +209,8 @@ subroutine solve_e_fpol ( iw )
            CALL zgemm( 'C', 'N', nbnd_occ (ik), nbnd_occ (ik), npw, &
                 (1.d0,0.d0), evc(1,1), npwx, dvpsi(1,1), npwx, (0.d0,0.d0), &
                 ps(1,1), nbnd )
-#ifdef __MPI
-           call mp_sum ( ps( :, 1:nbnd_occ(ik) ), intra_pool_comm )
-#endif
+
+           call mp_sum ( ps( :, 1:nbnd_occ(ik) ), intra_bgrp_comm )
            ! dpsi is used as work space to store S|evc>
            !
            CALL calbec (npw, vkb, evc, becp, nbnd_occ(ik) )
@@ -237,7 +238,7 @@ subroutine solve_e_fpol ( iw )
               ! starting value for  delta_psi is read from iudwf
               !
               nrec = (ipol - 1) * nksq + ik
-              call davcio (dpsi, lrdwf, iudwf, nrec, - 1)
+              call get_buffer(dpsi, lrdwf, iudwf, nrec)
               !
               ! threshold for iterative solution of the linear system
               !
@@ -275,13 +276,13 @@ subroutine solve_e_fpol ( iw )
            ltaver = ltaver + lter
            lintercall = lintercall + 1
            if (.not.conv_root) WRITE( stdout, "(5x,'kpoint',i4,' ibnd',i4, &
-                &         ' solve_e: root not converged ',e10.3)") ik &
+                &         ' solve_e: root not converged ',es10.3)") ik &
                 &, ibnd, anorm
            !
            ! writes delta_psi on iunit iudwf, k=kpoint,
            !
            nrec = (ipol - 1) * nksq + ik
-           call davcio (dpsi, lrdwf, iudwf, nrec, + 1)
+           call save_buffer (dpsi, lrdwf, iudwf, nrec)
            !
            ! calculates dvscf, sum over k => dvscf_q_ipert
            !
@@ -289,14 +290,12 @@ subroutine solve_e_fpol ( iw )
                             ik, dbecsum(1,1,current_spin,ipol), dpsi)
         enddo   ! on polarizations
      enddo      ! on k points
-#ifdef __MPI
      !
      !  The calculation of dbecsum is distributed across processors
      !  (see addusdbec) - we sum over processors the contributions
      !  coming from each slice of bands
      !
-     call mp_sum ( dbecsum, intra_pool_comm )
-#endif
+     call mp_sum ( dbecsum, intra_bgrp_comm )
 
      if (doublegrid) then
         do is=1,nspin
@@ -311,12 +310,8 @@ subroutine solve_e_fpol ( iw )
      !   dvscfout contains the (unsymmetrized) linear charge response
      !   for the three polarizations - symmetrize it
      !
-#ifdef __MPI
      call mp_sum ( dvscfout, inter_pool_comm )
      call psyme (dvscfout)
-#else
-     call syme (dvscfout)
-#endif
      !
      !   save the symmetrized linear charge response to file
      !   calculate the corresponding linear potential response
@@ -347,8 +342,8 @@ subroutine solve_e_fpol ( iw )
      WRITE( stdout, '(/,5x," iter # ",i3," total cpu time :",f8.1, &
           &      " secs   av.it.: ",f5.1)') iter, tcpu, averlt
      dr2 = dr2 / 3
-     WRITE( stdout, "(5x,' thresh=',e10.3, ' alpha_mix = ',f6.3, &
-          &      ' |ddv_scf|^2 = ',e10.3 )") thresh, alpha_mix (kter), dr2
+     WRITE( stdout, "(5x,' thresh=',es10.3, ' alpha_mix = ',f6.3, &
+          &      ' |ddv_scf|^2 = ',es10.3 )") thresh, alpha_mix (kter), dr2
      !
      CALL flush_unit( stdout )
      !
