@@ -29,11 +29,10 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   USE wavefunctions_module, ONLY : evc
   USE constants,            ONLY : degspin
   USE cell_base,            ONLY : at, tpiba2
-  USE ener,                 ONLY : ef
   USE klist,                ONLY : lgauss, degauss, ngauss, xk, wk
   USE gvect,                ONLY : g
   USE gvecs,                ONLY : doublegrid
-  USE fft_base,             ONLY : dfftp, dffts
+  USE fft_base,             ONLY : dfftp, dffts, tg_cgather
   USE lsda_mod,             ONLY : lsda, nspin, current_spin, isk
   USE spin_orb,             ONLY : domag
   USE wvfct,                ONLY : nbnd, npw, npwx, igk,g2kin,  et
@@ -44,29 +43,31 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   USE paw_variables,        ONLY : okpaw
   USE paw_onecenter,        ONLY : paw_dpotential
   USE paw_symmetry,         ONLY : paw_dusymmetrize, paw_dumqsymmetrize
+  USE buffers,              ONLY : save_buffer, get_buffer
   USE control_ph,           ONLY : rec_code, niter_ph, nmix_ph, tr2_ph, &
                                    alpha_pv, lgamma, lgamma_gamma, convt, &
-                                   nbnd_occ, alpha_mix, ldisp, rec_code_read, &
+                                   nbnd_occ, alpha_mix, rec_code_read, &
                                    where_rec, flmixdpot, ext_recover
   USE el_phon,              ONLY : elph
   USE nlcc_ph,              ONLY : nlcc_any
   USE units_ph,             ONLY : iudrho, lrdrho, iudwf, lrdwf, iubar, lrbar, &
-                                   iuwfc, lrwfc, iunrec, iudvscf, &
-                                   this_pcxpsi_is_on_file
+                                   iuwfc, lrwfc, iudvscf, iuint3paw, lint3paw
   USE output,               ONLY : fildrho, fildvscf
   USE phus,                 ONLY : int3_paw, becsumort
   USE eqv,                  ONLY : dvpsi, dpsi, evq, eprec
   USE qpoint,               ONLY : xq, npwq, igkq, nksq, ikks, ikqs
   USE modes,                ONLY : npertx, npert, u, t, irotmq, tmq, &
                                    minus_q, nsymq, rtau
+
   USE recover_mod,          ONLY : read_rec, write_rec
   ! used to write fildrho:
   USE dfile_autoname,       ONLY : dfile_name
   USE save_ph,              ONLY : tmp_dir_save
   ! used oly to write the restart file
-  USE mp_global,            ONLY : inter_pool_comm, intra_pool_comm
+  USE mp_pools,             ONLY : inter_pool_comm
+  USE mp_bands,             ONLY : intra_bgrp_comm, ntask_groups, me_bgrp
   USE mp,                   ONLY : mp_sum
-  !
+  USE efermi_shift,         ONLY : ef_shift, ef_shift_paw,  def
   implicit none
 
   integer :: irr, npe, imode0
@@ -87,8 +88,6 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   real(DP) :: dos_ef, weight, aux_avg (2)
   ! Misc variables for metals
   ! dos_ef: density of states at Ef
-  real(DP), external :: w0gauss, wgauss
-  ! functions computing the delta and theta function
 
   complex(DP), allocatable, target :: dvscfin(:,:,:)
   ! change of the scf potential
@@ -98,7 +97,8 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   ! change of rho / scf potential (output)
   ! change of scf potential (output)
   complex(DP), allocatable :: ldos (:,:), ldoss (:,:), mixin(:), mixout(:), &
-       dbecsum (:,:,:,:), dbecsum_nc(:,:,:,:,:), aux1 (:,:)
+       dbecsum (:,:,:,:), dbecsum_nc(:,:,:,:,:), aux1 (:,:), tg_dv(:,:), &
+       tg_psic(:,:), aux2(:,:)
   ! Misc work space
   ! ldos : local density of states af Ef
   ! ldoss: as above, without augmentation charges
@@ -122,10 +122,11 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
              ig,         & ! counter on G vectors
              ndim,       &
              is,         & ! counter on spin polarizations
-             nt,         & ! counter on types
-             na,         & ! counter on atoms
              nrec,       & ! the record number for dvpsi and dpsi
              ios,        & ! integer variable for I/O control
+             incr,       & ! used for tg
+             v_siz,      & ! size of the potential
+             ipol,       & ! counter on polarization
              mode          ! mode index
 
   integer  :: iq_dummy
@@ -137,6 +138,11 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   IF (rec_code_read > 20 ) RETURN
 
   call start_clock ('solve_linter')
+!
+!  This routine is task group aware
+!
+  IF ( ntask_groups > 1 ) dffts%have_task_groups=.TRUE.
+
   allocate (dvscfin ( dfftp%nnr , nspin_mag , npe))
   if (doublegrid) then
      allocate (dvscfins (dffts%nnr , nspin_mag , npe))
@@ -154,13 +160,27 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   IF (noncolin) allocate (dbecsum_nc (nhm,nhm, nat , nspin , npe))
   allocate (aux1 ( dffts%nnr, npol))
   allocate (h_diag ( npwx*npol, nbnd))
+  allocate (aux2(npwx*npol, nbnd))
+  incr=1
+  IF ( dffts%have_task_groups ) THEN
+     !
+     v_siz =  dffts%tg_nnr * dffts%nogrp
+     ALLOCATE( tg_dv   ( v_siz, nspin_mag ) )
+     ALLOCATE( tg_psic( v_siz, npol ) )
+     incr = dffts%nogrp
+     !
+  ENDIF
   !
   if (rec_code_read == 10.AND.ext_recover) then
      ! restart from Phonon calculation
      IF (okpaw) THEN
         CALL read_rec(dr2, iter0, npe, dvscfin, dvscfins, drhoscfh, dbecsum)
-        CALL setmixout(npe*dfftp%nnr*nspin_mag,(nhm*(nhm+1)*nat*nspin_mag*npe)/2, &
-                    mixin, dvscfin, dbecsum, ndim, -1 )
+        IF (convt) THEN
+           CALL PAW_dpotential(dbecsum,rho%bec,int3_paw,npe)
+        ELSE
+           CALL setmixout(npe*dfftp%nnr*nspin_mag,&
+           (nhm*(nhm+1)*nat*nspin_mag*npe)/2,mixin,dvscfin,dbecsum,ndim,-1)
+        ENDIF
      ELSE
         CALL read_rec(dr2, iter0, npe, dvscfin, dvscfins, drhoscfh)
      ENDIF
@@ -179,7 +199,6 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   END IF
 
   IF (convt) GOTO 155
-
   !
   ! if q=0 for a metal: allocate and compute local DOS at Ef
   !
@@ -188,12 +207,9 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   if (lmetq0) then
      allocate ( ldos ( dfftp%nnr  , nspin_mag) )
      allocate ( ldoss( dffts%nnr , nspin_mag) )
-     IF (okpaw) THEN
-        allocate (becsum1 ( (nhm * (nhm + 1))/2 , nat , nspin_mag))
-        call localdos_paw ( ldos , ldoss , becsum1, dos_ef )
-     ELSE
-        call localdos ( ldos , ldoss , dos_ef )
-     ENDIF
+     allocate (becsum1 ( (nhm * (nhm + 1))/2 , nat , nspin_mag))
+     call localdos_paw ( ldos , ldoss , becsum1, dos_ef )
+     IF (.NOT.okpaw) deallocate(becsum1)
   endif
   !
   !
@@ -236,10 +252,10 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
         !
         if (nksq.gt.1) then
            if (lgamma) then
-              call davcio (evc, lrwfc, iuwfc, ikk, - 1)
+              call get_buffer (evc, lrwfc, iuwfc, ikk)
            else
-              call davcio (evc, lrwfc, iuwfc, ikk, - 1)
-              call davcio (evq, lrwfc, iuwfc, ikq, - 1)
+              call get_buffer (evc, lrwfc, iuwfc, ikk)
+              call get_buffer (evq, lrwfc, iuwfc, ikq)
            endif
 
         endif
@@ -276,17 +292,44 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
               !
               ! After the first iteration dvbare_q*psi_kpoint is read from file
               !
-              call davcio (dvpsi, lrbar, iubar, nrec, - 1)
+              call get_buffer (dvpsi, lrbar, iubar, nrec)
               !
               ! calculates dvscf_q*psi_k in G_space, for all bands, k=kpoint
               ! dvscf_q from previous iteration (mix_potential)
               !
               call start_clock ('vpsifft')
-              do ibnd = 1, nbnd_occ (ikk)
-                 call cft_wave (evc (1, ibnd), aux1, +1)
-                 call apply_dpot(dffts%nnr,aux1, dvscfins(1,1,ipert), current_spin)
-                 call cft_wave (dvpsi (1, ibnd), aux1, -1)
+              IF ( ntask_groups > 1 ) dffts%have_task_groups=.TRUE.
+              IF( dffts%have_task_groups ) THEN
+                 IF (noncolin) THEN
+                    CALL tg_cgather( dffts, dvscfins(:,1,ipert), &
+                                                                tg_dv(:,1))
+                    IF (domag) THEN
+                       DO ipol=2,4
+                          CALL tg_cgather( dffts, dvscfins(:,ipol,ipert), &
+                                                             tg_dv(:,ipol))
+                       ENDDO
+                    ENDIF
+                 ELSE
+                    CALL tg_cgather( dffts, dvscfins(:,current_spin,ipert), &
+                                                             tg_dv(:,1))
+                 ENDIF
+              ENDIF
+              aux2=(0.0_DP,0.0_DP)
+              do ibnd = 1, nbnd_occ (ikk), incr
+                 IF( dffts%have_task_groups ) THEN
+                    call cft_wave_tg (evc, tg_psic, 1, v_siz, ibnd, &
+                                      nbnd_occ (ikk) )
+                    call apply_dpot(v_siz, tg_psic, tg_dv, 1)
+                    call cft_wave_tg (aux2, tg_psic, -1, v_siz, ibnd, &
+                                      nbnd_occ (ikk))
+                 ELSE
+                    call cft_wave (evc (1, ibnd), aux1, +1)
+                    call apply_dpot(dffts%nnr,aux1, dvscfins(1,1,ipert), current_spin)
+                    call cft_wave (aux2 (1, ibnd), aux1, -1)
+                 ENDIF
               enddo
+              dvpsi=dvpsi+aux2
+              dffts%have_task_groups=.FALSE.
               call stop_clock ('vpsifft')
               !
               !  In the case of US pseudopotentials there is an additional
@@ -300,7 +343,7 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
               !  and written to file
               !
               call dvqpsi_us (ik, u (1, mode),.false. )
-              call davcio (dvpsi, lrbar, iubar, nrec, 1)
+              call save_buffer (dvpsi, lrbar, iubar, nrec)
            endif
            !
            ! Ortogonalize dvpsi to valence states: ps = <evq|dvpsi>
@@ -312,7 +355,7 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
               !
               ! starting value for delta_psi is read from iudwf
               !
-              call davcio ( dpsi, lrdwf, iudwf, nrec, -1)
+              call get_buffer( dpsi, lrdwf, iudwf, nrec)
               !
               ! threshold for iterative solution of the linear system
               !
@@ -342,13 +385,13 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
            ltaver = ltaver + lter
            lintercall = lintercall + 1
            if (.not.conv_root) WRITE( stdout, '(5x,"kpoint",i4," ibnd",i4,  &
-                &              " solve_linter: root not converged ",e10.3)') &
+                &              " solve_linter: root not converged ",es10.3)') &
                 &              ik , ibnd, anorm
            !
            ! writes delta_psi on iunit iudwf, k=kpoint,
            !
            !               if (nksq.gt.1 .or. npert(irr).gt.1)
-           call davcio (dpsi, lrdwf, iudwf, nrec, + 1)
+           call save_buffer (dpsi, lrdwf, iudwf, nrec)
            !
            ! calculates dvscf, sum over k => dvscf_q_ipert
            !
@@ -364,17 +407,15 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
         enddo
         ! on k-points
      enddo
-#ifdef __MPI
      !
      !  The calculation of dbecsum is distributed across processors (see addusdbec)
      !  Sum over processors the contributions coming from each slice of bands
      !
      IF (noncolin) THEN
-        call mp_sum ( dbecsum_nc, intra_pool_comm )
+        call mp_sum ( dbecsum_nc, intra_bgrp_comm )
      ELSE
-        call mp_sum ( dbecsum, intra_pool_comm )
+        call mp_sum ( dbecsum, intra_bgrp_comm )
      ENDIF
-#endif
 
      if (doublegrid) then
         do is = 1, nspin_mag
@@ -393,14 +434,12 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      !    Now we compute for all perturbations the total charge and potential
      !
      call addusddens (drhoscfh, dbecsum, imode0, npe, 0)
-#ifdef __MPI
      !
      !   Reduce the delta rho across pools
      !
      call mp_sum ( drhoscf, inter_pool_comm )
      call mp_sum ( drhoscfh, inter_pool_comm )
      IF (okpaw) call mp_sum ( dbecsum, inter_pool_comm )
-#endif
 
      !
      ! q=0 in metallic case deserve special care (e_Fermi can shift)
@@ -423,14 +462,9 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      !   Here we symmetrize them ...
      !
      IF (.not.lgamma_gamma) THEN
-#ifdef __MPI
         call psymdvscf (npe, irr, drhoscfh)
         IF ( noncolin.and.domag ) &
            CALL psym_dmag( npe, irr, drhoscfh)
-#else
-        call symdvscf (npe, irr, drhoscfh)
-        IF ( noncolin.and.domag ) CALL sym_dmag( npe, irr, drhoscfh)
-#endif
         IF (okpaw) THEN
            IF (minus_q) CALL PAW_dumqsymmetrize(dbecsum,npe,irr, &
                              npertx,irotmq,rtau,xq,tmq)
@@ -509,8 +543,8 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      WRITE( stdout, '(/,5x," iter # ",i3," total cpu time :",f8.1, &
           &      " secs   av.it.: ",f5.1)') iter, tcpu, averlt
      dr2 = dr2 / npe
-     WRITE( stdout, '(5x," thresh=",e10.3, " alpha_mix = ",f6.3, &
-          &      " |ddv_scf|^2 = ",e10.3 )') thresh, alpha_mix (kter) , dr2
+     WRITE( stdout, '(5x," thresh=",es10.3, " alpha_mix = ",f6.3, &
+          &      " |ddv_scf|^2 = ",es10.3 )') thresh, alpha_mix (kter) , dr2
      !
      !    Here we save the information for recovering the run from this poin
      !
@@ -539,10 +573,16 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      call drhodvus (irr, imode0, dvscfin, npe)
      if (fildvscf.ne.' ') then
         do ipert = 1, npe
+           if(lmetq0) then
+                dvscfin(:,:,ipert) = dvscfin(:,:,ipert)-def(ipert)
+                if (doublegrid) dvscfins(:,:,ipert) = dvscfins(:,:,ipert)-def(ipert)
+           endif
            call davcio_drho ( dvscfin(1,1,ipert),  lrdrho, iudvscf, &
                          imode0 + ipert, +1 )
+           IF (okpaw.AND.me_bgrp==0) CALL davcio( int3_paw(:,:,ipert,:,:), lint3paw, &
+                                                  iuint3paw, imode0+ipert, + 1 )
         end do
-        if (elph) call elphel (npe, imode0, dvscfins)
+        if (elph) call elphel (irr, npe, imode0, dvscfins)
      end if
   endif
   if (convt.and.nlcc_any) call addnlcc (imode0, drhoscfh, npe)
@@ -552,7 +592,7 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   deallocate (aux1)
   deallocate (dbecsum)
   IF (okpaw) THEN
-     if (lmetq0.and.allocated(becsum1)) deallocate (becsum1)
+     if (allocated(becsum1)) deallocate (becsum1)
      deallocate (mixin)
      deallocate (mixout)
   ENDIF
@@ -561,6 +601,13 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   deallocate (drhoscfh)
   if (doublegrid) deallocate (dvscfins)
   deallocate (dvscfin)
+  deallocate(aux2)
+  IF ( ntask_groups > 1) dffts%have_task_groups=.TRUE.
+  IF ( dffts%have_task_groups ) THEN
+     DEALLOCATE( tg_dv )
+     DEALLOCATE( tg_psic )
+  ENDIF
+  dffts%have_task_groups=.FALSE.
 
   call stop_clock ('solve_linter')
 END SUBROUTINE solve_linter
@@ -568,13 +615,13 @@ END SUBROUTINE solve_linter
 
 SUBROUTINE setmixout(in1, in2, mix, dvscfout, dbecsum, ndim, flag )
 USE kinds, ONLY : DP
-USE mp_global, ONLY : intra_pool_comm
+USE mp_bands, ONLY : intra_bgrp_comm
 USE mp, ONLY : mp_sum
 IMPLICIT NONE
 INTEGER :: in1, in2, flag, ndim, startb, lastb
 COMPLEX(DP) :: mix(in1+in2), dvscfout(in1), dbecsum(in2)
 
-CALL divide (intra_pool_comm, in2, startb, lastb)
+CALL divide (intra_bgrp_comm, in2, startb, lastb)
 ndim=lastb-startb+1
 
 IF (flag==-1) THEN
@@ -584,15 +631,13 @@ ELSE
    dvscfout(1:in1)=mix(1:in1)
    dbecsum=(0.0_DP,0.0_DP)
    dbecsum(startb:lastb)=mix(in1+1:in1+ndim)
-#ifdef __MPI
-   CALL mp_sum(dbecsum, intra_pool_comm)
-#endif
+   CALL mp_sum(dbecsum, intra_bgrp_comm)
 ENDIF
 END SUBROUTINE setmixout
 
 SUBROUTINE check_all_convt(convt)
   USE mp,        ONLY : mp_sum
-  USE mp_global, ONLY : nproc_image, me_image, intra_image_comm
+  USE mp_images, ONLY : nproc_image, me_image, intra_image_comm
   IMPLICIT NONE
   LOGICAL,INTENT(in) :: convt
   INTEGER,ALLOCATABLE :: convt_check(:)
@@ -606,7 +651,7 @@ SUBROUTINE check_all_convt(convt)
   !
   CALL mp_sum(convt_check, intra_image_comm)
   !CALL mp_sum(ios, inter_pool_comm)
-  !CALL mp_sum(ios, intra_pool_com)
+  !CALL mp_sum(ios, intra_bgrp_comm)
   !
 !  convt = ALL(convt_check==0)
   IF(ANY(convt_check==0).and..not.ALL(convt_check==0) ) THEN
