@@ -17,6 +17,7 @@ subroutine zstar_eu
   USE cell_base, ONLY : bg
   USE ions_base, ONLY : nat, zv, ityp
   USE io_files,  ONLY : iunigk
+  USE buffers,   ONLY : get_buffer
   USE klist,     ONLY : wk, xk
   USE symme,     ONLY : symtensor
   USE wvfct,     ONLY : npw, npwx, igk
@@ -30,14 +31,16 @@ subroutine zstar_eu
   USE efield_mod,   ONLY : zstareu0, zstareu
   USE units_ph,  ONLY : iudwf, lrdwf, iuwfc, lrwfc
   USE control_ph,ONLY : nbnd_occ, done_zeu
+  USE ph_restart, ONLY : ph_writefile
 
-  USE mp_global,             ONLY : inter_pool_comm, intra_pool_comm
+  USE mp_pools,              ONLY : inter_pool_comm
+  USE mp_bands,              ONLY : intra_bgrp_comm
   USE mp,                    ONLY : mp_sum
 
   implicit none
 
   integer :: ibnd, ipol, jpol, icart, na, nu, mu, imode0, irr, &
-       imode, nrec, mode, ik
+       imode, nrec, mode, ik, ierr
   ! counters
   real(DP) :: weight
   complex(DP), external :: zdotc
@@ -53,7 +56,7 @@ subroutine zstar_eu
      if (nksq > 1) read (iunigk) npw, igk
      npwq = npw
      weight = wk (ik)
-     if (nksq > 1) call davcio (evc, lrwfc, iuwfc, ik, - 1)
+     if (nksq > 1) call get_buffer (evc, lrwfc, iuwfc, ik)
      call init_us_2 (npw, igk, xk (1, ik), vkb)
      imode0 = 0
      do irr = 1, nirr
@@ -69,7 +72,7 @@ subroutine zstar_eu
               !
               ! read dpsi(scf)/dE for electric field in jpol direction
               !
-              call davcio (dpsi, lrdwf, iudwf, nrec, - 1)
+              call get_buffer (dpsi, lrdwf, iudwf, nrec)
               do ibnd = 1, nbnd_occ(ik)
                  zstareu0(jpol,mode)=zstareu0(jpol, mode)-2.d0*weight*&
                       zdotc(npwx*npol,dpsi(1,ibnd),1,dvpsi(1,ibnd),1)
@@ -84,10 +87,8 @@ subroutine zstar_eu
   !
   if (okvan) call zstar_eu_us
 
-#ifdef __MPI
-  call mp_sum ( zstareu0, intra_pool_comm )
+  call mp_sum ( zstareu0, intra_bgrp_comm )
   call mp_sum ( zstareu0, inter_pool_comm )
-#endif
   !
   ! bring the mode index to cartesian coordinates
   ! NOTA BENE: the electric field is in crystal axis
@@ -119,6 +120,7 @@ subroutine zstar_eu
 
   done_zeu=.TRUE.
   call summarize_zeu()
+  CALL ph_writefile('tensors',0,0,ierr)
 
   call stop_clock ('zstar_eu')
   return
