@@ -15,9 +15,11 @@ subroutine zstar_eu_us
   !
   USE kinds,     ONLY : DP
   USE mp,        ONLY : mp_sum
-  USE mp_global, ONLY : inter_pool_comm, intra_pool_comm
+  USE mp_pools,  ONLY : inter_pool_comm
+  USE mp_bands,  ONLY : intra_bgrp_comm
   USE cell_base, ONLY : omega
   USE ions_base, ONLY : nat, ntyp => nsp, ityp
+  USE buffers,   ONLY : get_buffer
   USE klist,     ONLY : xk, wk
   USE gvecs,   ONLY : doublegrid
   USE fft_base, ONLY : dfftp, dffts
@@ -38,7 +40,7 @@ subroutine zstar_eu_us
   USE nlcc_ph,    ONLY : nlcc_any
   USE units_ph,   ONLY : lrdwf, iucom, lrcom, lrebar, iuebar, lrdrhous, &
                          iudrhous, iudwf, lrwfc, iuwfc
-  USE mp_global, ONLY : nproc_pool
+  USE mp_pools, ONLY : nproc_pool, npool
 
   !
   implicit none
@@ -66,8 +68,8 @@ subroutine zstar_eu_us
 #endif
 
   !  auxiliary space for <psi|ds/du|psi>
-  allocate (dvscf( dfftp%nnr , nspin, 3))
-  allocate (dbecsum( nhm*(nhm+1)/2, nat, nspin, 3))
+  allocate (dvscf( dfftp%nnr , nspin_mag, 3))
+  allocate (dbecsum( nhm*(nhm+1)/2, nat, nspin_mag, 3))
   if (noncolin) allocate (dbecsum_nc( nhm, nhm, nat, nspin, 3))
   allocate (aux1(  dffts%nnr))
   allocate (pdsp(nbnd,nbnd))
@@ -88,13 +90,13 @@ subroutine zstar_eu_us
   do ik = 1, nksq
      if (nksq.gt.1) read (iunigk) npw, igk
      npwq = npw
-     if (nksq.gt.1) call davcio (evc, lrwfc, iuwfc, ik, - 1)
+     if (nksq.gt.1) call get_buffer (evc, lrwfc, iuwfc, ik)
      if (lsda) current_spin = isk (ik)
      call init_us_2 (npw, igk, xk(1,ik), vkb)
      weight = wk (ik)
      do jpol = 1, 3
         nrec = (jpol - 1) * nksq + ik
-        call davcio (dpsi, lrdwf, iudwf, nrec, - 1)
+        call get_buffer(dpsi, lrdwf, iudwf, nrec)
         if (noncolin) then
            call incdrhoscf_nc (dvscf(1,1,jpol),weight,ik, &
                               dbecsum_nc(1,1,1,1,jpol), dpsi)
@@ -105,20 +107,18 @@ subroutine zstar_eu_us
      end do
   end do
 
-#ifdef __MPI
      IF (noncolin) THEN
-        call mp_sum ( dbecsum_nc, intra_pool_comm )
+        call mp_sum ( dbecsum_nc, intra_bgrp_comm )
      ELSE
-        call mp_sum ( dbecsum, intra_pool_comm )
+        call mp_sum ( dbecsum, intra_bgrp_comm )
      END IF
-#endif
 #ifdef TIMINIG_ZSTAR_US
   call stop_clock('zstar_us_1')
   call start_clock('zstar_us_2')
 #endif
 
   if (doublegrid) then
-     do is = 1, nspin
+     do is = 1, nspin_mag
         do ipol = 1, 3
            call cinterpolate(dvscf(1,is,ipol),dvscf(1,is,ipol), 1)
         end do
@@ -129,9 +129,7 @@ subroutine zstar_eu_us
 
   call addusddense (dvscf, dbecsum)
 
-#ifdef __MPI
   call mp_sum ( dvscf, inter_pool_comm )
-#endif
 
 #ifdef TIMINIG_ZSTAR_US
   call stop_clock('zstar_us_2')
@@ -150,11 +148,7 @@ subroutine zstar_eu_us
      !
      call dv_of_drho (0, dvscf (1, 1, ipol), .false.)
   enddo
-#ifdef __MPI
   call psyme (dvscf)
-#else
-  call syme (dvscf)
-#endif
 
 #ifdef TIMINIG_ZSTAR_US
   call stop_clock('zstar_us_3')
@@ -165,12 +159,12 @@ subroutine zstar_eu_us
 ! potenial
 !
   imode0 = 0
-  allocate(drhoscfh(dfftp%nnr,nspin))
+  allocate(drhoscfh(dfftp%nnr,nspin_mag))
   do irr = 1, nirr
      npe = npert(irr)
      do imode = 1, npe
         mode = imode0 + imode
-        call davcio (drhoscfh, lrdrhous, iudrhous, mode, -1)
+        call get_buffer(drhoscfh, lrdrhous, iudrhous, mode)
         do jpol = 1, 3
            do is=1,nspin_mag
               zstareu0(jpol,mode) =  zstareu0(jpol,mode) -                  &
@@ -195,7 +189,7 @@ subroutine zstar_eu_us
      if (nksq.gt.1) read (iunigk) npw, igk
      npwq = npw
      weight = wk (ik)
-     if (nksq.gt.1) call davcio (evc, lrwfc, iuwfc, ik, - 1)
+     if (nksq.gt.1) call get_buffer (evc, lrwfc, iuwfc, ik)
      call init_us_2 (npw, igk, xk (1, ik), vkb)
      call dvkb3(ik, dvkb)
      imode0 = 0
@@ -208,12 +202,12 @@ subroutine zstar_eu_us
               ! read the Commutator+add. terms
               !
               nrec = (jpol - 1) * nksq + ik
-              call davcio (dvpsi, lrebar, iuebar, nrec, - 1)
+              call get_buffer(dvpsi, lrebar, iuebar, nrec)
               !
               pdsp = (0.d0,0.d0)
               call psidspsi (ik, u (1, mode), pdsp )
 #ifdef __MPI
-              call mp_sum( pdsp, intra_pool_comm )
+              call mp_sum( pdsp, intra_bgrp_comm )
 #endif
               !
               ! add the term of the double summation
@@ -236,7 +230,7 @@ subroutine zstar_eu_us
               ! first we read  P_c [H-eS]|psi> and store it in dpsi
               !
               nrec = (jpol - 1) * nksq + ik
-              call davcio (dpsi, lrcom, iucom, nrec, -1)
+              call get_buffer(dpsi, lrcom, iucom, nrec)
               !
               ! Apply the matrix dS/du
               !
@@ -267,7 +261,7 @@ subroutine zstar_eu_us
 
   fact=1.0_DP
 #ifdef __MPI
-  fact=1.0_DP/nproc_pool
+  fact=1.0_DP/nproc_pool/npool
 #endif
   IF (okpaw) THEN
      imode0 = 0
